@@ -27,6 +27,15 @@
  ******************************************************************************
  *
  * $Log: cgiutil.c,v $
+ * Revision 1.22.2.2  2006/02/03 18:57:51  sdlime
+ * Argh! I mis-applied the patch for 1628 and didn't delete one line.
+ *
+ * Revision 1.22.2.1  2006/02/02 00:32:25  sdlime
+ * Fixed bug with default content-type and POST requests. (bug 1628)
+ *
+ * Revision 1.22  2005/07/22 17:26:11  frank
+ * bug 1259: fixed POST support in fastcgi mode
+ *
  * Revision 1.21  2005/06/14 16:03:33  dan
  * Updated copyright date to 2005
  *
@@ -45,7 +54,7 @@
 #include "cgiutil.h"
 #include "map.h"
 
-MS_CVSID("$Id: cgiutil.c,v 1.21 2005/06/14 16:03:33 dan Exp $")
+MS_CVSID("$Id: cgiutil.c,v 1.22.2.2 2006/02/03 18:57:51 sdlime Exp $")
 
 #define LF 10
 #define CR 13
@@ -71,7 +80,7 @@ static char *readPostBody( cgiRequestObj *request )
             exit( 1 );
         }
 
-        if ( (int) fread(data, 1, data_max, stdin) < data_max ) {
+        if ( (int) msIO_fread(data, 1, data_max, stdin) < data_max ) {
             msIO_printf("Content-type: text/html%c%c",10,10);
             msIO_printf("POST body is short\n");
             exit(1);
@@ -87,12 +96,9 @@ static char *readPostBody( cgiRequestObj *request )
     data_len = 0;
     data = (char *) malloc(data_max+1);
 
-    while( !feof(stdin) )
+    while( (chunk_size = msIO_fread( data + data_len, 1, 
+                                     data_max-data_len, stdin )) > 0 )
     {
-        chunk_size = fread( data + data_len, 1, data_max - data_len, stdin );
-        if( chunk_size <= 0 )
-            break;
-
         data_len += chunk_size;
 
         if( data_len == data_max )
@@ -109,9 +115,9 @@ static char *readPostBody( cgiRequestObj *request )
         }
     }
 
-   data[data_len] = '\0';
-
-   return data;
+    data[data_len] = '\0';
+    
+    return data;
 }
 
 
@@ -129,7 +135,13 @@ int loadParams(cgiRequestObj *request){
     char *post_data;
 
     request->type = MS_POST_REQUEST;
-    request->contenttype = strdup(getenv("CONTENT_TYPE"));
+
+    if(s = getenv("CONTENT_TYPE"))
+      request->contenttype = strdup(s);
+    /* we've to set default content-type which is
+     * application/octet-stream according to
+     * W3 RFC 2626 section 7.2.1 */
+    else request->contenttype = strdup("application/octet-stream");
 
     post_data = readPostBody( request );
     if(strcmp(request->contenttype, "application/x-www-form-urlencoded")) 
