@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: map.h,v 1.416.2.1 2005/08/25 21:07:43 dan Exp $
+ * $Id: map.h,v 1.437.2.8 2006/03/30 23:40:50 hobu Exp $
  *
  * Project:  MapServer
  * Purpose:  Primary MapServer include file.
@@ -28,8 +28,93 @@
  ******************************************************************************
  *
  * $Log: map.h,v $
- * Revision 1.416.2.1  2005/08/25 21:07:43  dan
- * Ready for 4.6.1 release
+ * Revision 1.437.2.8  2006/03/30 23:40:50  hobu
+ * bump version to 4.8.3
+ *
+ * Revision 1.437.2.7  2006/03/22 22:08:29  hobu
+ * Bump version to 4.8.2
+ *
+ * Revision 1.437.2.6  2006/03/19 17:58:13  tamas
+ * Hiding items not to be exposed to the mapscript interface (bug 1660)
+ *
+ * Revision 1.437.2.5  2006/02/03 19:12:19  dan
+ * Update for 4.8.1
+ *
+ * Revision 1.437.2.4  2006/02/02 16:29:37  dan
+ * Update for 4.8.0 release
+ *
+ * Revision 1.437.2.3  2006/01/30 15:06:32  dan
+ * Treat classindex as an int instead of a char in resultCacheMemberObj to
+ * prevent problems with more than 128 classes (bug 1633)
+ *
+ * Revision 1.437.2.2  2006/01/25 22:55:17  dan
+ * Update for 4.8.0-rc3
+ *
+ * Revision 1.437.2.1  2006/01/16 20:41:22  sdlime
+ * Fixed error with image legends (shifted text) introduced by the 1449 bug fix. (bug 1607)
+ *
+ * Revision 1.437  2006/01/10 03:01:01  dan
+ * Update for 4.8.0-rc2
+ *
+ * Revision 1.436  2005/12/23 06:20:50  sdlime
+ * Updated files for 4.8.0-rc1.
+ *
+ * Revision 1.435  2005/12/16 13:28:33  dan
+ * Update version to 4.8.0-beta3
+ *
+ * Revision 1.434  2005/12/14 19:06:31  sdlime
+ * Added browseformat and legendformat to webObj. (bug 1518)
+ *
+ * Revision 1.433  2005/12/08 19:05:32  hobu
+ * switch off SDE raster support
+ *
+ * Revision 1.432  2005/11/24 04:38:57  dan
+ * Update version for 4.8.0-beta2
+ *
+ * Revision 1.431  2005/11/22 22:14:43  sean
+ * expose maximum length of layer, class, and style arrays in mapscript (bug 1522)
+ *
+ * Revision 1.430  2005/11/04 20:58:35  dan
+ * Updated MS_VERSION to 4.8.0-beta1
+ *
+ * Revision 1.429  2005/11/01 05:35:50  frank
+ * added preliminary implementation of OGR based WKT translation, still untested
+ *
+ * Revision 1.428  2005/10/30 05:05:07  sdlime
+ * Initial support for WKT via GEOS. The reader is only integrated via the map file reader, with MapScript, CGI and URL support following ASAP. (bug 1466)
+ *
+ * Revision 1.427  2005/10/29 02:03:43  jani
+ * MS RFC 8: Pluggable External Feature Layer Providers (bug 1477).
+ *
+ * Revision 1.426  2005/10/28 02:08:14  frank
+ * a few mapogr.cpp funcs used from WFS code, exposed in map.h
+ *
+ * Revision 1.425  2005/10/28 01:09:41  jani
+ * MS RFC 3: Layer vtable architecture (bug 1477)
+ *
+ * Revision 1.424  2005/10/20 19:37:03  frank
+ * added msAddPointToLine
+ *
+ * Revision 1.423  2005/10/18 03:10:45  frank
+ * added msShapeDeleteLine
+ *
+ * Revision 1.422  2005/10/14 05:04:12  sdlime
+ * Added msRotateSymbol(), changed freeSymbol to a public function called msFreeSymbol() in mapsymbol.c. Added to map.h as well.
+ *
+ * Revision 1.421  2005/09/23 21:18:58  assefa
+ * Remove call to ming.h inside map.h (Bug 1479)
+ *
+ * Revision 1.420  2005/08/25 14:20:16  sdlime
+ * Applied patch for bug 1440.
+ *
+ * Revision 1.419  2005/07/16 19:07:38  jerryp
+ * Bug 1420: PostGIS connector no longer needs two layer close functions.
+ *
+ * Revision 1.418  2005/07/13 19:35:08  julien
+ * Bug 1381: Support for case-insensitive Expression
+ *
+ * Revision 1.417  2005/06/29 14:11:50  dan
+ * Set MS_VERSION to 4.7
  *
  * Revision 1.416  2005/06/14 18:09:52  dan
  * Ready for 4.6.0 release
@@ -222,10 +307,11 @@
  *
  */
 
+
 #ifndef MAP_H
 #define MAP_H
 
-/* $Id: map.h,v 1.416.2.1 2005/08/25 21:07:43 dan Exp $
+/* $Id: map.h,v 1.437.2.8 2006/03/30 23:40:50 hobu Exp $
 **
 ** Main includes. If a particular header was needed by several .c files then
 ** I just put it here. What the hell, it works and it's all right here. -SDL-
@@ -256,6 +342,13 @@ static char *cvsid_aw(void) { return( cvsid_aw() ? ((char *) NULL) : ms_cvsid );
 #define  MS_DLL_EXPORT
 #endif
 
+#if UINT_MAX == 65535
+typedef long ms_int32;
+#else
+typedef int ms_int32;
+#endif
+    
+
 #ifdef USE_MPATROL
 #include "mpatrol.h"
 #endif
@@ -277,9 +370,6 @@ static char *cvsid_aw(void) { return( cvsid_aw() ? ((char *) NULL) : ms_cvsid );
 #include <pdflib.h>
 #endif
 
-#ifdef USE_MING_FLASH
-#include "ming.h"
-#endif
 
 #include <sys/types.h> /* regular expression support */
 
@@ -288,9 +378,6 @@ static char *cvsid_aw(void) { return( cvsid_aw() ? ((char *) NULL) : ms_cvsid );
  */
 #include "mapregex.h"
 
-#ifdef USE_MING
-#include "ming.h"
-#endif
 
 #ifdef USE_OGR
 #include "ogr_api.h"
@@ -307,7 +394,7 @@ extern "C" {
 
 /* General defines, wrapable */
 
-#define MS_VERSION "4.6.1"
+#define MS_VERSION "4.8.3"
 
 #define MS_TRUE 1 /* logical control variables */
 #define MS_FALSE 0
@@ -326,6 +413,12 @@ extern "C" {
    with RGB map images */
 #define MS_GD_ALPHA 1000
 
+/* Bounds on the lengths of layer, class, and style arrays are now exposed
+   to mapscript (bug 1522) */
+#define MS_MAXCLASSES 250
+#define MS_MAXSTYLES 5
+#define MS_MAXLAYERS 200
+
 /* General defines, not wrapable */
 #ifndef SWIG
 #define MS_DEFAULT_MAPFILE_PATTERN "\\.map$"
@@ -349,10 +442,7 @@ extern "C" {
 
 #define MS_MAXIMAGESIZE_DEFAULT 2048
 
-#define MS_MAXCLASSES 250
-#define MS_MAXSTYLES 5
 #define MS_MAXPROJARGS 20
-#define MS_MAXLAYERS 200 /* maximum number of layers in a map file */
 #define MS_MAXJOINS 20
 #define MS_ITEMNAMELEN 32
 #define MS_NAMELEN 20
@@ -383,6 +473,11 @@ extern "C" {
 #define MS_STRING 2002
 #define MS_NUMBER 2003
 #define MS_COMMENT 2004
+#define MS_IREGEX 2005
+#define MS_ISTRING 2006
+
+/* boolean options for the expression object. */
+#define MS_EXP_INSENSITIVE 1
 
 /* General macro definitions */
 #define MS_MIN(a,b) (((a)<(b))?(a):(b))
@@ -485,7 +580,7 @@ extern "C" {
     enum MS_LABEL_POSITIONS {MS_UL, MS_LR, MS_UR, MS_LL, MS_CR, MS_CL, MS_UC, MS_LC, MS_CC, MS_AUTO, MS_XY}; /* arrangement matters for auto placement, don't change it */
     enum MS_BITMAP_FONT_SIZES {MS_TINY , MS_SMALL, MS_MEDIUM, MS_LARGE, MS_GIANT};
     enum MS_QUERYMAP_STYLES {MS_NORMAL, MS_HILITE, MS_SELECTED};
-    enum MS_CONNECTION_TYPE {MS_INLINE, MS_SHAPEFILE, MS_TILED_SHAPEFILE, MS_SDE, MS_OGR, MS_UNUSED_1, MS_POSTGIS, MS_WMS, MS_ORACLESPATIAL, MS_WFS, MS_GRATICULE, MS_MYGIS, MS_RASTER };
+    enum MS_CONNECTION_TYPE {MS_INLINE, MS_SHAPEFILE, MS_TILED_SHAPEFILE, MS_SDE, MS_OGR, MS_UNUSED_1, MS_POSTGIS, MS_WMS, MS_ORACLESPATIAL, MS_WFS, MS_GRATICULE, MS_MYGIS, MS_RASTER, MS_PLUGIN };
     enum MS_JOIN_CONNECTION_TYPE {MS_DB_XBASE, MS_DB_CSV, MS_DB_MYSQL, MS_DB_ORACLE, MS_DB_POSTGRES};
     enum MS_JOIN_TYPE {MS_JOIN_ONE_TO_ONE, MS_JOIN_ONE_TO_MANY};
 
@@ -497,7 +592,39 @@ extern "C" {
 
 
 #define MS_FILE_DEFAULT MS_FILE_MAP   
-   
+
+
+#ifndef SWIG
+/* Filter object */    
+typedef enum 
+{
+    FILTER_NODE_TYPE_UNDEFINED = -1,
+    FILTER_NODE_TYPE_LOGICAL = 0,
+    FILTER_NODE_TYPE_SPATIAL = 1,
+    FILTER_NODE_TYPE_COMPARISON = 2,
+    FILTER_NODE_TYPE_PROPERTYNAME = 3,
+    FILTER_NODE_TYPE_BBOX = 4,
+    FILTER_NODE_TYPE_LITERAL = 5,
+    FILTER_NODE_TYPE_BOUNDARY = 6,
+    FILTER_NODE_TYPE_GEOMETRY_POINT = 7,
+    FILTER_NODE_TYPE_GEOMETRY_LINE = 8,
+    FILTER_NODE_TYPE_GEOMETRY_POLYGON = 9
+} FilterNodeType;
+
+
+typedef struct _FilterNode
+{
+    FilterNodeType      eType;
+    char                *pszValue;
+    void                *pOther;
+
+    struct _FilterNode  *psLeftNode;
+    struct _FilterNode  *psRightNode;
+
+      
+}FilterEncodingNode;
+#endif /*SWIG*/
+
 
 /* FONTSET OBJECT - used to hold aliases for TRUETYPE fonts */
     typedef struct {
@@ -540,6 +667,9 @@ extern "C" {
     typedef struct {
         char *string;
         int type;
+        /* container for expression options such as case-insensitiveness */
+        /* This is a boolean container. */
+        int flags;
 
         /* logical expression options */
         char **items;
@@ -666,6 +796,8 @@ typedef struct {
   char *mintemplate, *maxtemplate;
 
   char *queryformat; /* what format is the query to be returned, given as a MIME type */
+  char *legendformat;
+  char *browseformat;
 
   hashTableObj metadata;
 
@@ -797,7 +929,7 @@ typedef struct {
 typedef struct {
   long shapeindex;
   int tileindex;
-  char classindex;
+  int classindex;
 } resultCacheMemberObj;
 #ifdef SWIG
 %mutable;
@@ -901,6 +1033,7 @@ typedef struct {
   struct map_obj *map;
 } legendObj;
 
+#ifndef SWIG
 typedef struct
 {
   double    dwhichlatitude;
@@ -927,6 +1060,11 @@ typedef struct
   char    *labelformat;
 } graticuleObj;
 
+
+struct layerVTable;
+typedef struct layerVTable layerVTableObj;
+#endif /*SWIG*/
+
 /* LAYER OBJECT - basic unit of a map */
 typedef struct layer_obj {
 
@@ -1006,9 +1144,13 @@ typedef struct layer_obj {
 #endif /* SWIG */
 
   char *connection;
+  char *plugin_library;
+  char *plugin_library_original; /* this is needed for mapfile writing */
   enum MS_CONNECTION_TYPE connectiontype;
- 
+
 #ifndef SWIG
+  layerVTableObj *vtable;
+
   struct layer_obj *sameconnection;
   /* SDL has converted OracleSpatial, SDE, Graticules, MyGIS */
   void *layerinfo; /* all connection types should use this generic pointer to a vendor specific structure */
@@ -1066,6 +1208,7 @@ typedef struct layer_obj {
 #endif /* SWIG */
 } layerObj;
 
+
 /* MAP OBJECT - encompasses everything used in an Internet mapping application */
 typedef struct map_obj{ /* structure for a map */
   char *name; /* small identifier for naming etc. */
@@ -1097,8 +1240,9 @@ typedef struct map_obj{ /* structure for a map */
   rectObj extent; /* map extent array */
   double cellsize; /* in map units */
 
-
+#ifndef SWIG
   geotransformObj gt; /* rotation / geotransform */
+#endif /*SWIG*/
   rectObj saved_extent;
 
   enum MS_UNITS units; /* units of the projection */
@@ -1108,7 +1252,9 @@ typedef struct map_obj{ /* structure for a map */
   char *shapepath; /* where are the shape files located */
   char *mappath; /* path of the mapfile, all path are relative to this path */
 
+#ifndef SWIG
   paletteObj palette; /* holds a map palette */
+#endif /*SWIG*/
   colorObj imagecolor; /* holds the initial image color value */
 
 #ifdef SWIG
@@ -1145,21 +1291,6 @@ typedef struct map_obj{ /* structure for a map */
   hashTableObj configoptions;
 } mapObj;
 
-/* SWF Object structure */
-#ifdef USE_MING_FLASH
-typedef struct  {
-  mapObj *map;
-  SWFMovie sMainMovie;
-  int nLayerMovies;
-  SWFMovie *pasMovies;
-  int nCurrentMovie;
-  int nCurrentLayerIdx;
-  int nCurrentShapeIdx;
-  void    *imagetmp;  /*used when the output format is SINGLE */
-                      /*(one movie for the whole map)*/
-  int *panLayerIndex; /* keeps the layer index for every movie created. */
-} SWFObj;
-#endif
 
 
 /* PDF Object structure */
@@ -1171,6 +1302,7 @@ typedef struct {
 } PDFObj; 
 #endif
 
+#ifndef SWIG
 typedef struct  {
   mapObj *map;
   FILE *stream;
@@ -1178,6 +1310,7 @@ typedef struct  {
   int streamclosed; /* track if a save image is done */
   int compressed; /*track if output is set to be svgz */
 } SVGObj;
+#endif /*SWIG*/
 
 /* IMAGE OBJECT - a wrapper for GD images */
 typedef struct {
@@ -1198,7 +1331,7 @@ typedef struct {
   union {
     gdImagePtr gd;
 #ifdef USE_MING_FLASH
-    SWFObj *swf;
+    void *swf;
 #endif
 #ifdef USE_PDF
     PDFObj *pdf;
@@ -1215,6 +1348,41 @@ typedef struct {
 } imageObj;
 
 
+/* LAYER_VTABLE, contains function pointers to the layer operations 
+ * If you add new functions to here, remember to update
+ * populateVirtualTable in maplayer.c
+ */
+#ifndef SWIG
+struct layerVTable {
+    int (*LayerInitItemInfo)(layerObj *layer);
+    void (*LayerFreeItemInfo)(layerObj *layer);
+    int (*LayerOpen)(layerObj *layer);
+    int (*LayerIsOpen)(layerObj *layer);
+    int (*LayerWhichShapes)(layerObj *layer, rectObj rect);
+    int (*LayerNextShape)(layerObj *layer, shapeObj *shape);
+    int (*LayerGetShape)(layerObj *layer, shapeObj *shape, 
+                         int tile, long record);
+    int (*LayerClose)(layerObj *layer);
+    int (*LayerGetItems)(layerObj *layer);
+    int (*LayerGetExtent)(layerObj *layer, rectObj *extent);
+    int (*LayerGetAutoStyle)(mapObj *map, layerObj *layer, classObj *c, 
+                             int tile, long record);
+
+    int (*LayerCloseConnection)(layerObj *layer);
+
+    int (*LayerSetTimeFilter)(layerObj *layer, 
+                              const char *timestring, 
+                              const char *timefield);
+
+    int (*LayerApplyFilterToLayer)(FilterEncodingNode *psNode, mapObj *map,
+                                   int iLayerIndex, 
+                                   int bOnlySpatialFilter);
+
+    int (*LayerCreateItems)(layerObj *layer, int nt);
+    int (*LayerGetNumFeatures)(layerObj *layer);
+};
+#endif /*SWIG*/
+
 /* Function prototypes, wrapable */
 MS_DLL_EXPORT int msSaveImage(mapObj *map, imageObj *img, char *filename);
 MS_DLL_EXPORT void msFreeImage(imageObj *img);
@@ -1224,7 +1392,7 @@ MS_DLL_EXPORT void msCleanup(void);
 /* Function prototypes, not wrapable */
 
 #ifndef SWIG
-int msDrawSDELayer(mapObj *map, layerObj *layer, gdImagePtr img); /* in mapsde.c */
+/*int msDrawSDELayer(mapObj *map, layerObj *layer, gdImagePtr img);  in mapsde.c */
 
 /*
 ** helper functions not part of the general API but needed in
@@ -1235,6 +1403,9 @@ int getDouble(double *d);
 int getInteger(int *i);
 int getSymbol(int n, ...);
 int getCharacter(char *c);
+
+int msBuildPluginLibraryPath(char **dest, const char *lib_str, mapObj *map);
+
 MS_DLL_EXPORT int  hex2int(char *hex);
 
 MS_DLL_EXPORT void initSymbol(symbolObj *s);
@@ -1422,11 +1593,13 @@ MS_DLL_EXPORT int msCopySymbolSet(symbolSetObj *dst, symbolSetObj *src, mapObj *
 MS_DLL_EXPORT void msInitSymbolSet(symbolSetObj *symbolset);
 MS_DLL_EXPORT int msAddImageSymbol(symbolSetObj *symbolset, char *filename);
 MS_DLL_EXPORT void msFreeSymbolSet(symbolSetObj *symbolset);
+MS_DLL_EXPORT void msFreeSymbol(symbolObj *symbol);
 MS_DLL_EXPORT int msAddNewSymbol(mapObj *map, char *name);
 MS_DLL_EXPORT int msAppendSymbol(symbolSetObj *symbolset, symbolObj *symbol);
 MS_DLL_EXPORT symbolObj *msRemoveSymbol(symbolSetObj *symbolset, int index);
 MS_DLL_EXPORT int msSaveSymbolSet(symbolSetObj *symbolset, const char *filename);
 MS_DLL_EXPORT int msLoadImageSymbol(symbolObj *symbol, const char *filename);
+MS_DLL_EXPORT symbolObj *msRotateSymbol(symbolObj *symbol, double angle);
 
 MS_DLL_EXPORT imageObj *msSymbolGetImageGD(symbolObj *symbol, outputFormatObj *format);
 MS_DLL_EXPORT int msSymbolSetImageGD(symbolObj *symbol, imageObj *image);
@@ -1445,7 +1618,7 @@ MS_DLL_EXPORT int msLoadFontSet(fontSetObj *fontSet, mapObj *map); /* in maplabe
 MS_DLL_EXPORT int msInitFontSet(fontSetObj *fontset);
 MS_DLL_EXPORT int msFreeFontSet(fontSetObj *fontset);
 
-MS_DLL_EXPORT int msGetLabelSize(char *string, labelObj *label, rectObj *rect, fontSetObj *fontSet, double scalefactor);
+MS_DLL_EXPORT int msGetLabelSize(char *string, labelObj *label, rectObj *rect, fontSetObj *fontSet, double scalefactor, int adjustBaseline);
 MS_DLL_EXPORT int msAddLabel(mapObj *map, int layerindex, int classindex, int shapeindex, int tileindex, pointObj *point, char *string, double featuresize, labelObj *);
 
 MS_DLL_EXPORT gdFontPtr msGetBitmapFont(int size);
@@ -1453,7 +1626,10 @@ MS_DLL_EXPORT int msImageTruetypePolyline(symbolSetObj *symbolset, gdImagePtr im
 MS_DLL_EXPORT int msImageTruetypeArrow(symbolSetObj *symbolset, gdImagePtr img, shapeObj *p, styleObj *style, double scalefactor);
 
 MS_DLL_EXPORT void msFreeShape(shapeObj *shape); /* in mapprimative.c */
+MS_DLL_EXPORT shapeObj *msShapeFromWKT(const char *string);
+MS_DLL_EXPORT char *msShapeToWKT(shapeObj *shape);
 MS_DLL_EXPORT void msInitShape(shapeObj *shape);
+MS_DLL_EXPORT void msShapeDeleteLine( shapeObj *shape, int line );
 MS_DLL_EXPORT int msCopyShape(shapeObj *from, shapeObj *to);
 MS_DLL_EXPORT int *msGetOuterList(shapeObj *shape);
 MS_DLL_EXPORT int *msGetInnerList(shapeObj *shape, int r, int *outerlist);
@@ -1469,6 +1645,7 @@ MS_DLL_EXPORT int msPolylineLabelPoint(shapeObj *p, pointObj *lp, int min_length
 MS_DLL_EXPORT int msPolygonLabelPoint(shapeObj *p, pointObj *lp, int min_dimension);
 MS_DLL_EXPORT int msAddLine(shapeObj *p, lineObj *new_line);
 MS_DLL_EXPORT int msAddLineDirectly(shapeObj *p, lineObj *new_line);
+MS_DLL_EXPORT int msAddPointToLine(lineObj *line, pointObj *point );
 
 MS_DLL_EXPORT int msDrawRasterLayer(mapObj *map, layerObj *layer, imageObj *image); /* in mapraster.c */
 MS_DLL_EXPORT imageObj *msDrawReferenceMap(mapObj *map);
@@ -1479,6 +1656,9 @@ MS_DLL_EXPORT int msGetBit(char *array, int index);
 MS_DLL_EXPORT void msSetBit(char *array, int index, int value);
 MS_DLL_EXPORT void msFlipBit(char *array, int index);
 
+MS_DLL_EXPORT int msLayerInitItemInfo(layerObj *layer);
+MS_DLL_EXPORT void msLayerFreeItemInfo(layerObj *layer); 
+
 MS_DLL_EXPORT int msLayerOpen(layerObj *layer); /* in maplayer.c */
 MS_DLL_EXPORT int msLayerIsOpen(layerObj *layer);
 MS_DLL_EXPORT void msLayerClose(layerObj *layer);
@@ -1501,109 +1681,55 @@ MS_DLL_EXPORT char* msLayerGetFilterString( layerObj *layer );
 
 MS_DLL_EXPORT int msLayerSetTimeFilter(layerObj *lp, const char *timestring, 
                                        const char *timefield);
+/* Helper functions for layers */ 
+MS_DLL_EXPORT int msLayerMakeBackticsTimeFilter(layerObj *lp, const char *timestring, 
+                                                 const char *timefield);
+
+MS_DLL_EXPORT int msLayerMakePlainTimeFilter(layerObj *lp, const char *timestring, 
+                                             const char *timefield);
+
+MS_DLL_EXPORT int msLayerApplyCondSQLFilterToLayer(FilterEncodingNode *psNode, mapObj *map, 
+                                                   int iLayerIndex, int bOnlySpatialFilter);
+
+MS_DLL_EXPORT int msLayerApplyPlainFilterToLayer(FilterEncodingNode *psNode, mapObj *map, 
+                                                 int iLayerIndex, int bOnlySpatialFilter);
+
 
 /* maplayer.c */
-MS_DLL_EXPORT int msINLINELayerGetShape(layerObj *layer, shapeObj *shape, int shapeindex);
 MS_DLL_EXPORT int msLayerGetNumFeatures(layerObj *layer);
 
-MS_DLL_EXPORT int msTiledSHPOpenFile(layerObj *layer); /* in mapshape.c */
-MS_DLL_EXPORT int msTiledSHPWhichShapes(layerObj *layer, rectObj rect);
-MS_DLL_EXPORT int msTiledSHPNextShape(layerObj *layer, shapeObj *shape);
-MS_DLL_EXPORT int msTiledSHPGetShape(layerObj *layer, shapeObj *shape, int tile, long record);
-MS_DLL_EXPORT void msTiledSHPClose(layerObj *layer);
-MS_DLL_EXPORT int msTiledSHPLayerGetItems(layerObj *layer);
-MS_DLL_EXPORT int msTiledSHPLayerInitItemInfo(layerObj *layer);
-MS_DLL_EXPORT int msTiledSHPLayerGetExtent(layerObj *layer, rectObj *extent);
+/* These are special because SWF is using these */
+int msOGRLayerNextShape(layerObj *layer, shapeObj *shape);
+int msOGRLayerGetItems(layerObj *layer);
+void msOGRLayerFreeItemInfo(layerObj *layer);
+int msOGRLayerGetShape(layerObj *layer, shapeObj *shape, int tile, long record);
+int msOGRLayerGetExtent(layerObj *layer, rectObj *extent);
 
-MS_DLL_EXPORT int msOGRLayerOpen(layerObj *layer, const char *pszOverrideConnection); /* in mapogr.cpp */
-MS_DLL_EXPORT int msOGRLayerIsOpen(layerObj *layer);
-MS_DLL_EXPORT int msOGRLayerClose(layerObj *layer);
-MS_DLL_EXPORT int msOGRLayerWhichShapes(layerObj *layer, rectObj rect);
-MS_DLL_EXPORT int msOGRLayerNextShape(layerObj *layer, shapeObj *shape);
-MS_DLL_EXPORT int msOGRLayerGetItems(layerObj *layer);
-MS_DLL_EXPORT int msOGRLayerInitItemInfo(layerObj *layer);
-MS_DLL_EXPORT void msOGRLayerFreeItemInfo(layerObj *layer);
-MS_DLL_EXPORT int msOGRLayerGetShape(layerObj *layer, shapeObj *shape, int tile, long record);
-MS_DLL_EXPORT int msOGRLayerGetExtent(layerObj *layer, rectObj *extent);
-MS_DLL_EXPORT int msOGRLayerGetAutoStyle(mapObj *map, layerObj *layer, classObj *c, int tile, long record);
 #ifdef USE_OGR
 MS_DLL_EXPORT int msOGRGeometryToShape(OGRGeometryH hGeometry, shapeObj *shape,
                          OGRwkbGeometryType type);
 #endif /* USE_OGR */
 
-MS_DLL_EXPORT int msPOSTGISLayerOpen(layerObj *layer); /* in mappostgis.c */
-MS_DLL_EXPORT int msPOSTGISLayerIsOpen(layerObj *layer);
-MS_DLL_EXPORT void msPOSTGISLayerFreeItemInfo(layerObj *layer);
-MS_DLL_EXPORT int msPOSTGISLayerInitItemInfo(layerObj *layer);
-MS_DLL_EXPORT int msPOSTGISLayerWhichShapes(layerObj *layer, rectObj rect);
-MS_DLL_EXPORT int msPOSTGISLayerClose(layerObj *layer);
-MS_DLL_EXPORT int msPOSTGISLayerNextShape(layerObj *layer, shapeObj *shape);
-MS_DLL_EXPORT int msPOSTGISLayerGetShape(layerObj *layer, shapeObj *shape, long record);
-MS_DLL_EXPORT int msPOSTGISLayerGetExtent(layerObj *layer, rectObj *extent);
 MS_DLL_EXPORT int msPOSTGISLayerGetShapeRandom(layerObj *layer, shapeObj *shape, long *record);
-MS_DLL_EXPORT int msPOSTGISLayerGetItems(layerObj *layer);
-MS_DLL_EXPORT int msPOSTGISLayerResultClose(layerObj *layer);
-
-MS_DLL_EXPORT int msMYGISLayerOpen(layerObj *layer); /* in mapmygis.c */
-MS_DLL_EXPORT int msMYGISLayerIsOpen(layerObj *layer);
-MS_DLL_EXPORT void msMYGISLayerFreeItemInfo(layerObj *layer);
-MS_DLL_EXPORT int msMYGISLayerInitItemInfo(layerObj *layer);
-MS_DLL_EXPORT int msMYGISLayerWhichShapes(layerObj *layer, rectObj rect);
-MS_DLL_EXPORT int msMYGISLayerClose(layerObj *layer);
-MS_DLL_EXPORT int msMYGISLayerNextShape(layerObj *layer, shapeObj *shape);
-MS_DLL_EXPORT int msMYGISLayerGetShape(layerObj *layer, shapeObj *shape, long record);
-MS_DLL_EXPORT int msMYGISLayerGetExtent(layerObj *layer, rectObj *extent);
 MS_DLL_EXPORT int msMYGISLayerGetShapeRandom(layerObj *layer, shapeObj *shape, long *record);
-MS_DLL_EXPORT int msMYGISLayerGetItems(layerObj *layer);
-
-MS_DLL_EXPORT int msSDELayerOpen(layerObj *layer); /* in mapsde.c */
-MS_DLL_EXPORT int msSDELayerIsOpen(layerObj *layer);
-MS_DLL_EXPORT void msSDELayerClose(layerObj *layer);
-MS_DLL_EXPORT int msSDELayerWhichShapes(layerObj *layer, rectObj rect);
-MS_DLL_EXPORT int msSDELayerNextShape(layerObj *layer, shapeObj *shape);
-MS_DLL_EXPORT int msSDELayerGetItems(layerObj *layer);
-MS_DLL_EXPORT int msSDELayerGetShape(layerObj *layer, shapeObj *shape, long record);
-MS_DLL_EXPORT int msSDELayerGetExtent(layerObj *layer, rectObj *extent);
-MS_DLL_EXPORT int msSDELayerInitItemInfo(layerObj *layer);
-MS_DLL_EXPORT void msSDELayerFreeItemInfo(layerObj *layer);
-MS_DLL_EXPORT char *msSDELayerGetSpatialColumn(layerObj *layer);
-MS_DLL_EXPORT char *msSDELayerGetRowIDColumn(layerObj *layer);
 MS_DLL_EXPORT int drawSDE(mapObj *map, layerObj *layer, gdImagePtr img);
 
-MS_DLL_EXPORT int msOracleSpatialLayerOpen(layerObj *layer);
-MS_DLL_EXPORT int msOracleSpatialLayerIsOpen(layerObj *layer);
-MS_DLL_EXPORT int msOracleSpatialLayerClose(layerObj *layer);
-MS_DLL_EXPORT int msOracleSpatialLayerWhichShapes(layerObj *layer, rectObj rect);
-MS_DLL_EXPORT int msOracleSpatialLayerNextShape(layerObj *layer, shapeObj *shape);
-MS_DLL_EXPORT int msOracleSpatialLayerGetItems(layerObj *layer);
-MS_DLL_EXPORT int msOracleSpatialLayerGetShape(layerObj *layer, shapeObj *shape, long record);
-MS_DLL_EXPORT int msOracleSpatialLayerGetExtent(layerObj *layer, rectObj *extent);
-MS_DLL_EXPORT int msOracleSpatialLayerInitItemInfo(layerObj *layer);
-MS_DLL_EXPORT void msOracleSpatialLayerFreeItemInfo(layerObj *layer);
-MS_DLL_EXPORT int msOracleSpatialLayerGetAutoStyle(mapObj *map, layerObj *layer, classObj *c, int tile, long record);   
+MS_DLL_EXPORT int msInitializeVirtualTable(layerObj *layer);
+MS_DLL_EXPORT int msConnectLayer(layerObj *layer, const int connectiontype, 
+                                 const char *library_str);
 
-MS_DLL_EXPORT int msGraticuleLayerOpen(layerObj *layer);   /* in mapGraticule.cpp */
-MS_DLL_EXPORT int msGraticuleLayerIsOpen(layerObj *layer);
-MS_DLL_EXPORT int msGraticuleLayerClose(layerObj *layer);
-MS_DLL_EXPORT int msGraticuleLayerWhichShapes(layerObj *layer, rectObj rect);
-MS_DLL_EXPORT int msGraticuleLayerNextShape(layerObj *layer, shapeObj *shape);
-MS_DLL_EXPORT int msGraticuleLayerGetItems(layerObj *layer);
-MS_DLL_EXPORT int msGraticuleLayerInitItemInfo(layerObj *layer);
-MS_DLL_EXPORT void msGraticuleLayerFreeItemInfo(layerObj *layer);
-MS_DLL_EXPORT int msGraticuleLayerGetShape(layerObj *layer, shapeObj *shape, int tile, long record);
-MS_DLL_EXPORT int msGraticuleLayerGetExtent(layerObj *layer, rectObj *extent);
-MS_DLL_EXPORT int msGraticuleLayerGetAutoStyle(mapObj *map, layerObj *layer, classObj *c, int tile, long record);
-
-MS_DLL_EXPORT int msRASTERLayerOpen(layerObj *layer); /* in maprasterquery.c */
-MS_DLL_EXPORT int msRASTERLayerIsOpen(layerObj *layer);
-MS_DLL_EXPORT void msRASTERLayerFreeItemInfo(layerObj *layer);
-MS_DLL_EXPORT int msRASTERLayerInitItemInfo(layerObj *layer);
-MS_DLL_EXPORT int msRASTERLayerWhichShapes(layerObj *layer, rectObj rect);
-MS_DLL_EXPORT int msRASTERLayerClose(layerObj *layer);
-MS_DLL_EXPORT int msRASTERLayerNextShape(layerObj *layer, shapeObj *shape);
-MS_DLL_EXPORT int msRASTERLayerGetShape(layerObj *layer, shapeObj *shape, int tile, long record);
-MS_DLL_EXPORT int msRASTERLayerGetExtent(layerObj *layer, rectObj *extent);
-MS_DLL_EXPORT int msRASTERLayerGetItems(layerObj *layer);
+MS_DLL_EXPORT int msINLINELayerInitializeVirtualTable(layerObj *layer);
+MS_DLL_EXPORT int msShapeFileLayerInitializeVirtualTable(layerObj *layer);
+MS_DLL_EXPORT int msTiledSHPLayerInitializeVirtualTable(layerObj *layer);
+MS_DLL_EXPORT int msSDELayerInitializeVirtualTable(layerObj *layer);
+MS_DLL_EXPORT int msOGRLayerInitializeVirtualTable(layerObj *layer);
+MS_DLL_EXPORT int msPOSTGISLayerInitializeVirtualTable(layerObj *layer);
+MS_DLL_EXPORT int msOracleSpatialLayerInitializeVirtualTable(layerObj *layer);
+MS_DLL_EXPORT int msWFSLayerInitializeVirtualTable(layerObj *layer);
+MS_DLL_EXPORT int msGraticuleLayerInitializeVirtualTable(layerObj *layer);
+MS_DLL_EXPORT int msMYGISLayerInitializeVirtualTable(layerObj *layer);
+MS_DLL_EXPORT int msRASTERLayerInitializeVirtualTable(layerObj *layer);
+MS_DLL_EXPORT int msPluginLayerInitializeVirtualTable(layerObj *layer);
 
 /* ==================================================================== */
 /*      Prototypes for functions in mapdraw.c                           */
@@ -1910,6 +2036,16 @@ MS_DLL_EXPORT int msInitDefaultGDALOutputFormat( outputFormatObj *format );
 /*      End of prototypes for functions in mapoutput.c                  */
 /* ==================================================================== */
 
+/* ==================================================================== */
+/*      Public prototype for mapogr.cpp functions.                      */
+/* ==================================================================== */
+int MS_DLL_EXPORT msOGRLayerWhichShapes(layerObj *layer, rectObj rect);
+int MS_DLL_EXPORT msOGRLayerOpen(layerObj *layer, const char *pszOverrideConnection); /* in mapogr.cpp */
+int MS_DLL_EXPORT msOGRLayerClose(layerObj *layer);
+
+char MS_DLL_EXPORT *msOGRShapeToWKT(shapeObj *shape);
+shapeObj MS_DLL_EXPORT *msOGRShapeFromWKT(const char *string);
+
 /* ==================================================================== */
 /*      prototypes for functions in mapcopy                             */
 /* ==================================================================== */
@@ -1941,6 +2077,8 @@ MS_DLL_EXPORT void msConnPoolFinalCleanup( void );
 /*      prototypes for functions in mapcpl.c                            */
 /* ==================================================================== */
 MS_DLL_EXPORT const char *msGetBasename( const char *pszFullFilename );
+MS_DLL_EXPORT void *msGetSymbol(const char *pszLibrary, 
+                                const char *pszEntryPoint);
 
 /* ==================================================================== */
 /*      include definitions from mapows.h                               */
@@ -1951,6 +2089,8 @@ MS_DLL_EXPORT const char *msGetBasename( const char *pszFullFilename );
 /*      prototypes for functions in mapgeos.cpp                         */
 /* ==================================================================== */
 MS_DLL_EXPORT void msGEOSFreeGeometry(shapeObj *shape);
+MS_DLL_EXPORT shapeObj *msGEOSShapeFromWKT(const char *string);
+MS_DLL_EXPORT char *msGEOSShapeToWKT(shapeObj *shape);
 MS_DLL_EXPORT shapeObj *msGEOSBuffer(shapeObj *shape, double width);
 MS_DLL_EXPORT shapeObj *msGEOSConvexHull(shapeObj *shape);
 
