@@ -39,10 +39,19 @@
  ******************************************************************************
  *
  * $Log: mapcopy.c,v $
- * Revision 1.40.2.2  2005/06/29 14:30:29  sean
+ * Revision 1.45  2005/12/14 19:13:47  sdlime
+ * Patched mapfile read/write/copy routines to deal with browseformat and legendformat.
+ *
+ * Revision 1.44  2005/11/28 04:32:42  sdlime
+ * Updated msCopyLabel() to copy encoding as well.
+ *
+ * Revision 1.43  2005/11/28 04:22:54  sdlime
+ * Removed some unnecessary and potentially harmful if statements in msCopyStyle(). The macros should be cufficient.
+ *
+ * Revision 1.42  2005/06/29 14:14:23  sean
  * no longer copying layer and join items, as these will be initialized on opening of layer and joining of data (bug 1403)
  *
- * Revision 1.40.2.1  2005/06/28 17:01:51  sean
+ * Revision 1.41  2005/06/28 16:33:23  sean
  * Fixed copy of layer processing directives with use of msLayerGetProcessing and msLayerAddProcessing (bug 1399). Also caught error in the upper bounds of looping over layer joins.
  *
  * Revision 1.40  2005/06/14 16:03:33  dan
@@ -78,7 +87,7 @@
 #include "map.h"
 #include "mapsymbol.h"
 
-MS_CVSID("$Id: mapcopy.c,v 1.40.2.2 2005/06/29 14:30:29 sean Exp $")
+MS_CVSID("$Id: mapcopy.c,v 1.45 2005/12/14 19:13:47 sdlime Exp $")
 
 #include "mapcopy.h"
 
@@ -236,8 +245,8 @@ int msCopyJoin(joinObj *dst, joinObj *src)
 {
     MS_COPYSTRING(dst->name, src->name);
 
-    /* makes no sense to copy the values or items since
-       they are runtime additions to the mapfile */
+    /* makes no sense to copy the items or values
+       since they are runtime additions to the mapfile */
 
     MS_COPYSTRING(dst->table, src->table);
     MS_COPYSTRING(dst->from, src->from);
@@ -320,6 +329,8 @@ int msCopyLabel(labelObj *dst, labelObj *src)
     MS_COPYSTELEM(partials);
     MS_COPYSTELEM(force);
 
+    MS_COPYSTRING(dst->encoding, src->encoding);
+
     return MS_SUCCESS;
 }
 
@@ -360,6 +371,10 @@ int msCopyWeb(webObj *dst, webObj *src, mapObj *map)
             return MS_FAILURE;
     }
 
+    MS_COPYSTRING(dst->queryformat, src->queryformat);
+    MS_COPYSTRING(dst->legendformat, src->legendformat);
+    MS_COPYSTRING(dst->browseformat, src->browseformat);
+
     return MS_SUCCESS ;
 }
 
@@ -379,10 +394,7 @@ int msCopyStyle(styleObj *dst, styleObj *src)
     MS_COPYCOLOR(&(dst->mincolor), &(src->mincolor));
     MS_COPYCOLOR(&(dst->maxcolor), &(src->maxcolor));
 
-    /* The macro should protect us but it isn't in this case */
-    if (src->symbolname) {
-        MS_COPYSTRING(dst->symbolname, src->symbolname);
-    }
+    MS_COPYSTRING(dst->symbolname, src->symbolname);
 
     MS_COPYSTELEM(symbol);
     MS_COPYSTELEM(size);
@@ -399,9 +411,9 @@ int msCopyStyle(styleObj *dst, styleObj *src)
     MS_COPYSTELEM(minvalue);
     MS_COPYSTELEM(maxvalue);
 
-    if (src->angleitem) MS_COPYSTRING(dst->angleitem, src->angleitem);
-    if (src->sizeitem) MS_COPYSTRING(dst->sizeitem, src->sizeitem);
-    if (src->rangeitem) MS_COPYSTRING(dst->rangeitem, src->rangeitem);
+    MS_COPYSTRING(dst->angleitem, src->angleitem);
+    MS_COPYSTRING(dst->sizeitem, src->sizeitem);
+    MS_COPYSTRING(dst->rangeitem, src->rangeitem);
 
     MS_COPYSTELEM(angleitemindex);
     MS_COPYSTELEM(sizeitemindex);
@@ -797,8 +809,8 @@ int msCopyLayer(layerObj *dst, layerObj *src)
     MS_COPYSTELEM(connectiontype);
     MS_COPYSTELEM(sameconnection);
 
-    /* layerinfo, items, and iteminfo are not copied. these are set when the
-       copied layer is opened */
+    /* Do not copy *layerinfo, items, or iteminfo. these are all initialized
+       when the copied layer is opened */
 
     return_value = msCopyExpression(&(dst->filter), &(src->filter));
     if (return_value != MS_SUCCESS) {
@@ -831,13 +843,14 @@ int msCopyLayer(layerObj *dst, layerObj *src)
     }
 
     MS_COPYSTELEM(numjoins);
+
     for (i = 0; i < dst->numjoins; i++) {
         return_value = msCopyJoin(&(dst->joins[i]), &(src->joins[i]));
         if (return_value != MS_SUCCESS)
             return MS_FAILURE;
     }
 
-    MS_COPYRECT(&(dst->extent), &(src->extent));
+		MS_COPYRECT(&(dst->extent), &(src->extent));
     
     return MS_SUCCESS;
 }
