@@ -27,7 +27,16 @@
  ******************************************************************************
  *
  * $Log: mapdraw.c,v $
- * Revision 1.93.2.1  2005/08/05 18:46:43  sdlime
+ * Revision 1.96.2.1  2006/01/16 20:41:22  sdlime
+ * Fixed error with image legends (shifted text) introduced by the 1449 bug fix. (bug 1607)
+ *
+ * Revision 1.96  2005/11/29 14:12:16  sdlime
+ * Fixed an error in mapdraw.c where item indicies where not being computed correctly with query maps. (bug 1492)
+ *
+ * Revision 1.95  2005/10/26 17:44:28  frank
+ * avoid warning when building without WMS and WFS.
+ *
+ * Revision 1.94  2005/08/05 18:45:07  sdlime
  * Added code to apply sizeitem and angleitem to points.
  *
  * Revision 1.93  2005/06/14 16:03:33  dan
@@ -94,7 +103,7 @@
 #include "map.h"
 #include "maptime.h"
 
-MS_CVSID("$Id: mapdraw.c,v 1.93.2.1 2005/08/05 18:46:43 sdlime Exp $")
+MS_CVSID("$Id: mapdraw.c,v 1.96.2.1 2006/01/16 20:41:22 sdlime Exp $")
 
 /*
  * Functions to reset any pen (color index) values previously set. Used primarily to reset things when
@@ -341,13 +350,13 @@ imageObj *msDrawMap(mapObj *map)
     int i;
     layerObj *lp=NULL;
     int status = MS_FAILURE;
-    enum MS_CONNECTION_TYPE lastconnectiontype;
     imageObj *image = NULL;
     struct mstimeval mapstarttime, mapendtime;
     struct mstimeval starttime, endtime;
     int oldAlphaBlending;  /* allows toggling of gd alpha blending (bug 490) */
 
 #if defined(USE_WMS_LYR) || defined(USE_WFS_LYR)
+    enum MS_CONNECTION_TYPE lastconnectiontype;
     httpRequestObj asOWSReqInfo[MS_MAXLAYERS+1];
     int numOWSRequests=0;
     wmsParamsObj sLastWMSParams;
@@ -1103,7 +1112,7 @@ int msDrawQueryLayer(mapObj *map, layerObj *layer, imageObj *image)
   if(status != MS_SUCCESS) return(MS_FAILURE);
 
   /* build item list */
-  status = msLayerWhichItems(layer, MS_FALSE, annotate, NULL); /* FIX: results have already been classified (this may change) */
+  status = msLayerWhichItems(layer, MS_TRUE, annotate, NULL); /* FIX: results have already been classified (this may change) */
   if(status != MS_SUCCESS) return(MS_FAILURE);
 
   msInitShape(&shape);
@@ -1502,7 +1511,7 @@ int msDrawShape(mapObj *map, layerObj *layer, shapeObj *shape, imageObj *image,
 
 	for(s=0; s<layer->class[c].numstyles; s++) {
 	  if(layer->class[c].styles[s].angleitemindex != -1) layer->class[c].styles[s].angle = atof(shape->values[layer->class[c].styles[s].angleitemindex]);
-	  if(layer->class[c].styles[s].sizeitemindex != -1) layer->class[c].styles[s].size = atoi(shape->values[layer->class[c].styles[s].sizeitemindex]);  	  
+	  if(layer->class[c].styles[s].sizeitemindex != -1) layer->class[c].styles[s].size = atoi(shape->values[layer->class[c].styles[s].sizeitemindex]);      
   	  msDrawMarkerSymbol(&map->symbolset, image, point, &(layer->class[c].styles[s]), layer->scalefactor);
 	}
 
@@ -1845,7 +1854,7 @@ int msDrawLabel(imageObj *image, pointObj labelPnt, char *string,
     pointObj p;
     rectObj r;
 
-    if(msGetLabelSize(string, label, &r, fontset, scalefactor) == -1) return(-1);
+    if(msGetLabelSize(string, label, &r, fontset, scalefactor, MS_FALSE) == -1) return(-1);
     p = get_metrics(&labelPnt, label->position, r, label->offsetx, label->offsety, label->angle, 0, NULL);
     msDrawText(image, p, string, label, fontset, scalefactor); /* actually draw the label */
   } else {
