@@ -28,6 +28,12 @@
  ******************************************************************************
  *
  * $Log: mapdrawgdal.c,v $
+ * Revision 1.47  2005/11/24 16:16:41  frank
+ * fixed raster crash with some grass rasters (bug 1541)
+ *
+ * Revision 1.46  2005/08/25 22:01:50  frank
+ * fix problem with ungeorerenced files (eg augsignals)
+ *
  * Revision 1.45  2005/06/14 16:03:33  dan
  * Updated copyright date to 2005
  *
@@ -136,7 +142,7 @@
 #include "mapresample.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mapdrawgdal.c,v 1.45 2005/06/14 16:03:33 dan Exp $")
+MS_CVSID("$Id: mapdrawgdal.c,v 1.47 2005/11/24 16:16:41 frank Exp $")
 
 extern int InvGeoTransform( double *gt_in, double *gt_out );
 
@@ -580,7 +586,8 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
           double dfNoDataValue = msGetGDALNoDataValue( layer, hBand1, 
                                                        &bGotNoData);
 
-          if( bGotNoData && dfNoDataValue >= 0 && dfNoDataValue <= 255.0 )
+          if( bGotNoData && dfNoDataValue >= 0 
+              && dfNoDataValue < GDALGetColorEntryCount( hColorMap ) )
           {
               GDALColorEntry sEntry;
               
@@ -1408,9 +1415,20 @@ int msGetGDALGeoTransform( GDALDatasetH hDS, mapObj *map, layerObj *layer,
 
 /* -------------------------------------------------------------------- */
 /*      We didn't find any info ... use the default.                    */
+/*      Reset our default geotransform.  GDALGetGeoTransform() may      */
+/*      have altered it even if GDALGetGeoTransform() failed.           */
 /* -------------------------------------------------------------------- */
     else
+    {
+        padfGeoTransform[0] = 0.0;
+        padfGeoTransform[1] = 1.0;
+        padfGeoTransform[2] = 0.0;
+        padfGeoTransform[3] = GDALGetRasterYSize(hDS);
+        padfGeoTransform[4] = 0.0;
+        padfGeoTransform[5] = -1.0;
+
         return MS_FAILURE;
+    }
 }
 
 /************************************************************************/
