@@ -27,6 +27,20 @@
  ******************************************************************************
  *
  * $Log: maperror.c,v $
+ * Revision 1.71.2.1  2006/03/19 19:14:03  tamas
+ * Fixed msGetErrorString so as not to truncate the length of the error messages
+ * Added msAddErrorDisplayString to read the displayable messages
+ *
+ * Revision 1.71  2005/12/12 16:57:56  sean
+ * write whether point/shape z and m is supported in version message
+ *
+ * Revision 1.70  2005/11/17 14:38:21  assefa
+ * Correct function msGetVersion to indicate if mapserver was build with
+ * MYGIS support.
+ *
+ * Revision 1.69  2005/09/13 23:43:31  frank
+ * fix leak in threaded case of error object
+ *
  * Revision 1.68  2005/06/14 16:03:33  dan
  * Updated copyright date to 2005
  *
@@ -80,7 +94,7 @@
 #include <fcntl.h>
 #endif
 
-MS_CVSID("$Id: maperror.c,v 1.68 2005/06/14 16:03:33 dan Exp $")
+MS_CVSID("$Id: maperror.c,v 1.71.2.1 2006/03/19 19:14:03 tamas Exp $")
 
 static char *ms_errorCodes[MS_NUMERRORCODES] = {"",
 						"Unable to access file.",
@@ -140,9 +154,10 @@ typedef struct te_info
     errorObj        ms_error;
 } te_info_t;
 
+static te_info_t *error_list = NULL;
+
 errorObj *msGetErrorObj()
 {
-    static te_info_t *error_list = NULL;
     te_info_t *link;
     int        thread_id;
     errorObj   *ret_obj;
@@ -269,6 +284,42 @@ void msResetErrorList()
   ms_error->code = MS_NOERR;
   ms_error->routine[0] = '\0';
   ms_error->message[0] = '\0';
+
+/* -------------------------------------------------------------------- */
+/*      Cleanup our entry in the thread list.  This is mainly           */
+/*      imprortant when msCleanup() calls msResetErrorList().           */
+/* -------------------------------------------------------------------- */
+#ifdef USE_THREAD
+  {
+      int  thread_id = msGetThreadId();
+      te_info_t *link;
+
+      msAcquireLock( TLOCK_ERROROBJ );
+      
+      /* find link for this thread */
+    
+      for( link = error_list; 
+           link != NULL && link->thread_id != thread_id
+               && link->next != NULL && link->next->thread_id != thread_id;
+           link = link->next ) {}
+      
+      if( link->thread_id == thread_id )
+      { 
+          /* presumably link is at head of list.  */
+          if( error_list == link )
+              error_list = link->next;
+
+          free( link );
+      }
+      else if( link->next != NULL && link->next->thread_id == thread_id )
+      {
+          te_info_t *next_link = link->next;
+          link->next = link->next->next;
+          free( next_link );
+      }
+      msReleaseLock( TLOCK_ERROROBJ );
+  }
+#endif
 }
 
 char *msGetErrorCodeString(int code) {
@@ -279,25 +330,35 @@ char *msGetErrorCodeString(int code) {
   return(ms_errorCodes[code]);
 }
 
+/* -------------------------------------------------------------------- */
+/*      Adding the displayable error string to a given string           */
+/*      and reallocates the memory enough to hold the characters.       */
+/*      If source is null returns a newly allocated string              */
+/* -------------------------------------------------------------------- */
+char *msAddErrorDisplayString(char *source, errorObj *error)
+{
+	if((source = strcatalloc(source, error->routine)) == NULL) return(NULL);
+	if((source = strcatalloc(source, ": ")) == NULL) return(NULL);
+	if((source = strcatalloc(source, ms_errorCodes[error->code])) == NULL) return(NULL);
+	if((source = strcatalloc(source, " ")) == NULL) return(NULL);
+	if((source = strcatalloc(source, error->message)) == NULL) return(NULL);
+	return source;
+}
+
 char *msGetErrorString(char *delimiter) 
 {
-  char errbuf[256];
   char *errstr=NULL;
 
   errorObj *error = msGetErrorObj();
 
   if(!delimiter || !error) return(NULL);
 
-  if((errstr = strdup("")) == NULL) return(NULL); /* empty at first */
   while(error && error->code != MS_NOERR) {
-    if(error->next && error->next->code != MS_NOERR) /* (peek ahead) more errors, use delimiter */
-      snprintf(errbuf, 255, "%s: %s %s%s", error->routine, ms_errorCodes[error->code], error->message, delimiter);
-    else
-      snprintf(errbuf, 255, "%s: %s %s", error->routine, ms_errorCodes[error->code], error->message);   
-
-    if((errstr = (char *) realloc(errstr, sizeof(char)*(strlen(errstr)+strlen(errbuf)+1))) == NULL) return(NULL);
-    strcat(errstr, errbuf);
-
+    if((errstr = msAddErrorDisplayString(errstr, error)) == NULL) return(NULL);
+	 
+	if(error->next && error->next->code != MS_NOERR) { /* (peek ahead) more errors, use delimiter */
+		if((errstr = strcatalloc(errstr, delimiter)) == NULL) return(NULL);
+	}
     error = error->next;   
   }
 
@@ -529,6 +590,9 @@ char *msGetVersion() {
 #ifdef USE_GEOS
   strcat(version, " SUPPORTS=GEOS");
 #endif
+#ifdef USE_POINT_Z_M
+  strcat(version, " SUPPORTS=POINT_Z_M");
+#endif
 #ifdef USE_TIFF
   strcat(version, " INPUT=TIFF");
 #endif
@@ -552,12 +616,14 @@ char *msGetVersion() {
 #endif
 #ifdef USE_GDAL
   strcat(version, " INPUT=GDAL");
+#endif
+#ifdef USE_MYGIS
+  strcat(version, " INPUT=MYGIS");
 #endif
   strcat(version, " INPUT=SHAPEFILE");
 #ifdef ENABLE_STDERR_DEBUG
   strcat(version, " DEBUG=MSDEBUG");
 #endif
-
   return(version);
 }
 
