@@ -27,6 +27,15 @@
  ******************************************************************************
  *
  * $Log: mapgeos.cpp,v $
+ * Revision 1.14.2.1  2006/01/18 07:14:38  sdlime
+ * Update GEOS toWKT function to produce a malloc'd pointer. Updated shape.i to produce a new object. (bug 1466)
+ *
+ * Revision 1.14  2005/10/31 04:59:43  sdlime
+ * Now catching exceptions when reading WKT strings.
+ *
+ * Revision 1.13  2005/10/30 05:05:07  sdlime
+ * Initial support for WKT via GEOS. The reader is only integrated via the map file reader, with MapScript, CGI and URL support following ASAP. (bug 1466)
+ *
  * Revision 1.12  2005/06/14 16:03:33  dan
  * Updated copyright date to 2005
  *
@@ -73,7 +82,7 @@
 
 #include "geos.h"
 
-MS_CVSID("$Id: mapgeos.cpp,v 1.12 2005/06/14 16:03:33 dan Exp $")
+MS_CVSID("$Id: mapgeos.cpp,v 1.14.2.1 2006/01/18 07:14:38 sdlime Exp $")
 
 using namespace geos;
 
@@ -463,7 +472,7 @@ static shapeObj *msGEOSGeometry2Shape_multiline(MultiLineString *g)
 
       for(i=0; i<numPoints; i++) {
 	c = coords->getAt(i);
-	
+
 	line.point[i].x = c.x;
 	line.point[i].y = c.y;
 	/* line.point[i].z = c.z; */	
@@ -584,7 +593,8 @@ static shapeObj *msGEOSGeometry2Shape_multipolygon(MultiPolygon *g)
 
 	line.point[i].x = c.x;
 	line.point[i].y = c.y;
-	/* line.point[i].z = c.z; */
+	/* line.point[i].z = c.z;
+	   line.point[i].m = 0; */
       }
       msAddLine(shape, &line);
       free(line.point);
@@ -687,6 +697,60 @@ void msGEOSFreeGeometry(shapeObj *shape)
 #endif
 }
 
+/*
+** WKT input and output functions
+*/
+shapeObj *msGEOSShapeFromWKT(const char *string)
+{
+#ifdef USE_GEOS
+  if(!string) 
+    return NULL;
+
+  /* if there is not an instance of a GeometeryFactory, create one */
+  if(!gf) 
+    msGEOSCreateGeometryFactory();
+
+  try {
+    WKTReader *r = new WKTReader(gf);
+    Geometry *g = r->read(string);
+    return msGEOSGeometry2Shape(g);
+  } catch (GEOSException *ge) {
+    msSetError(MS_GEOSERR, "%s", "msGEOSShapeFromWKT()", (char *) ge->toString().c_str());
+    delete ge;
+    return NULL;
+  } catch (...) {
+    return NULL;
+  }
+
+#else
+  msSetError(MS_GEOSERR, "GEOS support is not available.", "msGEOSShapeFromWKT()");
+  return NULL;
+#endif
+}
+
+char *msGEOSShapeToWKT(shapeObj *shape)
+{
+#ifdef USE_GEOS
+  char *wkt=NULL;
+
+  if(!shape) 
+    return NULL;
+
+  if(!shape->geometry) /* if no geometry for the shape then build one */
+    shape->geometry = msGEOSShape2Geometry(shape);
+  Geometry *g = (Geometry *) shape->geometry;
+  if(!g) 
+    return NULL;
+
+  wkt = strdup((char *) g->toString().c_str());
+
+  return wkt;
+#else
+  msSetError(MS_GEOSERR, "GEOS support is not available.", "msGEOSShapeToWKT()");
+  return NULL;
+#endif
+}
+
 /*
 ** Analytical functions exposed to MapServer/MapScript.
 */
