@@ -27,10 +27,46 @@
  ******************************************************************************
  *
  * $Log: mapgml.c,v $
- * Revision 1.66.2.2  2005/08/01 16:36:14  sdlime
+ * Revision 1.78.2.2  2006/02/16 08:03:47  sdlime
+ * Removed duplicate call to create outer ring list in writeGeometry_GML2().
+ *
+ * Revision 1.78.2.1  2006/01/23 22:41:47  julien
+ * Add gml:lineStringMember in GML2 MultiLineString geometry (bug 1569)
+ *
+ * Revision 1.78  2005/11/21 23:22:15  sdlime
+ * Trivial comment fixes.
+ *
+ * Revision 1.77  2005/11/01 16:55:18  sdlime
+ * Updated the layer loops in the GML writers to repsect layer order.
+ *
+ * Revision 1.76  2005/10/26 21:22:04  sdlime
+ * I changed my mind and removed the option to use the shape index as an ID. I think it is better practice to use a more stable key item. This should also make supporting featureid requests easier since it's just a shortcut for a filter. Normal filter processing does not support index queries.
+ *
+ * Revision 1.75  2005/10/26 21:07:35  sdlime
+ * WFS GML output includes an option to set a gml:id for each feature. The default is no id (current behavior), but this can be overidden using the metadata value gml_featureid, which takes an item name to use to create the id or the special value '_index' to use the shape->index. This partially addresses bug 1413.
+ *
+ * Revision 1.74  2005/10/25 20:29:52  sdlime
+ * Completed work to add constants to GML output. For example gml_constants 'aConstant'  gml_aConstant_value 'this is a constant', which results in output like <aConstant>this is a constant</aConstant>. Constants can appear in groups and can haves pecific types (default is string). Constants are NOT queryable so their use should be limited until some extensions to wfs 1.1 appear that will allow us to mark certain elements as queryable or not in capabilities output.
+ *
+ * Revision 1.73  2005/10/24 21:03:03  sdlime
+ * Removed unused variable geom_name from WFS GML writer.
+ *
+ * Revision 1.72  2005/10/24 20:52:46  sdlime
+ * Moved code to generate the layer_name outside of the feature loop. It was being executed for each feature when it only needed to be run once per layer.
+ *
+ * Revision 1.71  2005/10/24 20:46:46  sdlime
+ * Updated GML transformation metadata handling to use the msOWSLookupMetadata functions with prefixes OWS_, WFS_ and GML_. Previously only the GML_ prefix was supported.
+ *
+ * Revision 1.70  2005/10/12 15:34:27  sdlime
+ * Updated the gmlGroupObj to allow you to set the group type. This impacts schema location. If not set the complex type written by the schema generator is 'groupnameType' and via metadata you can override that (e.g. gml_groupname_type 'MynameType').
+ *
+ * Revision 1.69  2005/09/04 20:20:28  sdlime
+ * GML writes now support multiple ways of defining a multipoint. 1) one line string with multiple points and 2) multiplle line strings with one point each. (bug 1490)
+ *
+ * Revision 1.68  2005/08/01 16:34:28  sdlime
  * Fixed a problem with the GML writer not properly closing the geometry container for MultiPoints. (bug 1409).
  *
- * Revision 1.66.2.1  2005/07/06 15:18:09  dan
+ * Revision 1.67  2005/07/06 15:18:17  dan
  * Fixed missing space in GML MultiLineString output (bug 1408)
  *
  * Revision 1.66  2005/06/14 16:03:33  dan
@@ -117,7 +153,7 @@
 #include "map.h"
 #include "maperror.h"
 
-MS_CVSID("$Id: mapgml.c,v 1.66.2.2 2005/08/01 16:36:14 sdlime Exp $")
+MS_CVSID("$Id: mapgml.c,v 1.78.2.2 2006/02/16 08:03:47 sdlime Exp $")
 
 /* Use only mapgml.c if WMS or WFS is available */
 
@@ -233,21 +269,24 @@ static int gmlWriteGeometry_GML2(FILE *stream, gmlGeometryListObj *geometryList,
     if(geometry_simple_index >= 0) geometry_simple_name = geometryList->geometries[geometry_simple_index].name;
     if(geometry_aggregate_index >= 0) geometry_aggregate_name = geometryList->geometries[geometry_aggregate_index].name;
 
-    if((geometry_simple_index != -1 && shape->line[0].numpoints == 1) ||
+    if((geometry_simple_index != -1 && shape->line[0].numpoints == 1 && shape->numlines == 1) ||
        (geometry_simple_index != -1 && geometry_aggregate_index == -1) ||
-       (geometryList->numgeometries == 0 && shape->line[0].numpoints == 1)) { /* write a Point(s) */
-      for(i=0; i<shape->line[0].numpoints; i++) {
-        gmlStartGeometryContainer(stream, geometry_simple_name, namespace, tab);
+       (geometryList->numgeometries == 0 && shape->line[0].numpoints == 1 && shape->numlines == 1)) { /* write a Point(s) */
 
-        /* Point */
-        if(srsname_encoded)
-          msIO_fprintf(stream, "%s<gml:Point srsName=\"%s\">\n", tab, srsname_encoded);
-        else
-          msIO_fprintf(stream, "%s<gml:Point>\n", tab);
-        msIO_fprintf(stream, "%s  <gml:coordinates>%f,%f</gml:coordinates>\n", tab, shape->line[0].point[i].x, shape->line[0].point[i].y);
-        msIO_fprintf(stream, "%s</gml:Point>\n", tab);
+      for(i=0; i<shape->numlines; i++) {
+	for(j=0; j<shape->line[i].numpoints; j++) {
+	  gmlStartGeometryContainer(stream, geometry_simple_name, namespace, tab);
 
-        gmlEndGeometryContainer(stream, geometry_simple_name, namespace, tab);
+	  /* Point */
+	  if(srsname_encoded)
+	    msIO_fprintf(stream, "%s<gml:Point srsName=\"%s\">\n", tab, srsname_encoded);
+	  else
+	    msIO_fprintf(stream, "%s<gml:Point>\n", tab);
+	  msIO_fprintf(stream, "%s  <gml:coordinates>%f,%f</gml:coordinates>\n", tab, shape->line[i].point[j].x, shape->line[i].point[j].y);
+	  msIO_fprintf(stream, "%s</gml:Point>\n", tab);
+	  
+	  gmlEndGeometryContainer(stream, geometry_simple_name, namespace, tab);
+	}
       }
     } else if((geometry_aggregate_index != -1) || (geometryList->numgeometries == 0)) { /* write a MultiPoint */
       gmlStartGeometryContainer(stream, geometry_aggregate_name, namespace, tab);
@@ -257,15 +296,17 @@ static int gmlWriteGeometry_GML2(FILE *stream, gmlGeometryListObj *geometryList,
         msIO_fprintf(stream, "%s<gml:MultiPoint srsName=\"%s\">\n", tab, srsname_encoded);
       else
         msIO_fprintf(stream, "%s<gml:MultiPoint>\n", tab);
-     
-      for(i=0; i<shape->line[0].numpoints; i++) {
-        msIO_fprintf(stream, "%s  <gml:Point>\n", tab);
-        msIO_fprintf(stream, "%s    <gml:coordinates>%f,%f</gml:coordinates>\n", tab, shape->line[0].point[i].x, shape->line[0].point[i].y);
-        msIO_fprintf(stream, "%s  </gml:Point>\n", tab);
+
+      for(i=0; i<shape->numlines; i++) {
+        for(j=0; j<shape->line[i].numpoints; j++) {
+	  msIO_fprintf(stream, "%s  <gml:Point>\n", tab);
+	  msIO_fprintf(stream, "%s    <gml:coordinates>%f,%f</gml:coordinates>\n", tab, shape->line[i].point[j].x, shape->line[i].point[j].y);
+	  msIO_fprintf(stream, "%s  </gml:Point>\n", tab);
+	}
       }
-
+	
       msIO_fprintf(stream, "%s</gml:MultiPoint>\n", tab);
-
+	
       gmlEndGeometryContainer(stream, geometry_aggregate_name, namespace, tab);
     } else {
       msIO_fprintf(stream, "<!-- Warning: Cannot write geometry- no point/multipoint geometry defined. -->\n");
@@ -309,13 +350,15 @@ static int gmlWriteGeometry_GML2(FILE *stream, gmlGeometryListObj *geometryList,
         msIO_fprintf(stream, "%s<gml:MultiLineString>\n", tab);
 
       for(j=0; j<shape->numlines; j++) {
-        msIO_fprintf(stream, "%s  <gml:LineString>\n", tab); /* no srsname at this point */
+        msIO_fprintf(stream, "%s  <gml:lineStringMember>\n", tab); /* no srsname at this point */
+        msIO_fprintf(stream, "%s    <gml:LineString>\n", tab); /* no srsname at this point */
 	
-        msIO_fprintf(stream, "%s    <gml:coordinates>", tab);
+        msIO_fprintf(stream, "%s      <gml:coordinates>", tab);
         for(i=0; i<shape->line[j].numpoints; i++)
 	  msIO_fprintf(stream, "%f,%f ", shape->line[j].point[i].x, shape->line[j].point[i].y);
         msIO_fprintf(stream, "</gml:coordinates>\n");
-        msIO_fprintf(stream, "%s  </gml:LineString>\n", tab);
+        msIO_fprintf(stream, "%s    </gml:LineString>\n", tab);
+        msIO_fprintf(stream, "%s  </gml:lineStringMember>\n", tab);
       }
       
       msIO_fprintf(stream, "%s</gml:MultiLineString>\n", tab);
@@ -335,9 +378,6 @@ static int gmlWriteGeometry_GML2(FILE *stream, gmlGeometryListObj *geometryList,
     /* get a list of outter rings for this polygon */
     outerlist = msGetOuterList(shape);
   
-    /* get a list of outter rings for this polygon */
-    outerlist = msGetOuterList(shape);
-
     numouters = 0;
     for(i=0; i<shape->numlines; i++)
       if(outerlist[i] == MS_TRUE) numouters++;
@@ -346,7 +386,7 @@ static int gmlWriteGeometry_GML2(FILE *stream, gmlGeometryListObj *geometryList,
        (geometry_simple_index != -1 && geometry_aggregate_index == -1) ||
        (geometryList->numgeometries == 0 && shape->numlines == 1)) { /* write a Polygon(s) */
       for(i=0; i<shape->numlines; i++) {
-        if(outerlist[i] == MS_FALSE) break; /* skip non-outer rings, each outer ring is a new polygon */
+        if(outerlist[i] == MS_FALSE) break; /* skip non-outer rings, each outer ring is a new polygon */	
 	
         /* get a list of inner rings for this polygon */
         innerlist = msGetInnerList(shape, i, outerlist);
@@ -487,21 +527,24 @@ static int gmlWriteGeometry_GML3(FILE *stream, gmlGeometryListObj *geometryList,
     if(geometry_simple_index >= 0) geometry_simple_name = geometryList->geometries[geometry_simple_index].name;
     if(geometry_aggregate_index >= 0) geometry_aggregate_name = geometryList->geometries[geometry_aggregate_index].name;
 
-    if((geometry_simple_index != -1 && shape->line[0].numpoints == 1) ||
+    if((geometry_simple_index != -1 && shape->line[0].numpoints == 1 && shape->numlines == 1) ||
        (geometry_simple_index != -1 && geometry_aggregate_index == -1) ||
-       (geometryList->numgeometries == 0 && shape->line[0].numpoints == 1)) { /* write a Point(s) */
-      for(i=0; i<shape->line[0].numpoints; i++) {
-	gmlStartGeometryContainer(stream, geometry_simple_name, namespace, tab);
-	
-	/* Point */
-	if(srsname_encoded)
-	  msIO_fprintf(stream, "%s  <gml:Point srsName=\"%s\">\n", tab, srsname_encoded);
-	else
-	  msIO_fprintf(stream, "%s  <gml:Point>\n", tab);
-	msIO_fprintf(stream, "%s    <gml:pos>%f %f</gml:pos>\n", tab, shape->line[0].point[i].x, shape->line[0].point[i].y);	
-	msIO_fprintf(stream, "%s  </gml:Point>\n", tab);
+       (geometryList->numgeometries == 0 && shape->line[0].numpoints == 1 && shape->numlines == 1)) { /* write a Point(s) */
 
-	gmlEndGeometryContainer(stream, geometry_simple_name, namespace, tab);
+      for(i=0; i<shape->numlines; i++) {
+	for(j=0; j<shape->line[i].numpoints; j++) {
+	  gmlStartGeometryContainer(stream, geometry_simple_name, namespace, tab);
+	  
+	  /* Point */
+	  if(srsname_encoded)
+	    msIO_fprintf(stream, "%s  <gml:Point srsName=\"%s\">\n", tab, srsname_encoded);
+	  else
+	    msIO_fprintf(stream, "%s  <gml:Point>\n", tab);
+	  msIO_fprintf(stream, "%s    <gml:pos>%f %f</gml:pos>\n", tab, shape->line[i].point[j].x, shape->line[i].point[j].y);	
+	  msIO_fprintf(stream, "%s  </gml:Point>\n", tab);
+	  
+	  gmlEndGeometryContainer(stream, geometry_simple_name, namespace, tab);
+	}
       }
     } else if((geometry_aggregate_index != -1) || (geometryList->numgeometries == 0)) { /* write a MultiPoint */      
       gmlStartGeometryContainer(stream, geometry_aggregate_name, namespace, tab);
@@ -513,13 +556,15 @@ static int gmlWriteGeometry_GML3(FILE *stream, gmlGeometryListObj *geometryList,
         msIO_fprintf(stream, "%s  <gml:MultiPoint>\n", tab);
 
       msIO_fprintf(stream, "%s    <gml:pointMembers>\n", tab);
-      for(i=0; i<shape->line[0].numpoints; i++) {
-        msIO_fprintf(stream, "%s      <gml:Point>\n", tab);
-        msIO_fprintf(stream, "%s        <gml:pos>%f %f</gml:pos>\n", tab, shape->line[0].point[i].x, shape->line[0].point[i].y);
-        msIO_fprintf(stream, "%s      </gml:Point>\n", tab);
+      for(i=0; i<shape->numlines; i++) {
+	for(j=0; j<shape->line[i].numpoints; j++) {
+	  msIO_fprintf(stream, "%s      <gml:Point>\n", tab);
+	  msIO_fprintf(stream, "%s        <gml:pos>%f %f</gml:pos>\n", tab, shape->line[i].point[j].x, shape->line[i].point[j].y);
+	  msIO_fprintf(stream, "%s      </gml:Point>\n", tab);
+	}
       }
       msIO_fprintf(stream, "%s    </gml:pointMembers>\n", tab);
-
+      
       msIO_fprintf(stream, "%s  </gml:MultiPoint>\n", tab);
 
       gmlEndGeometryContainer(stream, geometry_aggregate_name, namespace, tab);
@@ -754,7 +799,8 @@ static int gmlWriteGeometry(FILE *stream, gmlGeometryListObj *geometryList, int
 /*
 ** GML specific metadata handling functions.
 */ 
-int msItemInGroups(gmlItemObj *item, gmlGroupListObj *groupList) 
+
+int msItemInGroups(char *name, gmlGroupListObj *groupList) 
 {
   int i, j;
   gmlGroupObj *group;
@@ -764,7 +810,7 @@ int msItemInGroups(gmlItemObj *item, gmlGroupListObj *groupList)
   for(i=0; i<groupList->numgroups; i++) {
     group = &(groupList->groups[i]);
     for(j=0; j<group->numitems; j++) {
-      if(strcasecmp(item->name, group->items[j]) == 0) return MS_TRUE;
+      if(strcasecmp(name, group->items[j]) == 0) return MS_TRUE;
     }
   }
 
@@ -797,12 +843,12 @@ gmlGeometryListObj *msGMLGetGeometries(layerObj *layer)
   gmlGeometryListObj *geometryList=NULL;
   gmlGeometryObj *geometry=NULL;
 
-  /* allocate memory and iinitialize the item collection */
+  /* allocate memory and initialize the item collection */
   geometryList = (gmlGeometryListObj *) malloc(sizeof(gmlGeometryListObj));
   geometryList->geometries = NULL;
   geometryList->numgeometries = 0;
 
-  if((value = msLookupHashTable(&(layer->metadata), "gml_geometries")) != NULL) {
+  if((value = msOWSLookupMetadata(&(layer->metadata), "OFG", "geometries")) != NULL) {
     names = split(value, ',', &numnames);
 
     /* allocation an array of gmlGeometryObj's */
@@ -817,12 +863,12 @@ gmlGeometryListObj *msGMLGetGeometries(layerObj *layer)
       geometry->occurmin = 0;
       geometry->occurmax = 1;      
 
-      snprintf(tag, 64, "gml_%s_type", names[i]);
-      if((value = msLookupHashTable(&(layer->metadata), tag)) != NULL)
+      snprintf(tag, 64, "%s_type", names[i]);
+      if((value =  msOWSLookupMetadata(&(layer->metadata), "OFG", tag)) != NULL)
 	geometry->type = strdup(value); /* TODO: validate input value */
 
-      snprintf(tag, 64, "gml_%s_occurances", names[i]);
-      if((value = msLookupHashTable(&(layer->metadata), tag)) != NULL) {
+      snprintf(tag, 64, "%s_occurances", names[i]);
+      if((value = msOWSLookupMetadata(&(layer->metadata), "OFG", tag)) != NULL) {
 	char **occur;
 	int numoccur;
 
@@ -877,15 +923,15 @@ gmlItemListObj *msGMLGetItems(layerObj *layer)
   gmlItemObj *item=NULL;
 
   /* get a list of items that should be included in output */
-  if((value = msLookupHashTable(&(layer->metadata), "gml_include_items")) != NULL)  
+  if((value = msOWSLookupMetadata(&(layer->metadata), "OFG", "include_items")) != NULL)  
     incitems = split(value, ',', &numincitems);
 
   /* get a list of items that should be excluded in output */
-  if((value = msLookupHashTable(&(layer->metadata), "gml_exclude_items")) != NULL)  
+  if((value = msOWSLookupMetadata(&(layer->metadata), "OFG", "exclude_items")) != NULL)  
     excitems = split(value, ',', &numexcitems);
 
   /* get a list of items that need don't get encoded */
-  if((value = msLookupHashTable(&(layer->metadata), "gml_xml_items")) != NULL)  
+  if((value = msOWSLookupMetadata(&(layer->metadata), "OFG", "xml_items")) != NULL)  
     xmlitems = split(value, ',', &numxmlitems);
 
   /* allocate memory and iinitialize the item collection */
@@ -931,12 +977,12 @@ gmlItemListObj *msGMLGetItems(layerObj *layer)
 	item->encode = MS_FALSE;
     }
 
-    snprintf(tag, 64, "gml_%s_alias", layer->items[i]);
-    if((value = msLookupHashTable(&(layer->metadata), tag)) != NULL) 
+    snprintf(tag, 64, "%s_alias", layer->items[i]);
+    if((value = msOWSLookupMetadata(&(layer->metadata), "OFG", tag)) != NULL) 
       item->alias = strdup(value);
 
-    snprintf(tag, 64, "gml_%s_type", layer->items[i]);
-    if((value = msLookupHashTable(&(layer->metadata), tag)) != NULL) 
+    snprintf(tag, 64, "%s_type", layer->items[i]);
+    if((value = msOWSLookupMetadata(&(layer->metadata), "OFG", tag)) != NULL) 
       item->type = strdup(value);
   }
 
@@ -997,6 +1043,89 @@ static void msGMLWriteItem(FILE *stream, gmlItemObj *item, char *value, const ch
   return;
 }
 
+gmlConstantListObj *msGMLGetConstants(layerObj *layer)
+{
+  int i;
+  const char *value=NULL;
+  char tag[64];
+
+  char **names=NULL;
+  int numnames=0;
+
+  gmlConstantListObj *constantList=NULL;
+  gmlConstantObj *constant=NULL;
+
+    /* allocate the collection */
+  constantList = (gmlConstantListObj *) malloc(sizeof(gmlConstantListObj)); 
+  constantList->constants = NULL;
+  constantList->numconstants = 0;
+
+  /* list of constants (TODO: make this automatic by parsing metadata) */
+  if((value = msOWSLookupMetadata(&(layer->metadata), "OFG", "constants")) != NULL) {
+    names = split(value, ',', &numnames);
+
+    /* allocation an array of gmlConstantObj's */
+    constantList->numconstants = numnames;
+    constantList->constants = (gmlConstantObj *) malloc(sizeof(gmlConstantObj)*constantList->numconstants);
+
+    for(i=0; i<constantList->numconstants; i++) {
+      constant = &(constantList->constants[i]);
+
+      constant->name = strdup(names[i]); /* initialize a few things */      
+      constant->value = NULL;
+      constant->type = NULL;
+
+      snprintf(tag, 64, "%s_value", constant->name);
+      if((value = msOWSLookupMetadata(&(layer->metadata), "OFG", tag)) != NULL) 
+      constant->value = strdup(value);
+
+      snprintf(tag, 64, "%s_type", constant->name);
+      if((value = msOWSLookupMetadata(&(layer->metadata), "OFG", tag)) != NULL) 
+      constant->type = strdup(value);
+    }
+
+    msFreeCharArray(names, numnames);
+  } 
+  
+  return constantList;
+}
+
+void msGMLFreeConstants(gmlConstantListObj *constantList)
+{
+  int i;
+
+  if(!constantList) return;
+
+  for(i=0; i<constantList->numconstants; i++) {
+    msFree(constantList->constants[i].name);
+    msFree(constantList->constants[i].value);
+    msFree(constantList->constants[i].type);
+  }
+
+  free(constantList);
+}
+
+static void msGMLWriteConstant(FILE *stream, gmlConstantObj *constant, const char *namespace, const char *tab)
+{
+  int add_namespace = MS_TRUE;  
+
+  if(!stream || !constant) return;
+  if(!constant->value) return;
+
+  if(!namespace) add_namespace = MS_FALSE;
+  if(strchr(constant->name, ':') != NULL) add_namespace = MS_FALSE;
+  
+  if(add_namespace == MS_TRUE && msIsXMLTagValid(constant->name) == MS_FALSE)
+    msIO_fprintf(stream, "<!-- WARNING: The value '%s' is not valid in a XML tag context. -->\n", constant->name);
+  
+  if(add_namespace == MS_TRUE)
+    msIO_fprintf(stream, "%s<%s:%s>%s</%s:%s>\n", tab, namespace, constant->name, constant->value, namespace, constant->name);
+  else
+    msIO_fprintf(stream, "%s<%s>%s</%s>\n", tab, constant->name, constant->value, constant->name);
+
+  return;
+}
+
 gmlGroupListObj *msGMLGetGroups(layerObj *layer)
 {
   int i;
@@ -1015,7 +1144,7 @@ gmlGroupListObj *msGMLGetGroups(layerObj *layer)
   groupList->numgroups = 0;
 
   /* list of groups (TODO: make this automatic by parsing metadata) */
-  if((value = msLookupHashTable(&(layer->metadata), "gml_groups")) != NULL) {
+  if((value = msOWSLookupMetadata(&(layer->metadata), "OFG", "groups")) != NULL) {
     names = split(value, ',', &numnames);
 
     /* allocation an array of gmlGroupObj's */
@@ -1028,10 +1157,15 @@ gmlGroupListObj *msGMLGetGroups(layerObj *layer)
       group->name = strdup(names[i]); /* initialize a few things */
       group->items = NULL;
       group->numitems = 0;
-      
-      snprintf(tag, 64, "gml_%s_group", group->name);
-      if((value = msLookupHashTable(&(layer->metadata), tag)) != NULL)
+      group->type = NULL;
+
+      snprintf(tag, 64, "%s_group", group->name);
+      if((value = msOWSLookupMetadata(&(layer->metadata), "OFG", tag)) != NULL)
 	group->items = split(value, ',', &group->numitems);
+
+      snprintf(tag, 64, "%s_type", group->name);
+      if((value = msOWSLookupMetadata(&(layer->metadata), "OFG", tag)) != NULL) 
+      group->type = strdup(value);
     }
 
     msFreeCharArray(names, numnames);
@@ -1049,22 +1183,24 @@ void msGMLFreeGroups(gmlGroupListObj *groupList)
   for(i=0; i<groupList->numgroups; i++) {
     msFree(groupList->groups[i].name);
     msFreeCharArray(groupList->groups[i].items, groupList->groups[i].numitems);
+    msFree(groupList->groups[i].type);
   }
 
   free(groupList);
 }
 
-static void msGMLWriteGroup(FILE *stream, gmlGroupObj *group, shapeObj *shape, gmlItemListObj *itemList, const char *namespace, const char *tab)
+static void msGMLWriteGroup(FILE *stream, gmlGroupObj *group, shapeObj *shape, gmlItemListObj *itemList, gmlConstantListObj *constantList, const char *namespace, const char *tab)
 {
   int i,j;
   int add_namespace = MS_TRUE;
   char *itemtab;
 
   gmlItemObj *item=NULL;
+  gmlConstantObj *constant=NULL;
 
   if(!stream || !group) return;
 
-  /* setup the item tab */
+  /* setup the item/constant tab */
   itemtab = (char *) malloc(sizeof(char)*strlen(tab)+3);
   if(!itemtab) return;
   sprintf(itemtab, "%s  ", tab);
@@ -1077,11 +1213,20 @@ static void msGMLWriteGroup(FILE *stream, gmlGroupObj *group, shapeObj *shape, g
   else
     msIO_fprintf(stream, "%s<%s>\n", tab, group->name);
   
-  /* now the items in the group */
-  for(i=0; i<group->numitems; i++) {
-    for(j=0; j<shape->numvalues; j++) {
+  /* now the items/constants in the group */  
+  for(i=0; i<group->numitems; i++) {    
+    for(j=0; j<constantList->numconstants; j++) {
+      constant = &(constantList->constants[j]);
+      if(strcasecmp(constant->name, group->items[i]) == 0) { 
+	msGMLWriteConstant(stream, constant, namespace, itemtab);
+	break;
+      }
+    }
+    if(j != constantList->numconstants) continue; /* found this one */
+    for(j=0; j<itemList->numitems; j++) {
       item = &(itemList->items[j]);
       if(strcasecmp(item->name, group->items[i]) == 0) { 
+	/* the number of items matches the number of values exactly */
 	msGMLWriteItem(stream, item, shape->values[j], namespace, itemtab);
 	break;
       }
@@ -1111,8 +1256,10 @@ int msGMLWriteQuery(mapObj *map, char *filename, const char *namespaces)
 
   gmlGroupListObj *groupList=NULL;
   gmlItemListObj *itemList=NULL;
+  gmlConstantListObj *constantList=NULL;
   gmlGeometryListObj *geometryList=NULL;
   gmlItemObj *item=NULL;
+  gmlConstantObj *constant=NULL;
 
   msInitShape(&shape);
 
@@ -1141,7 +1288,7 @@ int msGMLWriteQuery(mapObj *map, char *filename, const char *namespaces)
 
   /* step through the layers looking for query results */
   for(i=0; i<map->numlayers; i++) {
-    lp = &(map->layers[i]);
+    lp = &(map->layers[map->layerorder[i]]);
 
     if(lp->dump == MS_TRUE && lp->resultcache && lp->resultcache->numresults > 0) { /* found results */
 
@@ -1162,7 +1309,8 @@ int msGMLWriteQuery(mapObj *map, char *filename, const char *namespaces)
 
       /* populate item and group metadata structures (TODO: test for NULLs here, shouldn't happen) */
       itemList = msGMLGetItems(lp);
-      groupList = msGMLGetGroups(lp);
+      constantList = msGMLGetConstants(lp);
+      groupList = msGMLGetGroups(lp);      
       geometryList = msGMLGetGeometries(lp);
 
       for(j=0; j<lp->resultcache->numresults; j++) {
@@ -1203,14 +1351,22 @@ int msGMLWriteQuery(mapObj *map, char *filename, const char *namespaces)
 #endif
 
 	/* write the item/values */
-	for(k=0; k<lp->numitems; k++) {
+	for(k=0; k<itemList->numitems; k++) {
           item = &(itemList->items[k]);  
-          if(msItemInGroups(item, groupList) == MS_FALSE) 
+          if(msItemInGroups(item->name, groupList) == MS_FALSE) 
 	    msGMLWriteItem(stream, item, shape.values[k], NULL, "\t\t\t");
         }
 
+	/* write the constants */
+	for(k=0; k<constantList->numconstants; k++) {
+          constant = &(constantList->constants[k]);  
+          if(msItemInGroups(constant->name, groupList) == MS_FALSE) 
+	    msGMLWriteConstant(stream, constant, NULL, "\t\t\t");
+        }
+
+	/* write the groups */
 	for(k=0; k<groupList->numgroups; k++)
-	  msGMLWriteGroup(stream, &(groupList->groups[k]), &shape, itemList, NULL, "\t\t\t");
+	  msGMLWriteGroup(stream, &(groupList->groups[k]), &shape, itemList, constantList, NULL, "\t\t\t");
 
 	/* end this feature */
         /* specify a feature name if nothing provided */
@@ -1231,6 +1387,7 @@ int msGMLWriteQuery(mapObj *map, char *filename, const char *namespaces)
       msFree(value);
 
       msGMLFreeGroups(groupList);
+      msGMLFreeConstants(constantList);
       msGMLFreeItems(itemList);
       msGMLFreeGeometries(geometryList);
 
@@ -1269,8 +1426,10 @@ int msGMLWriteWFSQuery(mapObj *map, FILE *stream, int maxfeatures, char *wfs_nam
   
   gmlGroupListObj *groupList=NULL;
   gmlItemListObj *itemList=NULL;
+  gmlConstantListObj *constantList=NULL;
   gmlGeometryListObj *geometryList=NULL;
   gmlItemObj *item=NULL;
+  gmlConstantObj *constant=NULL;
 
   msInitShape(&shape);
 
@@ -1281,16 +1440,12 @@ int msGMLWriteWFSQuery(mapObj *map, FILE *stream, int maxfeatures, char *wfs_nam
   /* step through the layers looking for query results */
   for(i=0; i<map->numlayers; i++) {
 
-    lp = &(map->layers[i]);
+    lp = &(map->layers[map->layerorder[i]]);
 
     if(lp->dump == MS_TRUE && lp->resultcache && lp->resultcache->numresults > 0)  { /* found results */
-      const char *geom_name = "geometry"; /* what should the default be? (in mapwfs.c too) */
-      char *layer_name;
+      char *layerName;      
       const char *value;
-
-      value = msLookupHashTable(&(lp->metadata), "gml_geometry_name");
-      if(value) geom_name = value;
-      /* geom_name = msWFSGetGeomElementName(map, lp); */
+      int featureIdIndex=-1; /* no feature id */
 
       /* actually open the layer */
       status = msLayerOpen(lp);
@@ -1299,12 +1454,33 @@ int msGMLWriteWFSQuery(mapObj *map, FILE *stream, int maxfeatures, char *wfs_nam
       /* retrieve all the item names. (Note : there might be no attributes) */
       status = msLayerGetItems(lp);
       /* if(status != MS_SUCCESS) return(status); */
+      
+      value = msOWSLookupMetadata(&(lp->metadata), "OFG", "featureid");
+      if(value) { /* find the featureid amongst the items for this layer */
+  	for(j=0; j<lp->numitems; j++) {
+	  if(strcasecmp(lp->items[j], value) == 0) { /* found it */
+	    featureIdIndex = j;
+	    break;
+	  } 
+	}
+      }
 
       /* populate item and group metadata structures (TODO: test for NULLs here, shouldn't happen) */
       itemList = msGMLGetItems(lp);
+      constantList = msGMLGetConstants(lp);
       groupList = msGMLGetGroups(lp);
       geometryList = msGMLGetGeometries(lp);
 
+      /* set the layer name */
+      /* value = msOWSLookupMetadata(&(lp->metadata), "OFG", "layername");
+	 if(!value) value = lp->name; */
+      if (wfs_namespace) {	
+	layerName = (char *) malloc(strlen(wfs_namespace)+strlen(lp->name)+2);
+	sprintf(layerName, "%s:%s", wfs_namespace, lp->name);
+      } else {	
+	layerName = strdup(lp->name);
+      }
+
       for(j=0; j<lp->resultcache->numresults; j++) {
 	status = msLayerGetShape(lp, &shape, lp->resultcache->results[j].tileindex, lp->resultcache->results[j].shapeindex);
         if(status != MS_SUCCESS) return(status);
@@ -1315,18 +1491,17 @@ int msGMLWriteWFSQuery(mapObj *map, FILE *stream, int maxfeatures, char *wfs_nam
           msProjectShape(&lp->projection, &map->projection, &shape);
 #endif
         
-	/* start this feature */
-        if (wfs_namespace) {
-	  layer_name = (char *) malloc(strlen(wfs_namespace)+strlen(lp->name)+2);
-	  sprintf(layer_name, "%s:%s", wfs_namespace, lp->name);
-        } else
-	  layer_name = strdup(lp->name);
+	/* 
+	** start this feature 
+	*/	
 
 	msIO_fprintf(stream, "    <gml:featureMember>\n");
-        if(msIsXMLTagValid(layer_name) == MS_FALSE)
-            msIO_fprintf(stream, "<!-- WARNING: The value '%s' is not valid in a XML tag context. -->\n", layer_name);
-        msIO_fprintf(stream, "      <%s>\n", layer_name);
-
+        if(msIsXMLTagValid(layerName) == MS_FALSE)
+            msIO_fprintf(stream, "<!-- WARNING: The value '%s' is not valid in a XML tag context. -->\n", layerName);
+        if(featureIdIndex != -1)
+          msIO_fprintf(stream, "      <%s gml:id=\"%s\">\n", layerName, shape.values[featureIdIndex]);
+        else
+	  msIO_fprintf(stream, "      <%s>\n", layerName);
 
 	/* write the bounding box */
 	if(msOWSGetEPSGProj(&(map->projection), &(map->web.metadata), "FGO", MS_TRUE)) /* use the map projection first*/
@@ -1350,32 +1525,39 @@ int msGMLWriteWFSQuery(mapObj *map, FILE *stream, int maxfeatures, char *wfs_nam
 #endif
 
 	/* write the item/values */
-	for(k=0; k<lp->numitems; k++) {
+	for(k=0; k<itemList->numitems; k++) {
           item = &(itemList->items[k]);  
-          if(msItemInGroups(item, groupList) == MS_FALSE) 
+          if(msItemInGroups(item->name, groupList) == MS_FALSE) 
 	    msGMLWriteItem(stream, item, shape.values[k], wfs_namespace, "        ");
         }
 
+        /* write the constants */
+	for(k=0; k<constantList->numconstants; k++) {
+          constant = &(constantList->constants[k]);  
+          if(msItemInGroups(constant->name, groupList) == MS_FALSE) 
+	    msGMLWriteConstant(stream, constant, wfs_namespace, "        ");
+        }
+
+	/* write the groups */
 	for(k=0; k<groupList->numgroups; k++)
-	  msGMLWriteGroup(stream, &(groupList->groups[k]), &shape, itemList, wfs_namespace, "        ");
+	  msGMLWriteGroup(stream, &(groupList->groups[k]), &shape, itemList, constantList, wfs_namespace, "        ");
 
 	/* end this feature */
-        msIO_fprintf(stream, "      </%s>\n", layer_name);
-
+        msIO_fprintf(stream, "      </%s>\n", layerName);
 	msIO_fprintf(stream, "    </gml:featureMember>\n");
 
-        msFree(layer_name);
-
-	msFreeShape(&shape); /* init too */
+  	msFreeShape(&shape); /* init too */
 
          features++;
          if (maxfeatures > 0 && features == maxfeatures)
-           break;
-         
-         /* end this layer */
-      }
+           break; 
+      }      
+
+      /* done with this layer, do a little clean-up */      
+      msFree(layerName);
 
       msGMLFreeGroups(groupList);
+      msGMLFreeConstants(constantList);
       msGMLFreeItems(itemList);
       msGMLFreeGeometries(geometryList);
 
@@ -1395,4 +1577,3 @@ int msGMLWriteWFSQuery(mapObj *map, FILE *stream, int maxfeatures, char *wfs_nam
     return MS_FAILURE;
 #endif /* USE_WFS_SVR */
 }
-
