@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mapgraticule.c,v 1.13 2005/06/08 23:57:43 dan Exp $
+ * $Id: mapgraticule.c,v 1.15.2.2 2006/01/16 20:41:22 sdlime Exp $
  *
  * Name:     mapgraticule.c
  * Project:  MapServer
@@ -29,6 +29,18 @@
  * DEALINGS IN THE SOFTWARE.
  **********************************************************************
  * $Log: mapgraticule.c,v $
+ * Revision 1.15.2.2  2006/01/16 20:41:22  sdlime
+ * Fixed error with image legends (shifted text) introduced by the 1449 bug fix. (bug 1607)
+ *
+ * Revision 1.15.2.1  2006/01/11 04:52:33  sdlime
+ * Argh! Friggin' typos on my part. Fixed bug 1256.
+ *
+ * Revision 1.15  2005/11/30 21:42:43  julien
+ * Use MIN/MAXINTERVAL value when we define grid layers (bug1530)
+ *
+ * Revision 1.14  2005/10/28 01:09:41  jani
+ * MS RFC 3: Layer vtable architecture (bug 1477)
+ *
  * Revision 1.13  2005/06/08 23:57:43  dan
  * Propagate msGetlabelSize() errors in msGraticuleLayerNextShape() (part of
  * bug 828)
@@ -60,9 +72,10 @@
  **********************************************************************/
 
 #include "map.h"
+#include <assert.h>
 #include "mapproject.h"
 
-MS_CVSID("$Id: mapgraticule.c,v 1.13 2005/06/08 23:57:43 dan Exp $")
+MS_CVSID("$Id: mapgraticule.c,v 1.15.2.2 2006/01/16 20:41:22 sdlime Exp $")
 
 /**********************************************************************************************************************
  *
@@ -87,6 +100,8 @@ void DefineAxis( int iTickCountTarget, double *Min, double *Max, double *Inc );
 static int _AdjustLabelPosition( layerObj *pLayer, shapeObj *pShape, msGraticulePosition ePosition );
 static void _FormatLabel( layerObj *pLayer, shapeObj *pShape, double dDataToFormat );
 
+int msGraticuleLayerInitItemInfo(layerObj *layer);
+
 #define MAPGRATICULE_ARC_SUBDIVISION_DEFAULT	(256)
 #define MAPGRATICULE_ARC_MINIMUM				 (16)
 #define MAPGRATICULE_FORMAT_STRING_DEFAULT		"%5.2g"
@@ -127,7 +142,7 @@ int msGraticuleLayerOpen(layerObj *layer)
     pInfo->labelformat = (char *) malloc( strlen( MAPGRATICULE_FORMAT_STRING_DDMM ) + 1 );
     pInfo->ilabeltype = (int) lpDDMM;
     strcpy( pInfo->labelformat, MAPGRATICULE_FORMAT_STRING_DDMM );
-  } else if( strcmp( pInfo->labelformat, "DDMM" )   == 0 ) {
+  } else if( strcmp( pInfo->labelformat, "DD" )   == 0 ) {
     pInfo->labelformat = (char *) malloc( strlen( MAPGRATICULE_FORMAT_STRING_DD ) + 1 );
     pInfo->ilabeltype = (int) lpDD;
     strcpy( pInfo->labelformat, MAPGRATICULE_FORMAT_STRING_DD );
@@ -188,6 +203,21 @@ int msGraticuleLayerWhichShapes(layerObj *layer, rectObj rect)
 	pInfo->dendlongitude			= rect.maxx;
 	pInfo->bvertical				= 1;
 	pInfo->extent					= rect;
+	if( pInfo->minincrement > 0.0 )
+        {
+            pInfo->dincrementlongitude			= pInfo->minincrement;
+            pInfo->dincrementlatitude			= pInfo->minincrement;
+        }
+        else if( pInfo->maxincrement > 0.0 )
+        {
+            pInfo->dincrementlongitude			= pInfo->maxincrement;
+            pInfo->dincrementlatitude			= pInfo->maxincrement;
+        }
+        else
+        {
+            pInfo->dincrementlongitude			= 0;
+            pInfo->dincrementlatitude			= 0;
+        }
 	
 	if( pInfo->maxarcs > 0 )
 		iAxisTickCount				= (int) pInfo->maxarcs;
@@ -573,6 +603,37 @@ int msGraticuleLayerGetAutoStyle(mapObj *map, layerObj *layer, classObj *c, int
 	return MS_SUCCESS;
 }
 
+/**********************************************************************************************************************
+ *
+ */
+int
+msGraticuleLayerInitializeVirtualTable(layerObj *layer)
+{
+    assert(layer != NULL);
+    assert(layer->vtable != NULL);
+
+    layer->vtable->LayerInitItemInfo = msGraticuleLayerInitItemInfo;
+    layer->vtable->LayerFreeItemInfo = msGraticuleLayerFreeItemInfo;
+    layer->vtable->LayerOpen = msGraticuleLayerOpen;
+    layer->vtable->LayerIsOpen = msGraticuleLayerIsOpen;
+    layer->vtable->LayerWhichShapes = msGraticuleLayerWhichShapes;
+    layer->vtable->LayerNextShape = msGraticuleLayerNextShape;
+    layer->vtable->LayerGetShape = msGraticuleLayerGetShape;
+
+    layer->vtable->LayerClose = msGraticuleLayerClose;
+    layer->vtable->LayerGetItems = msGraticuleLayerGetItems;
+    layer->vtable->LayerGetExtent = msGraticuleLayerGetExtent;
+    layer->vtable->LayerGetAutoStyle = msGraticuleLayerGetAutoStyle;
+
+    /* layer->vtable->LayerCloseConnection, use default */;
+    layer->vtable->LayerSetTimeFilter = msLayerMakePlainTimeFilter;
+    /* layer->vtable->LayerApplyFilterToLayer, use default */
+    /* layer->vtable->LayerCreateItems, use default */
+    /* layer->vtable->LayerGetNumFeatures, use default */
+
+    return MS_SUCCESS;
+}
+
 /**********************************************************************************************************************
  *
  */
@@ -634,7 +695,7 @@ static int _AdjustLabelPosition( layerObj *pLayer, shapeObj *pShape, msGraticule
     if( pLayer->transform ) 
 		msTransformShapeToPixel( pShape, pLayer->map->extent, pLayer->map->cellsize );
 
-	if (msGetLabelSize( pShape->text, &pLayer->class[0].label, &rectLabel, &pLayer->map->fontset, 1.0 ) != 0)
+	if (msGetLabelSize( pShape->text, &pLayer->class[0].label, &rectLabel, &pLayer->map->fontset, 1.0, MS_FALSE) != 0)
         return MS_FAILURE;  /* msSetError already called */
 
 	switch( ePosition )
@@ -727,7 +788,10 @@ void DefineAxis( int iTickCountTarget, double *Min, double *Max, double *Inc )
 
 	/* compute candidate for increment */
 
+
 	Test_inc = pow( 10.0, ceil( log10( Range/10 ) ) ) ;
+        if( *Inc > 0 && ( Test_inc < *Inc || Test_inc > *Inc ) )
+            Test_inc=*Inc;
 
 	/* establish maximum scale value... */
 
@@ -735,7 +799,6 @@ void DefineAxis( int iTickCountTarget, double *Min, double *Max, double *Inc )
 
 	if( Test_max < *Max )
 		Test_max += Test_inc ;
-
 	/* establish minimum scale value... */
 
 	Test_min = Test_max ;
