@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: maphttp.c,v 1.19 2005/05/13 17:23:34 dan Exp $
+ * $Id: maphttp.c,v 1.21 2005/07/15 13:37:47 frank Exp $
  *
  * Project:  MapServer
  * Purpose:  Utility functions to access files via HTTP (requires libcurl)
@@ -27,6 +27,12 @@
  * DEALINGS IN THE SOFTWARE.
  **********************************************************************
  * $Log: maphttp.c,v $
+ * Revision 1.21  2005/07/15 13:37:47  frank
+ * Acquire TLOCK_OWS around whole func in msHTTPCleanup().
+ *
+ * Revision 1.20  2005/07/14 21:43:26  hobu
+ * Fix bug 1417 msHTTPInit() not ever called
+ *
  * Revision 1.19  2005/05/13 17:23:34  dan
  * First pass at properly handling XML exceptions from CONNECTIONTYPE WMS
  * layers. Still needs some work. (bug 1246)
@@ -98,8 +104,9 @@
 #include "map.h"
 #include "maperror.h"
 #include "mapows.h"
+#include "mapthread.h"
 
-MS_CVSID("$Id: maphttp.c,v 1.19 2005/05/13 17:23:34 dan Exp $")
+MS_CVSID("$Id: maphttp.c,v 1.21 2005/07/15 13:37:47 frank Exp $")
 
 #include <time.h>
 #ifndef _WIN32
@@ -135,9 +142,12 @@ int msHTTPInit()
      * many threads or libcurl sessions that'll be used) by every
      * application that uses libcurl.
      */
+     
+    msAcquireLock(TLOCK_OWS);
     if (!gbCurlInitialized && 
         curl_global_init(CURL_GLOBAL_ALL) != 0)
     {
+        msReleaseLock(TLOCK_OWS);
         msSetError(MS_HTTPERR, "Libcurl initialization failed.", 
                    "msHTTPInit()");
         return MS_FAILURE;
@@ -145,6 +155,7 @@ int msHTTPInit()
 
     gbCurlInitialized = MS_TRUE;
 
+    msReleaseLock(TLOCK_OWS);
     return MS_SUCCESS;
 }
 
@@ -155,10 +166,12 @@ int msHTTPInit()
  **********************************************************************/
 void msHTTPCleanup()
 {
+    msAcquireLock(TLOCK_OWS);
     if (gbCurlInitialized)
         curl_global_cleanup();
-
+        
     gbCurlInitialized = MS_FALSE;
+    msReleaseLock(TLOCK_OWS);
 }
 
 
@@ -177,6 +190,9 @@ void msHTTPCleanup()
 void msHTTPInitRequestObj(httpRequestObj *pasReqInfo, int numRequests) 
 {
     int i;
+    if (!gbCurlInitialized)
+        msHTTPInit();
+  
     for(i=0; i<numRequests; i++)
     {
         pasReqInfo[i].pszGetUrl = NULL;
