@@ -27,6 +27,12 @@
  ******************************************************************************
  *
  * $Log: mapio.c,v $
+ * Revision 1.8  2005/07/25 20:56:00  frank
+ * Undef msIO_fread() too.
+ *
+ * Revision 1.7  2005/07/22 17:26:11  frank
+ * bug 1259: fixed POST support in fastcgi mode
+ *
  * Revision 1.6  2004/11/04 21:13:02  frank
  * ensure we include io.h on win32
  *
@@ -56,7 +62,7 @@
 #include <io.h>
 #endif
 
-MS_CVSID("$Id: mapio.c,v 1.6 2004/11/04 21:13:02 frank Exp $")
+MS_CVSID("$Id: mapio.c,v 1.8 2005/07/25 20:56:00 frank Exp $")
 
 static int is_msIO_initialized = MS_FALSE;
 
@@ -74,6 +80,7 @@ static void msIO_Initialize( void );
 #  undef msIO_printf
 #  undef msIO_fprintf
 #  undef msIO_fwrite
+#  undef msIO_fread
 #  undef msIO_vfprintf
 #endif
 
@@ -263,6 +270,22 @@ int msIO_fwrite( const void *data, size_t size, size_t nmemb, FILE *fp )
         return msIO_contextWrite( context, data, size * nmemb ) / size;
 }
 
+/************************************************************************/
+/*                            msIO_fread()                              */
+/************************************************************************/
+
+int msIO_fread( void *data, size_t size, size_t nmemb, FILE *fp )
+
+{
+    msIOContext *context;
+
+    context = msIO_getHandler( fp );
+    if( context == NULL )
+        return fread( data, size, nmemb, fp );
+    else
+        return msIO_contextRead( context, data, size * nmemb ) / size;
+}
+
 /* ==================================================================== */
 /* ==================================================================== */
 /*      Internal default callbacks implementing stdio reading and       */
