@@ -27,6 +27,15 @@
  ******************************************************************************
  *
  * $Log: maplabel.c,v $
+ * Revision 1.79.2.1  2006/01/16 20:41:22  sdlime
+ * Fixed error with image legends (shifted text) introduced by the 1449 bug fix. (bug 1607)
+ *
+ * Revision 1.79  2005/11/28 04:24:59  sdlime
+ * Changed msAddLabel() to use msCopyStyle() rather than doing on its own. This should fix some de-allocation errors folks have been having. (bug 1398)
+ *
+ * Revision 1.78  2005/08/31 18:30:03  sdlime
+ * Applied patch suggested to adjust TTF baselines (bug 1449).
+ *
  * Revision 1.77  2005/06/14 16:03:33  dan
  * Updated copyright date to 2005
  *
@@ -55,7 +64,7 @@
 
 #include "map.h"
 
-MS_CVSID("$Id: maplabel.c,v 1.77 2005/06/14 16:03:33 dan Exp $")
+MS_CVSID("$Id: maplabel.c,v 1.79.2.1 2006/01/16 20:41:22 sdlime Exp $")
 
 int msAddLabel(mapObj *map, int layerindex, int classindex, int shapeindex, int tileindex, pointObj *point, char *string, double featuresize, labelObj *label )
 {
@@ -106,14 +115,13 @@ int msAddLabel(mapObj *map, int layerindex, int classindex, int shapeindex, int
   cachePtr->styles = NULL;
   cachePtr->numstyles = 0;
   if((layerPtr->type == MS_LAYER_ANNOTATION && classPtr->numstyles > 0) || layerPtr->type == MS_LAYER_POINT) {
-    cachePtr->styles = (styleObj *) malloc(sizeof(styleObj)*classPtr->numstyles);
-    memcpy(cachePtr->styles, classPtr->styles, sizeof(styleObj)*classPtr->numstyles);
     cachePtr->numstyles = classPtr->numstyles;
-
-    /* copy the style symbol name */
-    if (classPtr->numstyles > 0) {      
-      for(i=0; i<classPtr->numstyles; i++)
-        if (classPtr->styles[i].symbolname) cachePtr->styles[i].symbolname = strdup(classPtr->styles[i].symbolname);
+    cachePtr->styles = (styleObj *) malloc(sizeof(styleObj)*classPtr->numstyles);
+    if (classPtr->numstyles > 0) {
+      for(i=0; i<classPtr->numstyles; i++) {
+	initStyle(&(cachePtr->styles[i]));
+	msCopyStyle(&(cachePtr->styles[i]), &(classPtr->styles[i]));
+      }
     }
   }
 
@@ -280,7 +288,8 @@ int msLoadFontSet(fontSetObj *fontset, mapObj *map)
 ** LL corner of the text to be rendered, this is first line for TrueType fonts.
 */
 
-int msGetLabelSize(char *string, labelObj *label, rectObj *rect, fontSetObj *fontset, double scalefactor) /* assumes an angle of 0 */
+ /* assumes an angle of 0 regardless of the settings in the labelObj */
+int msGetLabelSize(char *string, labelObj *label, rectObj *rect, fontSetObj *fontset, double scalefactor, int adjustBaseline)
 {
   int size;
 
@@ -302,7 +311,6 @@ int msGetLabelSize(char *string, labelObj *label, rectObj *rect, fontSetObj *fon
       return(-1);
     }
 
-
     error = gdImageStringFT(NULL, bbox, 0, font, size, 0, 0, 0, string);
     if(error) {
       msSetError(MS_TTFERR, error, "msGetLabelSize()");
@@ -313,6 +321,12 @@ int msGetLabelSize(char *string, labelObj *label, rectObj *rect, fontSetObj *fon
     rect->miny = bbox[5];
     rect->maxx = bbox[2];
     rect->maxy = bbox[1];
+
+    // bug 1449 fix (adjust baseline)
+    if(adjustBaseline) {
+      label->offsety += MS_NINT(((bbox[5] + bbox[1]) + size) / 2);
+      label->offsetx += MS_NINT(bbox[0] / 2); // optional?
+    }
 #else
     msSetError(MS_TTFERR, "TrueType font support is not available.", "msGetLabelSize()");
     return(-1);
@@ -601,7 +615,7 @@ int msImageTruetypePolyline(symbolSetObj *symbolset, gdImagePtr img, shapeObj *p
   label.outlinecolor = style->outlinecolor;
   label.antialias = symbol->antialias;
   
-  if(msGetLabelSize(symbol->character, &label, &label_rect, symbolset->fontset, scalefactor) == -1)
+  if(msGetLabelSize(symbol->character, &label, &label_rect, symbolset->fontset, scalefactor, MS_FALSE) == -1)
     return(-1);
 
   label_width = (int) label_rect.maxx - (int) label_rect.minx;
