@@ -27,6 +27,21 @@
  ******************************************************************************
  *
  * $Log: maplegend.c,v $
+ * Revision 1.57.2.2  2006/03/27 05:50:52  sdlime
+ * Fixed symbol initialization error with embedded scalebars and legends. (bug 1725)
+ *
+ * Revision 1.57.2.1  2006/01/16 20:41:22  sdlime
+ * Fixed error with image legends (shifted text) introduced by the 1449 bug fix. (bug 1607)
+ *
+ * Revision 1.57  2005/11/17 05:56:30  sdlime
+ * Updated msDrawLegend to respect class min/max scale values. (bug 1524)
+ *
+ * Revision 1.56  2005/10/31 06:03:26  sdlime
+ * Updated msDrawLegend() to consider layer order. (bug 1484)
+ *
+ * Revision 1.55  2005/09/14 00:11:35  frank
+ * fixed leak of imageObj when embedding legends
+ *
  * Revision 1.54  2005/06/14 16:03:33  dan
  * Updated copyright date to 2005
  *
@@ -59,7 +74,7 @@
 
 #include "map.h"
 
-MS_CVSID("$Id: maplegend.c,v 1.54 2005/06/14 16:03:33 dan Exp $")
+MS_CVSID("$Id: maplegend.c,v 1.57.2.2 2006/03/27 05:50:52 sdlime Exp $")
 
 #define PSF .8
 #define VMARGIN 5 /* margin at top and bottom of legend graphic */
@@ -255,7 +270,7 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
   int maxwidth=0, maxheight=0, n=0;
   int *heights;
   rectObj rect;
-  imageObj      *image = NULL;
+  imageObj *image = NULL;
   outputFormatObj *format = NULL;
 
   if (!scale_independent) {
@@ -270,11 +285,13 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
   ** allocate heights array
   */
   for(i=0; i<map->numlayers; i++) {
-    if((map->layers[i].status == MS_OFF) || (map->layers[i].type == MS_LAYER_QUERY)) /* skip it */
+    lp = &(map->layers[map->layerorder[i]]);
+
+    if((lp->status == MS_OFF) || (lp->type == MS_LAYER_QUERY)) /* skip it */
       continue;
 
-    for(j=0;j<map->layers[i].numclasses;j++) {
-      if(!map->layers[i].class[j].name)
+    for(j=0;j<lp->numclasses;j++) {
+      if(!lp->class[j].name)
 	continue; /* skip it */
       n++;
     }
@@ -290,20 +307,30 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
   */
   n=0;
   for(i=0; i<map->numlayers; i++) { /* Need to find the longest legend label string */
-    if((map->layers[i].status == MS_OFF) || (map->layers[i].type == MS_LAYER_QUERY)) /* skip it */
+    lp = &(map->layers[map->layerorder[i]]);
+
+    if((lp->status == MS_OFF) || (lp->type == MS_LAYER_QUERY)) /* skip it */
       continue;
 
-   if(!scale_independent && map->scale > 0) {
-      if((map->layers[i].maxscale > 0) && (map->scale > map->layers[i].maxscale))
+    if(!scale_independent && map->scale > 0) {
+      if((lp->maxscale > 0) && (map->scale > lp->maxscale))
 	continue;
-      if((map->layers[i].minscale > 0) && (map->scale <= map->layers[i].minscale))
+      if((lp->minscale > 0) && (map->scale <= lp->minscale))
 	continue;
     }
  
-    for(j=0;j<map->layers[i].numclasses;j++) {
-      if(!map->layers[i].class[j].name)
+    for(j=0;j<lp->numclasses;j++) {
+      if(!lp->class[j].name)
 	continue; /* skip it */
-      if(msGetLabelSize(map->layers[i].class[j].name, &map->legend.label, &rect, &(map->fontset), 1.0) != 0)
+
+      if(!scale_independent && map->scale > 0) {  /* verify class scale here */
+	if((lp->class[j].maxscale > 0) && (map->scale > lp->class[j].maxscale))
+	  continue;
+	if((lp->class[j].minscale > 0) && (map->scale <= lp->class[j].minscale))
+	  continue;
+      }
+
+      if(msGetLabelSize(lp->class[j].name, &map->legend.label, &rect, &(map->fontset), 1.0, MS_FALSE) != 0)
 	return(NULL); /* something bad happened */
       maxheight = MS_MAX(maxheight, MS_NINT(rect.maxy - rect.miny));
       maxwidth = MS_MAX(maxwidth, MS_NINT(rect.maxx - rect.minx));
@@ -346,8 +373,7 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
     
   /* for(i=0; i<map->numlayers; i++) { */
   for(i=map->numlayers-1; i>=0; i--) {
-
-    lp = &(map->layers[i]); /* assign for brevity */
+    lp = &(map->layers[map->layerorder[i]]); /* for brevity */
 
     if((lp->numclasses == 0) || (lp->status == MS_OFF) || (lp->type == MS_LAYER_QUERY))
       continue; /* skip this layer */
@@ -362,7 +388,14 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
     for(j=0; j<lp->numclasses; j++) { /* always at least 1 class */
 
       if(!lp->class[j].name) continue; /* skip it */
-      
+     
+      if(!scale_independent && map->scale > 0) {  /* verify class scale here */
+        if((lp->class[j].maxscale > 0) && (map->scale > lp->class[j].maxscale))
+          continue;
+        if((lp->class[j].minscale > 0) && (map->scale <= lp->class[j].minscale))
+          continue;
+      }
+ 
       pnt.x = HMARGIN + map->legend.keysizex + map->legend.keyspacingx;
       
       /* TODO */
@@ -400,12 +433,20 @@ int msEmbedLegend(mapObj *map, gdImagePtr img)
       gdImageDestroy(map->symbolset.symbol[s].img);
   }
   
+  /* render the legend. */
   image = msDrawLegend(map, MS_FALSE);
-  map->symbolset.symbol[s].img = image->img.gd; /* TODO  */
+
+  /* steal the gdImage and free the rest of the imageObj */
+  map->symbolset.symbol[s].img = image->img.gd; 
+  image->img.gd = NULL;
+  msFreeImage( image );
+
   if(!map->symbolset.symbol[s].img) return(-1); /* something went wrong creating scalebar */
 
   map->symbolset.symbol[s].type = MS_SYMBOL_PIXMAP; /* intialize a few things */
   map->symbolset.symbol[s].name = strdup("legend");  
+  map->symbolset.symbol[s].sizex = map->symbolset.symbol[s].img->sx;
+  map->symbolset.symbol[s].sizey = map->symbolset.symbol[s].img->sy;
 
   /* I'm not too sure this test is sufficient ... NFW. */
   if(map->legend.transparent == MS_ON)
