@@ -28,6 +28,12 @@
  ******************************************************************************
  *
  * $Log: mapobject.c,v $
+ * Revision 1.23  2005/10/26 17:40:03  frank
+ * Avoid unused variable warning.
+ *
+ * Revision 1.22  2005/09/23 16:20:44  hobu
+ * add meatier error message for invalid extents
+ *
  * Revision 1.21  2005/02/18 03:06:46  dan
  * Turned all C++ (//) comments into C comments (bug 1238)
  *
@@ -100,7 +106,7 @@
 #  include "cpl_conv.h"
 #endif
 
-MS_CVSID("$Id: mapobject.c,v 1.21 2005/02/18 03:06:46 dan Exp $")
+MS_CVSID("$Id: mapobject.c,v 1.23 2005/10/26 17:40:03 frank Exp $")
 
 void freeWeb(webObj *web);
 void freeScalebar(scalebarObj *scalebar);
@@ -285,7 +291,8 @@ int msMapSetExtent( mapObj *map,
     map->extent.maxy = maxy;
     
     if (!MS_VALID_EXTENT(map->extent)) {
-        msSetError(MS_MISCERR, "Given map extent is invalid.", "setExtent()"); 
+        msSetError(MS_MISCERR, "Given map extent is invalid. Check that it " \
+        "is in the form: minx, miny, maxx, maxy", "setExtent()"); 
         return MS_FAILURE;
     }
       
@@ -717,8 +724,9 @@ int msSetLayersdrawingOrder(mapObj *self, int *panIndexes)
 int msMapLoadOWSParameters(mapObj *map, cgiRequestObj *request,
                            const char *wmtver)
 {
-    int version;
 #ifdef USE_WMS_SVR
+    int version;
+
     version = msOWSParseVersionString(wmtver);
     return msWMSLoadGetMapParams(map, version, request->ParamNames,
                                  request->ParamValues, request->NumParams);
