@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mapogcfilter.h,v 1.11 2005/05/12 17:38:54 assefa Exp $
+ * $Id: mapogcfilter.h,v 1.14 2005/10/29 02:03:43 jani Exp $
  *
  * Name:     mapogcfilter.h
  * Project:  MapServer
@@ -29,6 +29,15 @@
  * DEALINGS IN THE SOFTWARE.
  **********************************************************************
  * $Log: mapogcfilter.h,v $
+ * Revision 1.14  2005/10/29 02:03:43  jani
+ * MS RFC 8: Pluggable External Feature Layer Providers (bug 1477).
+ *
+ * Revision 1.13  2005/10/28 01:09:42  jani
+ * MS RFC 3: Layer vtable architecture (bug 1477)
+ *
+ * Revision 1.12  2005/07/20 13:55:34  assefa
+ * Added support of case insensitive matChase attribute.
+ *
  * Revision 1.11  2005/05/12 17:38:54  assefa
  * prototype for FLTGetShape changed : Now parsing unit value.
  *
@@ -86,40 +95,13 @@
 #include "cpl_minixml.h"
 
 
-typedef enum 
-{
-    FILTER_NODE_TYPE_UNDEFINED = -1,
-    FILTER_NODE_TYPE_LOGICAL = 0,
-    FILTER_NODE_TYPE_SPATIAL = 1,
-    FILTER_NODE_TYPE_COMPARISON = 2,
-    FILTER_NODE_TYPE_PROPERTYNAME = 3,
-    FILTER_NODE_TYPE_BBOX = 4,
-    FILTER_NODE_TYPE_LITERAL = 5,
-    FILTER_NODE_TYPE_BOUNDARY = 6,
-    FILTER_NODE_TYPE_GEOMETRY_POINT = 7,
-    FILTER_NODE_TYPE_GEOMETRY_LINE = 8,
-    FILTER_NODE_TYPE_GEOMETRY_POLYGON = 9
-} FilterNodeType;
-
-
-typedef struct _FilterNode
-{
-    FilterNodeType      eType;
-    char                *pszValue;
-    void                *pOther;
-
-    struct _FilterNode  *psLeftNode;
-    struct _FilterNode  *psRightNode;
-
-      
-}FilterEncodingNode;
-
 
 typedef struct
 {
-    char *pszWildCard;
-    char *pszSingleChar;
-    char *pszEscapeChar;
+      char *pszWildCard;
+      char *pszSingleChar;
+      char *pszEscapeChar;
+      int  bCaseInsensitive;
 }FEPropertyIsLike;
 
 /* -------------------------------------------------------------------- */
@@ -129,6 +111,12 @@ FilterEncodingNode *FLTParseFilterEncoding(char *szXMLString);
 FilterEncodingNode *FLTCreateFilterEncodingNode(void);
 int FLTApplyFilterToLayer(FilterEncodingNode *psNode, mapObj *map, 
                          int iLayerIndex, int bOnlySpatialFilter);
+
+int FLTLayerApplyCondSQLFilterToLayer(FilterEncodingNode *psNode, mapObj *map, 
+                                     int iLayerIndex, int bOnlySpatialFilter);
+int FLTLayerApplyPlainFilterToLayer(FilterEncodingNode *psNode, mapObj *map, 
+                                   int iLayerIndex, int bOnlySpatialFilter);
+
 int FLTApplySpatialFilterToLayer(FilterEncodingNode *psNode, mapObj *map, 
                                  int iLayerIndex);
 
