@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mapogr.cpp,v 1.87 2005/06/14 16:03:33 dan Exp $
+ * $Id: mapogr.cpp,v 1.93.2.2 2006/01/18 04:40:56 frank Exp $
  *
  * Name:     mapogr.cpp
  * Project:  MapServer
@@ -29,6 +29,30 @@
  * DEALINGS IN THE SOFTWARE.
  **********************************************************************
  * $Log: mapogr.cpp,v $
+ * Revision 1.93.2.2  2006/01/18 04:40:56  frank
+ * finished implementation of ogr shape2wkt
+ *
+ * Revision 1.93.2.1  2006/01/17 02:31:31  frank
+ * fixed ogr wkt support - bug 1614
+ *
+ * Revision 1.93  2005/11/01 19:38:13  frank
+ * Avoid using _2D geometry API if built with old GDAL/OGR.
+ *
+ * Revision 1.92  2005/11/01 05:35:50  frank
+ * added preliminary implementation of OGR based WKT translation, still untested
+ *
+ * Revision 1.91  2005/11/01 02:34:24  frank
+ * Make a few things static, trim log.
+ *
+ * Revision 1.90  2005/10/28 02:08:14  frank
+ * a few mapogr.cpp funcs used from WFS code, exposed in map.h
+ *
+ * Revision 1.89  2005/10/28 01:09:42  jani
+ * MS RFC 3: Layer vtable architecture (bug 1477)
+ *
+ * Revision 1.88  2005/06/27 19:52:22  frank
+ * Avoid leak of "0 length" point arrays in geometry.
+ *
  * Revision 1.87  2005/06/14 16:03:33  dan
  * Updated copyright date to 2005
  *
@@ -102,87 +126,6 @@
  * Revision 1.66  2004/01/06 21:18:02  julien
  * Make STYLEITEM AUTO work with an empty CLASS (Bug 531)
  *
- * Revision 1.65  2003/11/09 20:19:09  frank
- * fixed support for 3D multipoint datasets
- *
- * Revision 1.64  2003/10/17 18:51:25  dan
- * Open OGR files relative to shapepath first, then try relative to mapfile
- * path if not found. (bug 295)
- *
- * Revision 1.63  2003/10/03 13:38:53  frank
- * fix support for wkbLineString25D in ogrGeomLine()
- *
- * Revision 1.62  2003/05/25 18:48:06  dan
- * Fixed test on connection value in msOGRFileOpen()
- *
- * Revision 1.61  2003/03/26 17:04:34  frank
- * check layer->debug before reporting debug output
- *
- * Revision 1.60  2003/03/25 17:43:57  dan
- * Handle ogr-brush-1 (no fill) properly with STYLEITEM AUTO
- *
- * Revision 1.59  2003/03/05 16:13:05  frank
- * added support for layers derived from ExecuteSQL()
- *
- * Revision 1.58  2002/12/13 00:57:31  dan
- * Modified WFS implementation to behave more as a real vector data source
- *
- * Revision 1.57  2002/11/17 17:47:51  frank
- * use msTryBuildPath() for open statement
- *
- * Revision 1.56  2002/09/23 13:33:26  julien
- * Swapped map_path for mappath for consistency.
- *
- * Revision 1.55  2002/09/17 13:08:28  julien
- * Remove all chdir() function and replace them with the new msBuildPath function.
- * This have been done to make MapServer thread safe. (Bug 152)
- *
- * Revision 1.54  2002/08/16 20:50:10  julien
- * Fixed for new styleObj except for styleitem auto
- *
- * Revision 1.53  2002/08/14 17:30:44  dan
- * Fixed problem with numeric labels with STYLEITEM AUTO (bug 185)
- *
- * Revision 1.52  2002/07/11 18:38:57  frank
- * allow layer names as well as layer indexes in connection string.
- *
- * Revision 1.51  2002/04/26 18:00:35  julien
- * Enabled OGRMultiPoint in ogrGeomLine and ogrGeomPoint
- *
- * Revision 1.50  2002/04/04 19:05:41  frank
- * improve error reporting
- *
- * Revision 1.49  2002/03/26 21:00:04  frank
- * Set the tileindex value in shapeObjs.  Support fetching based on tile+recordid.
- * Support refetching features for autostyling in tiled layers.
- *
- * Revision 1.48  2002/03/26 16:53:12  dan
- * Fixed msDebug() format string in msOGRFileClose()
- *
- * Revision 1.47  2002/03/18 20:40:14  frank
- * initial pass on tiled ogr support
- *
- * Revision 1.46  2002/02/22 22:56:26  dan
- * Avoid filling an object with a pen and no brush in layers of type POLYGON
- * when using auto styling.
- *
- * Revision 1.45  2002/02/20 22:28:23  frank
- * fixed some problems with 3D geometry support
- *
- * Revision 1.44  2002/02/20 19:49:22  frank
- * fixed memory leak of OGRFeatures under some circumstanes
- *
- * Revision 1.43  2002/02/08 21:29:18  frank
- * ensure 3D geometries supported as well as 2D geometries
- *
- * Revision 1.42  2002/01/15 01:00:51  dan
- * Attempt at fixing thick polygon outline problem (bug 92).  Also rewrote
- * the symbol mapping for brushes and pens at the same time to use the same
- * rules as point symbols.
- *
- * Revision 1.41  2002/01/09 05:10:28  frank
- * avoid use of ms_error global
- *
  * ...
  *
  * Revision 1.1  2000/08/25 18:41:05  dan
@@ -190,16 +133,18 @@
  *
  **********************************************************************/
 
+#include <assert.h>
 #include "map.h"
 #include "mapproject.h"
 
 #if defined(USE_OGR) || defined(USE_GDAL)
 #  include "ogr_spatialref.h"
+#  include "gdal_version.h"
 #  include "cpl_conv.h"
 #  include "cpl_string.h"
 #endif
 
-MS_CVSID("$Id: mapogr.cpp,v 1.87 2005/06/14 16:03:33 dan Exp $")
+MS_CVSID("$Id: mapogr.cpp,v 1.93.2.2 2006/01/18 04:40:56 frank Exp $")
 
 #ifdef USE_OGR
 
@@ -221,6 +166,11 @@ typedef struct ms_ogr_file_info_t
 
 } msOGRFileInfo;
 
+static int msOGRLayerIsOpen(layerObj *layer);
+static int msOGRLayerInitItemInfo(layerObj *layer);
+static int msOGRLayerGetAutoStyle(mapObj *map, layerObj *layer, classObj *c, 
+                                  int tile, long record);
+
 // Undefine this if you are using a very old GDAL without OpenShared(). 
 #define USE_SHARED_ACCESS  
 
@@ -373,7 +323,7 @@ static int ogrGeomPoints(OGRGeometry *poGeom, shapeObj *outshp)
  * ------------------------------------------------------------------ */
   lineObj *line = outshp->line + outshp->numlines-1;
 
-  if( line->numpoints == 0 )
+  if( line->point == NULL )
       line->point = (pointObj *) malloc(sizeof(pointObj) * numpoints);
   else
       line->point = (pointObj *) 
@@ -664,8 +614,11 @@ int msOGRGeometryToShape(OGRGeometryH hGeometry, shapeObj *psShape,
         else if (nType == wkbPolygon)
           return ogrConvertGeometry((OGRGeometry *)hGeometry,
                                     psShape,  MS_LAYER_POLYGON);
+        else
+            return MS_FAILURE;
     }
-        return MS_FALSE;
+    else
+        return MS_FAILURE;
 }
 
 
@@ -782,8 +735,8 @@ static char **msOGRGetValues(layerObj *layer, OGRFeature *poFeature)
  *
  * Returns MS_SUCCESS/MS_FAILURE
  **********************************************************************/
-int msOGRSpatialRef2ProjectionObj(OGRSpatialReference *poSRS,
-                                  projectionObj *proj, int debug_flag )
+static int msOGRSpatialRef2ProjectionObj(OGRSpatialReference *poSRS,
+                                         projectionObj *proj, int debug_flag )
 {
 #ifdef USE_PROJ
   // First flush the "auto" name from the projargs[]... 
@@ -1627,6 +1580,16 @@ int msOGRLayerOpen(layerObj *layer, const char *pszOverrideConnection)
 #endif /* USE_OGR */
 }
 
+/**********************************************************************
+ *                     msOGRLayerOpenVT()
+ *
+ * Overloaded version of msOGRLayerOpen for virtual table architecture
+ **********************************************************************/
+static int msOGRLayerOpenVT(layerObj *layer) 
+{
+	return msOGRLayerOpen(layer, NULL);
+}
+
 /**********************************************************************
  *                     msOGRLayerClose()
  **********************************************************************/
@@ -1660,7 +1623,7 @@ int msOGRLayerClose(layerObj *layer)
 /**********************************************************************
  *                     msOGRLayerIsOpen()
  **********************************************************************/
-int msOGRLayerIsOpen(layerObj *layer) 
+static int msOGRLayerIsOpen(layerObj *layer) 
 {
 #ifdef USE_OGR
   if (layer->ogrlayerinfo)
@@ -1770,7 +1733,7 @@ int msOGRLayerGetItems(layerObj *layer)
  *
  * Init the itemindexes array after items[] has been reset in a layer.
  **********************************************************************/
-int msOGRLayerInitItemInfo(layerObj *layer)
+static int msOGRLayerInitItemInfo(layerObj *layer)
 {
 #ifdef USE_OGR
   msOGRFileInfo *psInfo =(msOGRFileInfo*)layer->ogrlayerinfo;
@@ -1938,7 +1901,8 @@ int msOGRLayerNextShape(layerObj *layer, shapeObj *shape)
  *
  * Returns MS_SUCCESS/MS_FAILURE
  **********************************************************************/
-int msOGRLayerGetShape(layerObj *layer, shapeObj *shape, int tile, long record)
+int msOGRLayerGetShape(layerObj *layer, shapeObj *shape, int tile, 
+                       long record)
 {
 #ifdef USE_OGR
   msOGRFileInfo *psInfo =(msOGRFileInfo*)layer->ogrlayerinfo;
@@ -2078,8 +2042,8 @@ static int msOGRGetSymbolId(symbolSetObj *symbolset, const char *pszSymbolId,
  * The returned classObj is a ref. to a static structure valid only until
  * the next call and that shouldn't be freed by the caller.
  **********************************************************************/
-int msOGRLayerGetAutoStyle(mapObj *map, layerObj *layer, classObj *c,
-                           int tile, long record)
+static int msOGRLayerGetAutoStyle(mapObj *map, layerObj *layer, classObj *c,
+                                  int tile, long record)
 {
 #ifdef USE_OGR
   msOGRFileInfo *psInfo =(msOGRFileInfo*)layer->ogrlayerinfo;
@@ -2165,6 +2129,8 @@ int msOGRLayerGetAutoStyle(mapObj *map, layerObj *layer, classObj *c,
               c->label.angle = poLabelStyle->Angle(bIsNull);
 
               c->label.size = (int)poLabelStyle->Size(bIsNull);
+              if( c->label.size < 1 ) /* no point dropping to zero size */
+                  c->label.size = 1;
 
               // msDebug("** Label size=%d, angle=%f, string=%s **\n", c->label.size, c->label.angle, poLabelStyle->TextString(bIsNull));
 
@@ -2384,3 +2350,173 @@ void msOGRCleanup( void )
 #endif
 }
 
+/************************************************************************/
+/*                  msOGRLayerInitializeVirtualTable()                  */
+/************************************************************************/
+int
+msOGRLayerInitializeVirtualTable(layerObj *layer)
+{
+    assert(layer != NULL);
+    assert(layer->vtable != NULL);
+
+    layer->vtable->LayerInitItemInfo = msOGRLayerInitItemInfo;
+    layer->vtable->LayerFreeItemInfo = msOGRLayerFreeItemInfo;
+    layer->vtable->LayerOpen = msOGRLayerOpenVT;
+    layer->vtable->LayerIsOpen = msOGRLayerIsOpen;
+    layer->vtable->LayerWhichShapes = msOGRLayerWhichShapes;
+    layer->vtable->LayerNextShape = msOGRLayerNextShape;
+    layer->vtable->LayerGetShape = msOGRLayerGetShape;
+
+    layer->vtable->LayerClose = msOGRLayerClose;
+    layer->vtable->LayerGetItems = msOGRLayerGetItems;
+    layer->vtable->LayerGetExtent = msOGRLayerGetExtent;
+    layer->vtable->LayerGetAutoStyle = msOGRLayerGetAutoStyle;
+
+    /* layer->vtable->LayerCloseConnection, use default */
+
+    layer->vtable->LayerApplyFilterToLayer = msLayerApplyCondSQLFilterToLayer;
+    
+    layer->vtable->LayerSetTimeFilter = msLayerMakeBackticsTimeFilter;
+    /* layer->vtable->LayerCreateItems, use default */
+    /* layer->vtable->LayerGetNumFeatures, use default */
+
+
+    return MS_SUCCESS;
+}
+
+/************************************************************************/
+/*                         msOGRShapeFromWKT()                          */
+/************************************************************************/
+shapeObj *msOGRShapeFromWKT(const char *string)
+{
+#ifdef USE_OGR
+    OGRGeometryH hGeom = NULL;
+    shapeObj *shape=NULL;
+    
+    if(!string) 
+        return NULL;
+    
+    if( OGR_G_CreateFromWkt( (char **)&string, NULL, &hGeom ) != OGRERR_NONE )
+    {
+        msSetError(MS_OGRERR, "Failed to parse WKT string.", 
+                   "msOGRShapeFromWKT()" );
+        return NULL;
+    }
+
+    /* Initialize a corresponding shapeObj */
+
+    shape = (shapeObj *) malloc(sizeof(shapeObj));
+    msInitShape(shape);
+
+    /* translate WKT into an OGRGeometry. */
+  
+    if( msOGRGeometryToShape( hGeom, shape, 
+                              wkbFlatten(OGR_G_GetGeometryType(hGeom)) )
+                              == MS_FAILURE )
+    {
+        free( shape );
+        return NULL;
+    }
+
+    OGR_G_DestroyGeometry( hGeom );
+
+    return shape;
+#else
+  msSetError(MS_OGRERR, "OGR support is not available.","msOGRShapeFromWKT()");
+  return NULL;
+#endif
+}
+
+/************************************************************************/
+/*                          msOGRShapeToWKT()                           */
+/************************************************************************/
+char *msOGRShapeToWKT(shapeObj *shape)
+{
+#ifdef USE_OGR
+    OGRGeometryH hGeom = NULL;
+    int          i;
+    char        *wkt = NULL;
+
+    if(!shape) 
+        return NULL;
+
+    if( shape->type == MS_SHAPE_POINT && shape->numlines == 1
+        && shape->line[0].numpoints == 1 )
+    {
+        hGeom = OGR_G_CreateGeometry( wkbPoint );
+#if GDAL_VERSION_NUM >= 1310
+        OGR_G_SetPoint_2D( hGeom, 0, 
+                           shape->line[0].point[0].x, 
+                           shape->line[0].point[0].y );
+#else
+        OGR_G_SetPoint( hGeom, 0, 
+                        shape->line[0].point[0].x, 
+                        shape->line[0].point[0].y,
+                        0.0 );
+#endif
+    }
+    else if( shape->type == MS_SHAPE_LINE && shape->numlines == 1 )
+    {
+        hGeom = OGR_G_CreateGeometry( wkbLineString );
+        for( i = 0; i < shape->line[0].numpoints; i++ )
+        {
+#if GDAL_VERSION_NUM >= 1310
+            OGR_G_AddPoint_2D( hGeom, 
+                               shape->line[0].point[i].x, 
+                               shape->line[0].point[i].y );
+#else
+            OGR_G_AddPoint( hGeom, 
+                            shape->line[0].point[i].x, 
+                            shape->line[0].point[i].y,
+                            0.0 );
+#endif
+        }
+    }
+    else if( shape->type == MS_SHAPE_POLYGON )
+    {
+        int iLine;
+
+        /* actually, it is pretty hard to be sure rings 1+ are interior */
+        hGeom = OGR_G_CreateGeometry( wkbPolygon );
+        for( iLine = 0; iLine < shape->numlines; iLine++ )
+        {
+            OGRGeometryH hRing;
+            hRing = OGR_G_CreateGeometry( wkbLinearRing );
+            
+            for( i = 0; i < shape->line[iLine].numpoints; i++ )
+            {
+#if GDAL_VERSION_NUM >= 1310
+                OGR_G_AddPoint_2D( hRing, 
+                                   shape->line[iLine].point[i].x, 
+                                   shape->line[iLine].point[i].y );
+#else
+                OGR_G_AddPoint( hRing, 
+                                shape->line[iLine].point[i].x, 
+                                shape->line[iLine].point[i].y, 
+                                0.0 );
+#endif
+            }
+            OGR_G_AddGeometryDirectly( hGeom, hRing );
+        }
+    }
+    else
+    {
+        msSetError(MS_OGRERR, "OGR support is not available.", "msOGRShapeToWKT()");
+    }
+
+    if( hGeom != NULL )
+    {
+        char *pszOGRWkt;
+
+        OGR_G_ExportToWkt( hGeom, &pszOGRWkt );
+        wkt = strdup( pszOGRWkt );
+        CPLFree( pszOGRWkt );
+    }
+
+    return wkt;
+#else
+    msSetError(MS_OGRERR, "OGR support is not available.", "msOGRShapeToWKT()");
+    return NULL;
+#endif
+}
+
