@@ -27,6 +27,9 @@
  ******************************************************************************
  *
  * $Log: mapoutput.c,v $
+ * Revision 1.35  2005/11/10 15:41:16  frank
+ * Added missing function header.
+ *
  * Revision 1.34  2005/04/07 17:23:16  assefa
  * Remove #ifdef USE_SVG. It was added during development.
  *
@@ -139,7 +142,7 @@
 #include <assert.h>
 #include "map.h"
 
-MS_CVSID("$Id: mapoutput.c,v 1.34 2005/04/07 17:23:16 assefa Exp $")
+MS_CVSID("$Id: mapoutput.c,v 1.35 2005/11/10 15:41:16 frank Exp $")
 
 static outputFormatObj *msAllocOutputFormat( mapObj *map, const char *name, 
                                              const char *driver );
@@ -906,6 +909,10 @@ void msGetOutputFormatMimeListGD( mapObj *map, char **mime_list, int max_mime )
 }
 
 
+/************************************************************************/
+/*                  msGetOutputFormatMimeListRaster()                   */
+/************************************************************************/
+
 void msGetOutputFormatMimeListRaster( mapObj *map, char **mime_list, int max_mime )
 
 {
