@@ -5,7 +5,7 @@
  * Author:   Daniel Morissette, DM Solutions Group (morissette@dmsolutions.ca)
  *
  ******************************************************************************
- * Copyright (c) 1996-2004 Regents of the University of Minnesota.
+ * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,6 +27,24 @@
  ******************************************************************************
  *
  * $Log: mapows.c,v $
+ * Revision 1.60  2005/06/14 16:03:34  dan
+ * Updated copyright date to 2005
+ *
+ * Revision 1.59  2005/05/24 18:52:45  julien
+ * Bug 1149: From WMS 1.1.1, SRS are given in individual tags.
+ *
+ * Revision 1.58  2005/04/12 23:13:20  sean
+ * use non static strings for temp values in msOWSGetLayerExtent() and msOWSGetEPSGProj() (bug 1311).
+ *
+ * Revision 1.57  2005/02/25 21:32:12  frank
+ * fix msOWSGetLayerExtent() for bug 1118
+ *
+ * Revision 1.56  2005/02/18 03:06:46  dan
+ * Turned all C++ (//) comments into C comments (bug 1238)
+ *
+ * Revision 1.55  2004/12/21 15:59:14  dan
+ * Do not include port number in online resource if it's http/80 (bug 1075)
+ *
  * Revision 1.54  2004/11/28 02:51:02  frank
  * blow an error if any WxS services are enabled without PROJ
  *
@@ -218,7 +236,7 @@
 #include <stdarg.h> 
 #include <assert.h>
 
-MS_CVSID("$Id: mapows.c,v 1.54 2004/11/28 02:51:02 frank Exp $")
+MS_CVSID("$Id: mapows.c,v 1.60 2005/06/14 16:03:34 dan Exp $")
 
 #if defined(USE_WMS_SVR) || defined (USE_WFS_SVR) || defined (USE_WCS_SVR)
 
@@ -283,7 +301,7 @@ int msOWSMakeAllLayersUnique(mapObj *map)
 {
   int i, j;
 
-  // Make sure all layers in the map file have valid and unique names
+  /* Make sure all layers in the map file have valid and unique names */
   for(i=0; i<map->numlayers; i++)
   {
       int count=1;
@@ -303,7 +321,7 @@ int msOWSMakeAllLayersUnique(mapObj *map)
           }
       }
 
-      // Don't forget to rename the first layer if duplicates were found
+      /* Don't forget to rename the first layer if duplicates were found */
       if (count > 1 && msRenameLayer(&(map->layers[i]), 1) != MS_SUCCESS)
       {
           return MS_FAILURE;
@@ -340,7 +358,7 @@ char * msOWSTerminateOnlineResource(const char *src_url)
 
     strcpy(online_resource, src_url);
 
-    // Append trailing '?' or '&' if missing.
+    /* Append trailing '?' or '&' if missing. */
     if (strchr(online_resource, '?') == NULL)
         strcat(online_resource, "?");
     else
@@ -369,11 +387,11 @@ char * msOWSGetOnlineResource(mapObj *map, const char *namespaces, const char *m
     const char *value;
     char *online_resource = NULL;
 
-    // We need this script's URL, including hostname.
-    // Default to use the value of the "onlineresource" metadata, and if not
-    // set then build it: "http://$(SERVER_NAME):$(SERVER_PORT)$(SCRIPT_NAME)?"
-    // (+append the map=... param if it was explicitly passed in QUERY_STRING)
-    //
+    /* We need this script's URL, including hostname. */
+    /* Default to use the value of the "onlineresource" metadata, and if not */
+    /* set then build it: "http://$(SERVER_NAME):$(SERVER_PORT)$(SCRIPT_NAME)?" */
+    /* (+append the map=... param if it was explicitly passed in QUERY_STRING) */
+    /*  */
     if ((value = msOWSLookupMetadata(&(map->web.metadata), namespaces, metadata_name))) 
     {
         online_resource = msOWSTerminateOnlineResource(value);
@@ -387,8 +405,8 @@ char * msOWSGetOnlineResource(mapObj *map, const char *namespaces, const char *m
         port = getenv("SERVER_PORT");
         script = getenv("SCRIPT_NAME");
 
-        // HTTPS is set by Apache to "on" in an HTTPS server ... if not set
-        // then check SERVER_PORT: 443 is the default https port.
+        /* HTTPS is set by Apache to "on" in an HTTPS server ... if not set */
+        /* then check SERVER_PORT: 443 is the default https port. */
         if ( ((value=getenv("HTTPS")) && strcasecmp(value, "on") == 0) ||
              ((value=getenv("SERVER_PORT")) && atoi(value) == 443) )
         {
@@ -415,7 +433,12 @@ char * msOWSGetOnlineResource(mapObj *map, const char *namespaces, const char *m
             online_resource = (char*)malloc(sizeof(char)*(strlen(hostname)+strlen(port)+strlen(script)+mapparam_len+10));
             if (online_resource) 
             {
-                sprintf(online_resource, "%s://%s:%s%s?", protocol, hostname, port, script);
+                if ((atoi(port) == 80 && strcmp(protocol, "http") == 0) ||
+                    (atoi(port) == 443 && strcmp(protocol, "https") == 0) )
+                    sprintf(online_resource, "%s://%s%s?", protocol, hostname, script);
+                else
+                    sprintf(online_resource, "%s://%s:%s%s?", protocol, hostname, port, script);
+
                 if (mapparam)
                 {
                     int baselen;
@@ -457,7 +480,7 @@ const char *msOWSGetSchemasLocation(mapObj *map)
     schemas_location = msLookupHashTable(&(map->web.metadata), 
                                          "ows_schemas_location");
     if (schemas_location == NULL)
-      schemas_location = "http://schemas.opengeospatial.net";//"..";
+      schemas_location = "http://schemas.opengeospatial.net"; /* ".."; */
 
     return schemas_location;
 }
@@ -577,8 +600,8 @@ const char *msOWSLookupMetadata(hashTableObj *metadata,
                 buf[2] = 'l';
                 break;
               default:
-                // We should never get here unless an invalid code (typo) is
-                // present in the code, but since this happened before...
+                /* We should never get here unless an invalid code (typo) is */
+                /* present in the code, but since this happened before... */
                 msSetError(MS_WMSERR, 
                            "Unsupported metadata namespace code (%c).",
                            "msOWSLookupMetadata()", *namespaces );
@@ -822,7 +845,7 @@ int msOWSPrintURLType(FILE *stream, hashTableObj *metadata,
 
     metadata_name = (char*)malloc(strlen(name)*sizeof(char)+10);
 
-    // Get type
+    /* Get type */
     if(type_format != NULL)
     {
         sprintf(metadata_name, "%s_type", name);
@@ -836,7 +859,7 @@ int msOWSPrintURLType(FILE *stream, hashTableObj *metadata,
         }
     }
 
-    // Get width
+    /* Get width */
     if(width_format != NULL)
     {
         sprintf(metadata_name, "%s_width", name);
@@ -850,7 +873,7 @@ int msOWSPrintURLType(FILE *stream, hashTableObj *metadata,
         }
     }
 
-    // Get height
+    /* Get height */
     if(height_format != NULL)
     {
         sprintf(metadata_name, "%s_height", name);
@@ -864,7 +887,7 @@ int msOWSPrintURLType(FILE *stream, hashTableObj *metadata,
         }
     }
 
-    // Get format
+    /* Get format */
     if(urlfrmt_format != NULL)
     {
         sprintf(metadata_name, "%s_format", name);
@@ -878,7 +901,7 @@ int msOWSPrintURLType(FILE *stream, hashTableObj *metadata,
         }
     }
 
-    // Get href
+    /* Get href */
     if(href_format != NULL)
     {
         sprintf(metadata_name, "%s_href", name);
@@ -1112,6 +1135,51 @@ int msOWSPrintEncodeMetadataList(FILE *stream, hashTableObj *metadata,
     return MS_FALSE;
 }
 
+/* msOWSPrintEncodeParamList()
+**
+** Same as msOWSPrintEncodeMetadataList() but applied to mapfile parameters.
+**/
+int msOWSPrintEncodeParamList(FILE *stream, const char *name, 
+                              const char *value, int action_if_not_found, 
+                              char delimiter, const char *startTag, 
+                              const char *endTag, const char *format, 
+                              const char *default_value) 
+{
+    int status = MS_NOERR;
+    char *encoded;
+    char **items = NULL;
+    int numitems = 0, i;
+
+    if(value && strlen(value) > 0)
+        items = split(value, delimiter, &numitems);
+    else
+    {
+        if (action_if_not_found == OWS_WARN)
+        {
+            msIO_fprintf(stream, "<!-- WARNING: Mandatory mapfile parameter '%s' was missing in this context. -->\n", name);
+            status = action_if_not_found;
+        }
+
+        if (default_value)
+            items = split(default_value, delimiter, &numitems);
+    }
+
+    if(items && numitems > 0)
+    {
+        if(startTag) msIO_fprintf(stream, "%s", startTag);
+        for(i=0; i<numitems; i++)
+        {
+            encoded = msEncodeHTMLEntities(items[i]);
+            msIO_fprintf(stream, format, encoded);
+            msFree(encoded);
+        }
+        if(endTag) msIO_fprintf(stream, "%s", endTag);
+        msFreeCharArray(items, numitems);
+    }
+
+    return status;
+}
+
 /*
 ** msOWSPrintLatLonBoundingBox()
 **
@@ -1192,9 +1260,9 @@ void msOWSPrintContactInfo( FILE *stream, const char *tabspace,
 {
   int bEnableContact = 0;
 
-  // contact information is a required element in 1.0.7 but the
-  // sub-elements such as ContactPersonPrimary, etc. are not!
-  // In 1.1.0, ContactInformation becomes optional.
+  /* contact information is a required element in 1.0.7 but the */
+  /* sub-elements such as ContactPersonPrimary, etc. are not! */
+  /* In 1.1.0, ContactInformation becomes optional. */
   if (nVersion > OWS_1_0_0) 
   {
     if(msOWSLookupMetadata(metadata, namespaces, "contactperson") ||
@@ -1206,8 +1274,8 @@ void msOWSPrintContactInfo( FILE *stream, const char *tabspace,
           bEnableContact = 1;
       }
 
-      // ContactPersonPrimary is optional, but when present then all its 
-      // sub-elements are mandatory
+      /* ContactPersonPrimary is optional, but when present then all its  */
+      /* sub-elements are mandatory */
       msIO_fprintf(stream, "%s  <ContactPersonPrimary>\n", tabspace);
 
       msOWSPrintEncodeMetadata(stream, metadata, namespaces, "contactperson", 
@@ -1248,8 +1316,8 @@ void msOWSPrintContactInfo( FILE *stream, const char *tabspace,
           bEnableContact = 1;
       }
 
-      // ContactAdress is optional, but when present then all its 
-      // sub-elements are mandatory
+      /* ContactAdress is optional, but when present then all its  */
+      /* sub-elements are mandatory */
       msIO_fprintf(stream, "%s  <ContactAddress>\n", tabspace);
 
       msOWSPrintEncodeMetadata(stream, metadata, namespaces,"addresstype", OWS_WARN,
@@ -1337,7 +1405,7 @@ void msOWSPrintContactInfo( FILE *stream, const char *tabspace,
 */
 int msOWSGetLayerExtent(mapObj *map, layerObj *lp, const char *namespaces, rectObj *ext)
 {
-  static const char *value;
+  const char *value;
 
   if ((value = msOWSLookupMetadata(&(lp->metadata), namespaces, "extent")) != NULL)
   {
@@ -1358,7 +1426,7 @@ int msOWSGetLayerExtent(mapObj *map, layerObj *lp, const char *namespaces, rectO
     msFreeCharArray(tokens, n);
     return MS_SUCCESS;
   }
-  else
+  else if( lp->type != MS_LAYER_RASTER )
   {
       return msLayerGetExtent(lp, ext);
   }
@@ -1378,7 +1446,7 @@ int msOWSExecuteRequests(httpRequestObj *pasReqInfo, int numRequests,
 {
     int nStatus, iReq;
 
-    // Execute requests
+    /* Execute requests */
 #if defined(USE_WMS_LYR) || defined(USE_WFS_LYR)
     nStatus = msHTTPExecuteRequests(pasReqInfo, numRequests, bCheckLocalCache);
 #else
@@ -1387,8 +1455,8 @@ int msOWSExecuteRequests(httpRequestObj *pasReqInfo, int numRequests,
     return MS_FAILURE;
 #endif
 
-    // Scan list of layers and call the handler for each layer type to
-    // pass them the request results.
+    /* Scan list of layers and call the handler for each layer type to */
+    /* pass them the request results. */
     for(iReq=0; iReq<numRequests; iReq++)
     {
         if (pasReqInfo[iReq].nLayerId >= 0 && 
@@ -1443,11 +1511,11 @@ void msOWSProcessException(layerObj *lp, const char *pszFname,
         pszBuf[nBufSize] = '\0';
 
 
-        // OK, got the data in the buffer.  Look for the <Message> tags
-        if ((strstr(pszBuf, "<WFS_Exception>") &&            // WFS style
+        /* OK, got the data in the buffer.  Look for the <Message> tags */
+        if ((strstr(pszBuf, "<WFS_Exception>") &&            /* WFS style */
              (pszStart = strstr(pszBuf, "<Message>")) &&
              (pszEnd = strstr(pszStart, "</Message>")) ) ||
-            (strstr(pszBuf, "<ServiceExceptionReport>") &&   // WMS style
+            (strstr(pszBuf, "<ServiceExceptionReport>") &&   /* WMS style */
              (pszStart = strstr(pszBuf, "<ServiceException>")) &&
              (pszEnd = strstr(pszStart, "</ServiceException>")) ))
         {
@@ -1534,14 +1602,14 @@ char *msOWSBuildURLFilename(const char *pszPath, const char *pszURL,
 const char *msOWSGetEPSGProj(projectionObj *proj, hashTableObj *metadata, const char *namespaces, int bReturnOnlyFirstOne)
 {
   static char epsgCode[20] ="";
-  static char *value;
+  char *value;
 
-  // metadata value should already be in format "EPSG:n" or "AUTO:..."
+  /* metadata value should already be in format "EPSG:n" or "AUTO:..." */
   if (metadata && ((value = (char *) msOWSLookupMetadata(metadata, namespaces, "srs")) != NULL)) {
     
     if (!bReturnOnlyFirstOne) return value;
 
-    // caller requested only first projection code
+    /* caller requested only first projection code */
     strncpy(epsgCode, value, 19);
     epsgCode[19] = '\0';
     if ((value=strchr(epsgCode, ' ')) != NULL) *value = '\0';
