@@ -1,10 +1,26 @@
 /**********************************************************************
- * $Id: mapows.h,v 1.59 2005/06/02 20:32:01 sdlime Exp $
+ * $Id: mapows.h,v 1.64 2006/01/05 16:25:10 assefa Exp $
  *
  * mapows.h - OGC Web Services (WMS, WFS, WCS) support function definitions
  *
  **********************************************************************
  * $Log: mapows.h,v $
+ * Revision 1.64  2006/01/05 16:25:10  assefa
+ * Correct mapscript windows build problem when flag USE_WMS_SVR was
+ * not set (Bug 1529)
+ *
+ * Revision 1.63  2005/10/25 20:29:52  sdlime
+ * Completed work to add constants to GML output. For example gml_constants 'aConstant'  gml_aConstant_value 'this is a constant', which results in output like <aConstant>this is a constant</aConstant>. Constants can appear in groups and can havespecific types (default is string). Constants are NOT queryable so their use should be limited untilsome extensions to wfs 1.1 appear that will allow us to mark certain elements as queryable or not in capabilities output.
+ *
+ * Revision 1.62  2005/10/24 23:53:47  sdlime
+ * A few comment fixes.
+ *
+ * Revision 1.61  2005/10/24 23:49:55  sdlime
+ * Added gmlConstantObj and gmlConstantListObj to mapows.h. These create a mechanism to output constant values as part of GML outout (and schema creation) without defining new attributes in the source data.
+ *
+ * Revision 1.60  2005/10/12 15:34:27  sdlime
+ * Updated the gmlGroupObj to allow you to set the group type. This impacts schema location. If not set the complex type written by the schema generator is 'groupnameType' and via metadata you can override that (e.g. gml_groupname_type 'MynameType').
+ *
  * Revision 1.59  2005/06/02 20:32:01  sdlime
  * Changed metadata reference from ...getfeature_collection to ...feature_collection.
  *
@@ -356,7 +372,7 @@ const char *msOWSGetEPSGProj(projectionObj *proj, hashTableObj *metadata, const
 typedef struct {
   char *name;     /* name of the item */
   char *alias;    /* is the item aliased for presentation? (NULL if not) */
-  char *type;     /* raw type for these item (NULL for a "string") (TODO: should this be a lookup table instead?) */
+  char *type;     /* raw type for this item (NULL for a "string") (TODO: should this be a lookup table instead?) */
   int encode;     /* should the value be HTML encoded? Default is MS_TRUE */
   int visible;    /* should this item be output, default is MS_FALSE */  
 } gmlItemObj;
@@ -366,6 +382,17 @@ typedef struct {
   int numitems;
 } gmlItemListObj;
 
+typedef struct {
+  char *name;     /* name of the constant */
+  char *type;     /* raw type for this item (NULL for a "string") */
+  char *value;    /* output value for this constant (output will look like: <name>value</name>) */
+} gmlConstantObj;
+
+typedef struct {
+  gmlConstantObj *constants;
+  int numconstants;
+} gmlConstantListObj;
+
 typedef struct {
   char *name;     /* name of the geometry (type of GML property) */
   char *type;     /* raw type for these geometries (point|multipoint|line|multiline|polygon|multipolygon */
@@ -381,6 +408,7 @@ typedef struct {
   char *name;     /* name of the group */
   char **items;   /* list of items in the group */
   int numitems;   /* number of items */
+  char *type;     /* name of the complex type */
 } gmlGroupObj;
 
 typedef struct {
@@ -388,19 +416,20 @@ typedef struct {
   int numgroups;
 } gmlGroupListObj;
 
-MS_DLL_EXPORT int msItemInGroups(gmlItemObj *item, gmlGroupListObj *groupList);
+MS_DLL_EXPORT int msItemInGroups(char *name, gmlGroupListObj *groupList);
 MS_DLL_EXPORT gmlItemListObj *msGMLGetItems(layerObj *layer);
 MS_DLL_EXPORT void msGMLFreeItems(gmlItemListObj *itemList);
+MS_DLL_EXPORT gmlConstantListObj *msGMLGetConstants(layerObj *layer);
+MS_DLL_EXPORT void msGMLFreeConstants(gmlConstantListObj *constantList);
 MS_DLL_EXPORT gmlGeometryListObj *msGMLGetGeometries(layerObj *layer);
 MS_DLL_EXPORT void msGMLFreeGeometries(gmlGeometryListObj *geometryList);
 MS_DLL_EXPORT gmlGroupListObj *msGMLGetGroups(layerObj *layer);
 MS_DLL_EXPORT void msGMLFreeGroups(gmlGroupListObj *groupList);
 #endif
 
-#ifdef USE_WMS_SVR
 /* export to fix bug 851 */
 MS_DLL_EXPORT int msGMLWriteQuery(mapObj *map, char *filename, const char *namespaces);
-#endif
+
 
 #ifdef USE_WFS_SVR
 MS_DLL_EXPORT int msGMLWriteWFSQuery(mapObj *map, FILE *stream, int maxfeatures, char *wfs_namespace, int outputformat);
