@@ -28,6 +28,15 @@
  ******************************************************************************
  *
  * $Log: mappdf.c,v $
+ * Revision 1.37.2.1  2006/01/16 20:41:22  sdlime
+ * Fixed error with image legends (shifted text) introduced by the 1449 bug fix. (bug 1607)
+ *
+ * Revision 1.37  2005/12/14 01:21:38  assefa
+ * Add dash line support (bug 492).
+ *
+ * Revision 1.36  2005/12/12 17:02:07  assefa
+ * Segfault on annotation layer when no style is set (Bug 1559)
+ *
  * Revision 1.35  2005/06/14 16:03:34  dan
  * Updated copyright date to 2005
  *
@@ -62,7 +71,7 @@
 #endif
 #include "map.h"
 
-MS_CVSID("$Id: mappdf.c,v 1.35 2005/06/14 16:03:34 dan Exp $")
+MS_CVSID("$Id: mappdf.c,v 1.37.2.1 2006/01/16 20:41:22 sdlime Exp $")
 
 /* -------------------------------------------------------------------- */
 /*      prototypes.                                                     */
@@ -297,6 +306,70 @@ void drawPolylinePDF(PDF *pdf, shapeObj *p, colorObj  *c, double width)
 
 }
 
+#if PDFLIB_MAJORVERSION >= 6
+ /************************************************************************/
+ /* void drawDashedPolylinePDF                                           */
+ /*                                                                      */
+ /* internal function to do line drawing in the pdf, using dashed patterns*/
+ /************************************************************************/
+ void drawDashedPolylinePDF(PDF *pdf, shapeObj *p, symbolObj  *s, colorObj *c, 
+                            double width)
+ {
+     int i, j;
+    int optlistlength=14;
+    char *optlist=NULL;
+     for(i=0;i<s->stylelength;i++) {
+       if(!s->style[i]) /*in case length is zero*/ 
+         optlistlength+=2;
+       else
+        if(s->style[i]>0)
+         optlistlength+=(int)(log10(s->style[i]))+2;
+        else{
+        	msSetError(MS_SYMERR, "Symbol styles must be positive", "drawDashedPolylinePDF()");
+        	return;
+        	}
+     }
+     optlist = (char*)malloc(optlistlength*sizeof(char));
+     sprintf(optlist,"dasharray={");
+     for(i=0;i<s->stylelength;i++)
+       sprintf(optlist,"%s %d",optlist,s->style[i]);
+     sprintf(optlist,"%s }",optlist);
+     PDF_setdashpattern(pdf,optlist);
+     
+    if (width)
+     {
+         PDF_setlinejoin(pdf,1);
+         PDF_setlinewidth(pdf,(float)width);
+     }
+ 
+     if (c)
+     {
+         PDF_setrgbcolor(pdf,((float)c->red)/255,
+                             ((float)c->green)/255,
+                             ((float)c->blue)/255);
+     }
+     for (i = 0; i < p->numlines; i++)
+     {
+         if (p->line[i].numpoints)
+         {
+             PDF_moveto(pdf,(float)p->line[i].point[0].x,
+                (float)p->line[i].point[0].y);
+         }
+         for(j=1; j<p->line[i].numpoints; j++)
+         {
+             PDF_lineto(pdf,(float)p->line[i].point[j].x,
+                (float)p->line[i].point[j].y);
+         }
+     }
+     
+     PDF_stroke(pdf);
+    PDF_setdashpattern(pdf,"");
+    free(optlist); 
+     PDF_setlinewidth(pdf,1);
+}
+
+#endif
+ 
 /************************************************************************/
 /*  void msDrawLineSymbolPDF                                            */
 /*                                                                      */
@@ -329,8 +402,16 @@ void msDrawLineSymbolPDF(symbolSetObj *symbolset, imageObj *image, shapeObj *p,
     if(size < 1) /* size too small */
         return;
 
-    drawPolylinePDF(pdf, p, &(style->color), size);
+#if PDFLIB_MAJORVERSION >= 6
+     if(symbolset && symbolset->symbol[style->symbol].stylelength)
+       drawDashedPolylinePDF(pdf, p, &(symbolset->symbol[style->symbol]), 
+                             &(style->color),size);
+     else
+#endif
+       drawPolylinePDF(pdf, p, &(style->color),  size);
+
     return;
+
 }
 
 /************************************************************************/
@@ -405,7 +486,7 @@ int msDrawLabelPDF(imageObj *image, pointObj labelPnt, char *string,
         pointObj p;
         rectObj r;
 
-        if(msGetLabelSize(string, label, &r, fontset, scalefactor) == -1) return(-1);
+        if(msGetLabelSize(string, label, &r, fontset, scalefactor, MS_FALSE) == -1) return(-1);
         p = get_metrics(&labelPnt, label->position, r,
                                    label->offsetx,
                                    label->offsety,
@@ -478,7 +559,7 @@ int msDrawLabelCachePDF(imageObj *image, mapObj *map)
         if(strlen(cachePtr->text) == 0)
             continue; /* not an error, just don't want to do anything */
 
-        if(msGetLabelSize(cachePtr->text, labelPtr, &r, &(map->fontset), layerPtr->scalefactor) == -1)
+        if(msGetLabelSize(cachePtr->text, labelPtr, &r, &(map->fontset), layerPtr->scalefactor, MS_TRUE) == -1)
             return(-1);
 
         if(labelPtr->autominfeaturesize && ((r.maxx-r.minx) > cachePtr->featuresize))
@@ -489,7 +570,9 @@ int msDrawLabelCachePDF(imageObj *image, mapObj *map)
         { /* there *is* a marker */
 
 	    /* TO DO: at the moment only checks the bottom style, perhaps should check all of them */
-            if(msGetMarkerSize(&map->symbolset, &(cachePtr->styles[0]), &marker_width, &marker_height, layerPtr->scalefactor) != MS_SUCCESS)
+            if(cachePtr->numstyles==0)
+              marker_width = marker_height = 1;
+            else if(msGetMarkerSize(&map->symbolset, &(cachePtr->styles[0]), &marker_width, &marker_height, layerPtr->scalefactor) != MS_SUCCESS)
 	      return(-1);
 
             marker_offset_x = MS_NINT(marker_width/2.0);
