@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mappool.c,v 1.10.2.1 2005/07/07 14:51:16 frank Exp $
+ * $Id: mappool.c,v 1.12 2005/07/16 21:18:34 jerryp Exp $
  *
  * Project:  MapServer
  * Purpose:  Implement new style connection pooling.
@@ -28,7 +28,10 @@
  ******************************************************************************
  *
  * $Log: mappool.c,v $
- * Revision 1.10.2.1  2005/07/07 14:51:16  frank
+ * Revision 1.12  2005/07/16 21:18:34  jerryp
+ * msConnPoolClose now only copies connections when needed.
+ *
+ * Revision 1.11  2005/07/07 14:51:25  frank
  * bug1402: any thread can release a connection
  *
  * Revision 1.10  2005/06/14 16:03:34  dan
@@ -168,7 +171,7 @@ o Currently the connection pooling API makes not attempt to address
 #include "map.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mappool.c,v 1.10.2.1 2005/07/07 14:51:16 frank Exp $")
+MS_CVSID("$Id: mappool.c,v 1.12 2005/07/16 21:18:34 jerryp Exp $")
 
 /* defines for lifetime.  
    A positive number is a time-from-last use in seconds */
@@ -337,21 +340,21 @@ static void msConnPoolClose( int conn_index )
     /* free malloced() stuff in this connection */
     free( conn->connection );
 
-    /* move the last connection in place of our now closed one */
-    memcpy( connections + conn_index, 
-            connections + connectionCount - 1, 
-            sizeof(connectionObj) );
-    
     connectionCount--;
-
-    /* if there are no connections left we will "cleanup".  */
-
     if( connectionCount == 0 )
     {
+        /* if there are no connections left we will "cleanup".  */
         connectionMax = 0;
         free( connections );
         connections = NULL;
     }
+    else
+    {
+        /* move the last connection in place of our now closed one */
+        memcpy( connections + conn_index, 
+                connections + connectionCount, 
+                sizeof(connectionObj) );
+    }
 }
 
 /************************************************************************/
