@@ -27,7 +27,38 @@
  ******************************************************************************
  *
  * $Log: mappostgis.c,v $
- * Revision 1.55.2.2  2005/06/23 05:11:55  jerryp
+ * Revision 1.65.2.3  2006/03/30 05:20:46  sdlime
+ * Applied patch for bug 1629.
+ *
+ * Revision 1.65.2.2  2006/02/25 19:10:41  pramsey
+ * Added checks for MS_FAILURE to the ParseData function calls.
+ * Patch from Tamas Szekeres
+ *
+ * Revision 1.65.2.1  2006/01/19 15:58:16  sean
+ * move gBYTE_ORDER inside the pg layerinfo structure to allow for differently byte ordered connections (bug 1587).
+ *
+ * Revision 1.65  2005/12/14 21:25:47  sean
+ * test for the case of a selection w/out srid for postgis data and fix from Daryl (bug 1443)
+ *
+ * Revision 1.64  2005/12/14 17:24:45  sdlime
+ * Fixed documentation URL in the generic PostGIS error message. (bug 1558)
+ *
+ * Revision 1.63  2005/12/11 00:16:29  sean
+ * postgis layer test cases and fix for broken view and sub-select layers (bug 1443)
+ *
+ * Revision 1.62  2005/10/28 01:09:42  jani
+ * MS RFC 3: Layer vtable architecture (bug 1477)
+ *
+ * Revision 1.61  2005/09/15 20:25:39  pramsey
+ * Added cases to allow the connector to use both the FILTERITEM and FILTER tags, if present.
+ *
+ * Revision 1.60  2005/07/16 21:12:59  jerryp
+ * Fixed memory leaks.
+ *
+ * Revision 1.59  2005/07/16 19:07:39  jerryp
+ * Bug 1420: PostGIS connector no longer needs two layer close functions.
+ *
+ * Revision 1.58  2005/06/23 05:11:38  jerryp
  * Fixed a buffer overflow (bug 1392).
  *
  * Revision 1.57  2005/06/23 04:45:19  jerryp
@@ -82,8 +113,10 @@
  *
  */
 
-/* $Id: mappostgis.c,v 1.55.2.2 2005/06/23 05:11:55 jerryp Exp $ */
+/* $Id: mappostgis.c,v 1.65.2.3 2006/03/30 05:20:46 sdlime Exp $ */
+#include <assert.h>
 #include "map.h"
+#include "maptime.h"
 
 #ifndef FLT_MAX
 #define FLT_MAX 25000000.0
@@ -102,7 +135,7 @@
 #include <string.h>
 #include <ctype.h> /* tolower() */
 
-MS_CVSID("$Id: mappostgis.c,v 1.55.2.2 2005/06/23 05:11:55 jerryp Exp $")
+MS_CVSID("$Id: mappostgis.c,v 1.65.2.3 2006/03/30 05:20:46 sdlime Exp $")
 
 typedef struct ms_POSTGIS_layer_info_t
 {
@@ -112,6 +145,7 @@ typedef struct ms_POSTGIS_layer_info_t
     PGresult    *query_result;  /* for fetching rows from the db */
     char        *urid_name;     /* name of user-specified unique identifier or OID */
     char        *user_srid;     /* zero length = calculate, non-zero means using this value! */
+    int gBYTE_ORDER;            /* no longer a global variable */
 
 } msPOSTGISLayerInfo;
 
@@ -120,7 +154,7 @@ typedef struct ms_POSTGIS_layer_info_t
     "Error with POSTGIS data variable. You specified '%s'.<br>\n" \
     "Standard ways of specifiying are : <br>\n(1) 'geometry_column from geometry_table' <br>\n(2) 'geometry_column from (&lt;sub query&gt;) as foo using unique &lt;column name&gt; using SRID=&lt;srid#&gt;' <br><br>\n\n" \
     "Make sure you put in the 'using unique  &lt;column name&gt;' and 'using SRID=#' clauses in.\n\n<br><br>" \
-    "For more help, please see http://postgis.refractions.net/documentation.php \n\n<br><br>" \
+    "For more help, please see http://postgis.refractions.net/documentation/ \n\n<br><br>" \
     "Mappostgis.c - version of Jan 23/2004.\n"
 
 
@@ -211,14 +245,12 @@ char *strstrIgnoreCase(const char *haystack, const char *needle)
 
 static int msPOSTGISLayerParseData(layerObj *layer, char **geom_column_name, char **table_name, char **urid_name, char **user_srid, int debug);
 
-#ifndef USE_THREAD
 static void msPOSTGISCloseConnection(void *conn_handle)
 {
     PQfinish((PGconn*) conn_handle);
 }
-#endif
 
-static int gBYTE_ORDER = 0;
+/*static int gBYTE_ORDER = 0;*/
 
 /* open up a connection to the postgresql database using the connection string in layer->connection */
 /* ie. "host=192.168.50.3 user=postgres port=5555 dbname=mapserv" */
@@ -255,10 +287,9 @@ int msPOSTGISLayerOpen(layerObj *layer)
     layerinfo->urid_name = NULL;
     layerinfo->user_srid = NULL;
     layerinfo->conn = NULL;
+    layerinfo->gBYTE_ORDER = 0;
 
-#ifndef USE_THREAD
     layerinfo->conn = (PGconn *) msConnPoolRequest(layer);
-#endif
     if(!layerinfo->conn) {
         if(layer->debug) {
             msDebug("MSPOSTGISLayerOpen -- shared connection not available.\n");
@@ -287,22 +318,19 @@ int msPOSTGISLayerOpen(layerObj *layer)
             return MS_FAILURE;
         }
 
-#ifndef USE_THREAD
         msConnPoolRegister(layer, layerinfo->conn, msPOSTGISCloseConnection);
-#endif
 
         PQsetNoticeProcessor(layerinfo->conn, postresql_NOTICE_HANDLER, (void *) layer);
     }
 
 
-    setPostGISLayerInfo(layer, layerinfo);
-
     if(((char *) &order_test)[0] == 1) {
-        gBYTE_ORDER = LITTLE_ENDIAN;
+        layerinfo->gBYTE_ORDER = LITTLE_ENDIAN;
     } else {
-        gBYTE_ORDER = BIG_ENDIAN;
+        layerinfo->gBYTE_ORDER = BIG_ENDIAN;
     }
 
+    setPostGISLayerInfo(layer, layerinfo);
     return MS_SUCCESS;
 }
 
@@ -442,7 +470,7 @@ static int prepare_database(const char *geom_table, const char *geom_column, lay
 
     if(layer->numitems == 0) {
         columns_wanted = (char *) malloc(55 + strlen(geom_column) + strlen(urid_name) + 1);
-        if (gBYTE_ORDER == LITTLE_ENDIAN) {
+        if (layerinfo->gBYTE_ORDER == LITTLE_ENDIAN) {
             sprintf(columns_wanted, "asbinary(force_collection(force_2d(%s)),'NDR'),%s::text", geom_column, urid_name);
         } else {
             sprintf(columns_wanted, "asbinary(force_collection(force_2d(%s)),'XDR'),%s::text", geom_column, urid_name);
@@ -459,7 +487,7 @@ static int prepare_database(const char *geom_table, const char *geom_column, lay
             strcat(columns_wanted, "::text,");
         }
         temp = columns_wanted + strlen(columns_wanted);
-        if(gBYTE_ORDER == LITTLE_ENDIAN) {
+        if(layerinfo->gBYTE_ORDER == LITTLE_ENDIAN) {
             sprintf(temp,"asbinary(force_collection(force_2d(%s)),'NDR'),%s::text", geom_column, urid_name);
         } else {
             sprintf(temp,"asbinary(force_collection(force_2d(%s)),'XDR'),%s::text", geom_column, urid_name);
@@ -647,7 +675,9 @@ int msPOSTGISLayerWhichShapes(layerObj *layer, rectObj rect)
         return MS_FAILURE;
     }
 
-    msPOSTGISLayerParseData(layer, &geom_column_name, &table_name, &urid_name, &user_srid, layer->debug);
+	if (msPOSTGISLayerParseData(layer, &geom_column_name, &table_name, &urid_name, &user_srid, layer->debug) != MS_SUCCESS) {
+		return MS_FAILURE;
+	}
 
     set_up_result = prepare_database(table_name, geom_column_name, layer, &(layerinfo->query_result), rect, &query_str, urid_name, user_srid);
 
@@ -685,32 +715,6 @@ int msPOSTGISLayerWhichShapes(layerObj *layer, rectObj rect)
     return MS_SUCCESS;
 }
 
-/* Releases the cursor and closes the results of the given layer */
-int msPOSTGISLayerResultClose(layerObj *layer)
-{
-    msPOSTGISLayerInfo  *layerinfo;
-    PGresult            *res;
-
-    if (layer->debug) {
-        /* TODO Can layer->data be NULL? */
-        msDebug("msPOSTGISLayerResultClose: %s\n", layer->data);
-    }
-
-    layerinfo = getPostGISLayerInfo(layer);
-    if(layerinfo) {
-        res = PQexec(layerinfo->conn, "COMMIT");
-        if(res) {
-            PQclear(res);
-        }
-        if(layerinfo->query_result) {
-            PQclear(layerinfo->query_result);
-        }
-        layerinfo->query_result = NULL;
-    }
-
-    return MS_SUCCESS;
-}
-
 /* Close the postgis record set and connection */
 int msPOSTGISLayerClose(layerObj *layer)
 {
@@ -737,13 +741,7 @@ int msPOSTGISLayerClose(layerObj *layer)
             msDebug("msPOSTGISLayerClose -- query_result is NULL\n");
         }
 
-#ifndef USE_THREAD
         msConnPoolRelease(layer, layerinfo->conn);
-#else
-        if(layerinfo->conn) {
-            PQfinish(layerinfo->conn);
-        }
-#endif
         layerinfo->conn = NULL;
 
         if(layerinfo->urid_name) {
@@ -1202,12 +1200,14 @@ int msPOSTGISLayerGetShape(layerObj *layer, shapeObj *shape, long record)
         return MS_FAILURE;
     }
 
-    msPOSTGISLayerParseData(layer, &geom_column_name, &table_name, &urid_name, &user_srid, layer->debug);
+    if (msPOSTGISLayerParseData(layer, &geom_column_name, &table_name, &urid_name, &user_srid, layer->debug) != MS_SUCCESS) {
+		return MS_FAILURE;
+	}
 
     if(layer->numitems == 0) {
         /* Don't need the oid since its really record */
         columns_wanted = (char *) malloc(46 + strlen(geom_column_name) + 1);
-        if(gBYTE_ORDER == LITTLE_ENDIAN) {
+        if(layerinfo->gBYTE_ORDER == LITTLE_ENDIAN) {
             sprintf(columns_wanted, "asbinary(force_collection(force_2d(%s)),'NDR')", geom_column_name);
         } else {
             sprintf(columns_wanted, "asbinary(force_collection(force_2d(%s)),'XDR')", geom_column_name);
@@ -1224,7 +1224,7 @@ int msPOSTGISLayerGetShape(layerObj *layer, shapeObj *shape, long record)
             strcat(columns_wanted, "::text,");
         }
         temp = columns_wanted + strlen(columns_wanted);
-        if(gBYTE_ORDER == LITTLE_ENDIAN) {
+        if(layerinfo->gBYTE_ORDER == LITTLE_ENDIAN) {
             sprintf(temp, "asbinary(force_collection(force_2d(%s)),'NDR')", geom_column_name);
         } else {
             sprintf(temp, "asbinary(force_collection(force_2d(%s)),'XDR')", geom_column_name);
@@ -1457,7 +1457,9 @@ int msPOSTGISLayerGetItems(layerObj *layer)
     }
     /* get the table name and geometry column name */
 
-    msPOSTGISLayerParseData(layer, &geom_column_name, &table_name, &urid_name, &user_srid, layer->debug);
+    if (msPOSTGISLayerParseData(layer, &geom_column_name, &table_name, &urid_name, &user_srid, layer->debug) != MS_SUCCESS) {
+		return MS_FAILURE;
+	}
 
     /* two cases here.  One, its a table (use select * from table) otherwise, just use the select clause */
     sql = (char *) malloc(36 + strlen(table_name) + 1);
@@ -1691,9 +1693,9 @@ int msPOSTGISLayerRetrievePGVersion(layerObj *layer, int debug, int *major, int
 
 int msPOSTGISLayerRetrievePK(layerObj *layer, char **urid_name, char* table_name, int debug) 
 {
-    PGresult   *query_result;
-    char        *sql;
-    msPOSTGISLayerInfo *layerinfo;
+    PGresult   *query_result = 0;
+    char        *sql = 0;
+    msPOSTGISLayerInfo *layerinfo = 0;
     int major, minor, point, tmpint;
     
     if(msPOSTGISLayerRetrievePGVersion(layer, debug, &major, &minor, &point) == MS_FAILURE) 
@@ -1749,6 +1751,7 @@ int msPOSTGISLayerRetrievePK(layerObj *layer, char **urid_name, char* table_name
     {
       char tmp1[42] = "Layer does not have a postgis connection.";
       msSetError(MS_QUERYERR, tmp1, "msPOSTGISLayerRetrievePK()");
+      free(sql);
       return(MS_FAILURE);
     }
 
@@ -1764,6 +1767,7 @@ int msPOSTGISLayerRetrievePK(layerObj *layer, char **urid_name, char* table_name
       strcat(tmp2, sql);
       msSetError(MS_QUERYERR, tmp2, "msPOSTGISLayerRetrievePK()");
       free(tmp2);
+      free(sql);
       return(MS_FAILURE);
 
     }
@@ -1775,6 +1779,7 @@ int msPOSTGISLayerRetrievePK(layerObj *layer, char **urid_name, char* table_name
         msDebug("msPOSTGISLayerRetrievePK: No results found.\n");
       }
       PQclear(query_result);
+      free(sql);
       return MS_FAILURE;
     }
 
@@ -1785,15 +1790,16 @@ int msPOSTGISLayerRetrievePK(layerObj *layer, char **urid_name, char* table_name
         msDebug("msPOSTGISLayerRetrievePK: Null result returned.\n");
       }
       PQclear(query_result);
+      free(sql);
       return MS_FAILURE;
     }
 
     tmpint = PQgetlength(query_result, 0, 0);
-    msDebug("msPOSTGISLayerRetrievePK: field length = $i", tmpint);
     *urid_name = (char *)malloc(tmpint + 1);
     strcpy(*urid_name, PQgetvalue(query_result, 0, 0));
 
     PQclear(query_result);
+    free(sql);
     return MS_SUCCESS;
 }
 
@@ -1804,11 +1810,10 @@ int msPOSTGISLayerRetrievePK(layerObj *layer, char **urid_name, char* table_name
 
 static int msPOSTGISLayerParseData(layerObj *layer, char **geom_column_name, char **table_name, char **urid_name, char **user_srid, int debug)
 {
-    char    *pos_opt, *pos_scn, *tmp, *pos_srid, *data;
-    int     slength, urid_found;
+    char    *pos_opt, *pos_scn, *tmp, *pos_srid, *pos_urid, *data;
+    int     slength;
 
     data = layer->data;
-    urid_found = 0;
 
     /* given a string of the from 'geom from ctivalues' or 'geom from () as foo'
      * return geom_column_name as 'geom'
@@ -1816,23 +1821,17 @@ static int msPOSTGISLayerParseData(layerObj *layer, char **geom_column_name, cha
      */
 
     /* First look for the optional ' using unique ID' string */
-    pos_opt = strstrIgnoreCase(data, " using unique ");
+    pos_urid = strstrIgnoreCase(data, " using unique ");
 
-    if(!pos_opt) {
-        /* No user specified unique id so we will use the Postgesql OID */
-        *urid_name = (char *) malloc(4);
-        strcpy(*urid_name, "OID");
-    } else {
+    if(pos_urid) {
         /* CHANGE - protect the trailing edge for thing like 'using unique ftab_id using srid=33' */
-        tmp = strstr(pos_opt + 14, " ");
+        tmp = strstr(pos_urid + 14, " ");
         if(!tmp) {
-            tmp = pos_opt + strlen(pos_opt);
+            tmp = pos_urid + strlen(pos_urid);
         }
-        *urid_name = (char *) malloc((tmp - (pos_opt + 14)) + 1);
-        strncpy(*urid_name, pos_opt + 14, tmp - (pos_opt + 14));
-        (*urid_name)[tmp - (pos_opt + 14)] = 0;
-
-        urid_found = 1;
+        *urid_name = (char *) malloc((tmp - (pos_urid + 14)) + 1);
+        strncpy(*urid_name, pos_urid + 14, tmp - (pos_urid + 14));
+        (*urid_name)[tmp - (pos_urid + 14)] = 0;
     }
 
     /* Find the srid */
@@ -1856,16 +1855,14 @@ static int msPOSTGISLayerParseData(layerObj *layer, char **geom_column_name, cha
 
     /* this is a little hack so the rest of the code works.  If the ' using SRID=' comes before */
     /* the ' using unique ', then make sure pos_opt points to where the ' using SRID' starts! */
-
-    if(!pos_opt) {
-        pos_opt = pos_srid;
-    } else {
-        if(pos_srid) {
-            if(pos_opt > pos_srid) {
-                pos_opt = pos_srid;
-            }
-        }
+    
+    /*pos_opt = (pos_srid > pos_urid) ? pos_urid : pos_srid; */
+    if (!pos_srid && !pos_urid) { pos_opt = pos_urid; }
+    else if (pos_srid && pos_urid) 
+    { 
+        pos_opt = (pos_srid > pos_urid)? pos_urid : pos_srid;
     }
+    else { pos_opt = (pos_srid > pos_urid) ? pos_srid : pos_urid; }
 
     /* Scan for the table or sub-select clause */
     pos_scn = strstrIgnoreCase(data, " from ");
@@ -1894,10 +1891,15 @@ static int msPOSTGISLayerParseData(layerObj *layer, char **geom_column_name, cha
         return MS_FAILURE;
     }
 
-    if(!urid_found) {
-      msPOSTGISLayerRetrievePK(layer, urid_name, *table_name, debug);
+    if( !pos_urid ) {
+        if( msPOSTGISLayerRetrievePK(layer, urid_name, *table_name, debug) != MS_SUCCESS ) {
+            /* No user specified unique id so we will use the Postgesql OID */
+            *urid_name = (char *) malloc(4);
+            strcpy(*urid_name, "OID");
+        }
     }
 
+
     if(debug) {
         msDebug("msPOSTGISLayerParseData: unique column = %s, srid='%s', geom_column_name = %s, table_name=%s\n", *urid_name, *user_srid, *geom_column_name, *table_name);
     }
@@ -1938,12 +1940,6 @@ int msPOSTGISLayerWhichShapes(layerObj *layer, rectObj rect)
     return MS_FAILURE;
 }
 
-int msPOSTGISLayerResultClose(layerObj *layer)
-{
-    msSetError(MS_QUERYERR, "msPOSTGISLayerResultClose called but unimplemented!(mapserver not compiled with postgis support)", "msPOSTGISLayerResultClose()");
-    return MS_FAILURE;
-}
-
 int msPOSTGISLayerClose(layerObj *layer)
 {
     msSetError(MS_QUERYERR, "msPOSTGISLayerClose called but unimplemented!(mapserver not compiled with postgis support)", "msPOSTGISLayerClose()");
@@ -1983,3 +1979,424 @@ int msPOSTGISLayerGetItems(layerObj *layer)
 
 /* end above's #ifdef USE_POSTGIS */
 #endif
+
+int msPOSTGISLayerSetTimeFilter(layerObj *lp, 
+                                const char *timestring, 
+                                const char *timefield)
+{
+    char *tmpstimestring = NULL;
+    char *timeresolution = NULL;
+    int timesresol = -1;
+    char **atimes, **tokens = NULL;
+    int numtimes=0,i=0,ntmp=0,nlength=0;
+    char buffer[512];
+
+    buffer[0] = '\0';
+
+    if (!lp || !timestring || !timefield)
+      return MS_FALSE;
+
+    if (strstr(timestring, ",") == NULL && 
+        strstr(timestring, "/") == NULL) /* discrete time */
+      tmpstimestring = strdup(timestring);
+    else
+    {
+        atimes = split (timestring, ',', &numtimes);
+        if (atimes == NULL || numtimes < 1)
+          return MS_FALSE;
+
+        if (numtimes >= 1)
+        {
+            tokens = split(atimes[0],  '/', &ntmp);
+            if (ntmp == 2) /* ranges */
+            {
+                tmpstimestring = strdup(tokens[0]);
+                msFreeCharArray(tokens, ntmp);
+            }
+            else if (ntmp == 1) /* multiple times */
+            {
+                tmpstimestring = strdup(atimes[0]);
+            }
+        }
+        msFreeCharArray(atimes, numtimes);
+    }
+    if (!tmpstimestring)
+      return MS_FALSE;
+        
+    timesresol = msTimeGetResolution((const char*)tmpstimestring);
+    if (timesresol < 0)
+      return MS_FALSE;
+
+    free(tmpstimestring);
+
+    switch (timesresol)
+    {
+        case (TIME_RESOLUTION_SECOND):
+          timeresolution = strdup("second");
+          break;
+
+        case (TIME_RESOLUTION_MINUTE):
+          timeresolution = strdup("minute");
+          break;
+
+        case (TIME_RESOLUTION_HOUR):
+          timeresolution = strdup("hour");
+          break;
+
+        case (TIME_RESOLUTION_DAY):
+          timeresolution = strdup("day");
+          break;
+
+        case (TIME_RESOLUTION_MONTH):
+          timeresolution = strdup("month");
+          break;
+
+        case (TIME_RESOLUTION_YEAR):
+          timeresolution = strdup("year");
+          break;
+
+        default:
+          break;
+    }
+
+    if (!timeresolution)
+      return MS_FALSE;
+
+    /* where date_trunc('month', _cwctstamp) = '2004-08-01' */
+    if (strstr(timestring, ",") == NULL && 
+        strstr(timestring, "/") == NULL) /* discrete time */
+    {
+        if(lp->filteritem) free(lp->filteritem);
+        lp->filteritem = strdup(timefield);
+        if (&lp->filter)
+          freeExpression(&lp->filter);
+        
+
+        strcat(buffer, "(");
+
+        strcat(buffer, "date_trunc('");
+        strcat(buffer, timeresolution);
+        strcat(buffer, "', ");        
+        strcat(buffer, timefield);
+        strcat(buffer, ")");        
+        
+         
+        strcat(buffer, " = ");
+        strcat(buffer,  "'");
+        strcat(buffer, timestring);
+        /* make sure that the timestring is complete and acceptable */
+        /* to the date_trunc function : */
+        /* - if the resolution is year (2004) or month (2004-01),  */
+        /* a complete string for time would be 2004-01-01 */
+        /* - if the resolluion is hour or minute (2004-01-01 15), a  */
+        /* complete time is 2004-01-01 15:00:00 */
+        if (strcasecmp(timeresolution, "year")==0)
+        {
+            nlength = strlen(timestring);
+            if (timestring[nlength-1] != '-')
+              strcat(buffer,"-01-01");
+            else
+              strcat(buffer,"01-01");
+        }            
+        else if (strcasecmp(timeresolution, "month")==0)
+        {
+            nlength = strlen(timestring);
+            if (timestring[nlength-1] != '-')
+              strcat(buffer,"-01");
+            else
+              strcat(buffer,"01");
+        }            
+        else if (strcasecmp(timeresolution, "hour")==0)
+        {
+            nlength = strlen(timestring);
+            if (timestring[nlength-1] != ':')
+              strcat(buffer,":00:00");
+            else
+              strcat(buffer,"00:00");
+        }            
+        else if (strcasecmp(timeresolution, "minute")==0)
+        {
+            nlength = strlen(timestring);
+            if (timestring[nlength-1] != ':')
+              strcat(buffer,":00");
+            else
+              strcat(buffer,"00");
+        }            
+        
+
+        strcat(buffer,  "'");
+
+        strcat(buffer, ")");
+        
+        /* loadExpressionString(&lp->filter, (char *)timestring); */
+        loadExpressionString(&lp->filter, buffer);
+
+        free(timeresolution);
+        return MS_TRUE;
+    }
+    
+    atimes = split (timestring, ',', &numtimes);
+    if (atimes == NULL || numtimes < 1)
+      return MS_FALSE;
+
+    if (numtimes >= 1)
+    {
+        /* check to see if we have ranges by parsing the first entry */
+        tokens = split(atimes[0],  '/', &ntmp);
+        if (ntmp == 2) /* ranges */
+        {
+            msFreeCharArray(tokens, ntmp);
+            for (i=0; i<numtimes; i++)
+            {
+                tokens = split(atimes[i],  '/', &ntmp);
+                if (ntmp == 2)
+                {
+                    if (strlen(buffer) > 0)
+                      strcat(buffer, " OR ");
+                    else
+                      strcat(buffer, "(");
+
+                    strcat(buffer, "(");
+                    
+                    strcat(buffer, "date_trunc('");
+                    strcat(buffer, timeresolution);
+                    strcat(buffer, "', ");        
+                    strcat(buffer, timefield);
+                    strcat(buffer, ")");        
+ 
+                    strcat(buffer, " >= ");
+                    
+                    strcat(buffer,  "'");
+
+                    strcat(buffer, tokens[0]);
+                    /* - if the resolution is year (2004) or month (2004-01),  */
+                    /* a complete string for time would be 2004-01-01 */
+                    /* - if the resolluion is hour or minute (2004-01-01 15), a  */
+                    /* complete time is 2004-01-01 15:00:00 */
+                    if (strcasecmp(timeresolution, "year")==0)
+                    {
+                        nlength = strlen(tokens[0]);
+                        if (tokens[0][nlength-1] != '-')
+                          strcat(buffer,"-01-01");
+                        else
+                          strcat(buffer,"01-01");
+                    }            
+                    else if (strcasecmp(timeresolution, "month")==0)
+                    {
+                        nlength = strlen(tokens[0]);
+                        if (tokens[0][nlength-1] != '-')
+                          strcat(buffer,"-01");
+                        else
+                          strcat(buffer,"01");
+                    }            
+                    else if (strcasecmp(timeresolution, "hour")==0)
+                    {
+                        nlength = strlen(tokens[0]);
+                        if (tokens[0][nlength-1] != ':')
+                          strcat(buffer,":00:00");
+                        else
+                          strcat(buffer,"00:00");
+                    }            
+                    else if (strcasecmp(timeresolution, "minute")==0)
+                    {
+                        nlength = strlen(tokens[0]);
+                        if (tokens[0][nlength-1] != ':')
+                          strcat(buffer,":00");
+                        else
+                          strcat(buffer,"00");
+                    }            
+
+                    strcat(buffer,  "'");
+                    strcat(buffer, " AND ");
+
+                    
+                    strcat(buffer, "date_trunc('");
+                    strcat(buffer, timeresolution);
+                    strcat(buffer, "', ");        
+                    strcat(buffer, timefield);
+                    strcat(buffer, ")");  
+
+                    strcat(buffer, " <= ");
+                    
+                    strcat(buffer,  "'");
+                    strcat(buffer, tokens[1]);
+
+                    /* - if the resolution is year (2004) or month (2004-01),  */
+                    /* a complete string for time would be 2004-01-01 */
+                    /* - if the resolluion is hour or minute (2004-01-01 15), a  */
+                    /* complete time is 2004-01-01 15:00:00 */
+                    if (strcasecmp(timeresolution, "year")==0)
+                    {
+                        nlength = strlen(tokens[1]);
+                        if (tokens[1][nlength-1] != '-')
+                          strcat(buffer,"-01-01");
+                        else
+                          strcat(buffer,"01-01");
+                    }            
+                    else if (strcasecmp(timeresolution, "month")==0)
+                    {
+                        nlength = strlen(tokens[1]);
+                        if (tokens[1][nlength-1] != '-')
+                          strcat(buffer,"-01");
+                        else
+                          strcat(buffer,"01");
+                    }            
+                    else if (strcasecmp(timeresolution, "hour")==0)
+                    {
+                        nlength = strlen(tokens[1]);
+                        if (tokens[1][nlength-1] != ':')
+                          strcat(buffer,":00:00");
+                        else
+                          strcat(buffer,"00:00");
+                    }            
+                    else if (strcasecmp(timeresolution, "minute")==0)
+                    {
+                        nlength = strlen(tokens[1]);
+                        if (tokens[1][nlength-1] != ':')
+                          strcat(buffer,":00");
+                        else
+                          strcat(buffer,"00");
+                    }            
+
+                    strcat(buffer,  "'");
+                    strcat(buffer, ")");
+                }
+                 
+                msFreeCharArray(tokens, ntmp);
+            }
+            if (strlen(buffer) > 0)
+              strcat(buffer, ")");
+        }
+        else if (ntmp == 1) /* multiple times */
+        {
+            msFreeCharArray(tokens, ntmp);
+            strcat(buffer, "(");
+            for (i=0; i<numtimes; i++)
+            {
+                if (i > 0)
+                  strcat(buffer, " OR ");
+
+                strcat(buffer, "(");
+                  
+                strcat(buffer, "date_trunc('");
+                strcat(buffer, timeresolution);
+                strcat(buffer, "', ");        
+                strcat(buffer, timefield);
+                strcat(buffer, ")");   
+
+                strcat(buffer, " = ");
+                  
+                strcat(buffer,  "'");
+
+                strcat(buffer, atimes[i]);
+                
+                /* make sure that the timestring is complete and acceptable */
+                /* to the date_trunc function : */
+                /* - if the resolution is year (2004) or month (2004-01),  */
+                /* a complete string for time would be 2004-01-01 */
+                /* - if the resolluion is hour or minute (2004-01-01 15), a  */
+                /* complete time is 2004-01-01 15:00:00 */
+                if (strcasecmp(timeresolution, "year")==0)
+                {
+                    nlength = strlen(atimes[i]);
+                    if (atimes[i][nlength-1] != '-')
+                      strcat(buffer,"-01-01");
+                    else
+                      strcat(buffer,"01-01");
+                }            
+                else if (strcasecmp(timeresolution, "month")==0)
+                {
+                    nlength = strlen(atimes[i]);
+                    if (atimes[i][nlength-1] != '-')
+                      strcat(buffer,"-01");
+                    else
+                      strcat(buffer,"01");
+                }            
+                else if (strcasecmp(timeresolution, "hour")==0)
+                {
+                    nlength = strlen(atimes[i]);
+                    if (atimes[i][nlength-1] != ':')
+                      strcat(buffer,":00:00");
+                    else
+                      strcat(buffer,"00:00");
+                }            
+                else if (strcasecmp(timeresolution, "minute")==0)
+                {
+                    nlength = strlen(atimes[i]);
+                    if (atimes[i][nlength-1] != ':')
+                      strcat(buffer,":00");
+                    else
+                      strcat(buffer,"00");
+                }            
+
+                strcat(buffer,  "'");
+                strcat(buffer, ")");
+            } 
+            strcat(buffer, ")");
+        }
+        else
+        {
+            msFreeCharArray(atimes, numtimes);
+            return MS_FALSE;
+        }
+
+        msFreeCharArray(atimes, numtimes);
+
+        /* load the string to the filter */
+        if (strlen(buffer) > 0)
+        {
+            if(lp->filteritem) 
+              free(lp->filteritem);
+            lp->filteritem = strdup(timefield);     
+            if (&lp->filter)
+              freeExpression(&lp->filter);
+            loadExpressionString(&lp->filter, buffer);
+        }
+
+        free(timeresolution);
+        return MS_TRUE;
+                 
+    }
+    
+    return MS_FALSE;
+}
+
+int 
+msPOSTGISLayerGetShapeVT(layerObj *layer, shapeObj *shape, int tile, long record)
+{
+    return msPOSTGISLayerGetShape(layer, shape, record);
+}
+
+
+int
+msPOSTGISLayerInitializeVirtualTable(layerObj *layer)
+{
+    assert(layer != NULL);
+    assert(layer->vtable != NULL);
+
+    layer->vtable->LayerInitItemInfo = msPOSTGISLayerInitItemInfo;
+    layer->vtable->LayerFreeItemInfo = msPOSTGISLayerFreeItemInfo;
+    layer->vtable->LayerOpen = msPOSTGISLayerOpen;
+    layer->vtable->LayerIsOpen = msPOSTGISLayerIsOpen;
+    layer->vtable->LayerWhichShapes = msPOSTGISLayerWhichShapes;
+    layer->vtable->LayerNextShape = msPOSTGISLayerNextShape;
+    layer->vtable->LayerGetShape = msPOSTGISLayerGetShapeVT;
+
+    layer->vtable->LayerClose = msPOSTGISLayerClose;
+
+    layer->vtable->LayerGetItems = msPOSTGISLayerGetItems;
+    layer->vtable->LayerGetExtent = msPOSTGISLayerGetExtent;
+
+    layer->vtable->LayerApplyFilterToLayer = msLayerApplyCondSQLFilterToLayer;
+
+    /* layer->vtable->LayerGetAutoStyle, not supported for this layer */
+    layer->vtable->LayerCloseConnection = msPOSTGISLayerClose;
+    
+    layer->vtable->LayerSetTimeFilter = msPOSTGISLayerSetTimeFilter;
+    /* layer->vtable->LayerCreateItems, use default */
+    /* layer->vtable->LayerGetNumFeatures, use default */
+
+
+    return MS_SUCCESS;
+}
+
