@@ -27,6 +27,30 @@
  ******************************************************************************
  *
  * $Log: mapprimitive.c,v $
+ * Revision 1.60.2.1  2006/02/17 03:06:13  sdlime
+ * Fixed a flaw in routine that computes outer ring list. In certain cases it could miss an outer ring with holes in certain places. (bug 1648)
+ *
+ * Revision 1.60  2005/11/01 05:35:50  frank
+ * added preliminary implementation of OGR based WKT translation, still untested
+ *
+ * Revision 1.59  2005/10/31 04:58:14  sdlime
+ * Added a check in msFreeShape() to make sure the incoming shapeObj is not NULL.
+ *
+ * Revision 1.58  2005/10/30 05:05:07  sdlime
+ * Initial support for WKT via GEOS. The reader is only integrated via the map file reader, with MapScript, CGI and URL support following ASAP. (bug 1466)
+ *
+ * Revision 1.57  2005/10/20 19:37:03  frank
+ * added msAddPointToLine
+ *
+ * Revision 1.56  2005/10/18 03:20:44  frank
+ * fixed use of memmove in msShapeDeleteLine()
+ *
+ * Revision 1.55  2005/10/18 03:10:45  frank
+ * added msShapeDeleteLine
+ *
+ * Revision 1.54  2005/07/27 18:21:58  frank
+ * bug 1432: optimized msAddLine() to use realloc()
+ *
  * Revision 1.53  2005/06/14 16:03:34  dan
  * Updated copyright date to 2005
  *
@@ -70,8 +94,9 @@
 
 #include "map.h"
 #include "mapprimitive.h"
+#include <assert.h>
 
-MS_CVSID("$Id: mapprimitive.c,v 1.53 2005/06/14 16:03:34 dan Exp $")
+MS_CVSID("$Id: mapprimitive.c,v 1.60.2.1 2006/02/17 03:06:13 sdlime Exp $")
 
 typedef enum {CLIP_LEFT, CLIP_MIDDLE, CLIP_RIGHT} CLIP_STATE;
 
@@ -96,6 +121,30 @@ void msPrintShape(shapeObj *p)
   }
 }
 
+shapeObj *msShapeFromWKT(const char *string)
+{
+#ifdef USE_GEOS
+  return msGEOSShapeFromWKT(string);
+#elif defined(USE_OGR)
+  return msOGRShapeFromWKT(string);
+#else
+  msSetError(MS_MISCERR, "WKT support is not available, please compile MapServer with GEOS or OGR support.", "msShapeFromWKT()");
+  return NULL;
+#endif
+}
+
+char *msShapeToWKT(shapeObj *shape)
+{
+#ifdef USE_GEOS
+  return msGEOSShapeToWKT(shape);
+#elif defined(USE_OGR)
+  return msOGRShapeToWKT(shape);
+#else
+  msSetError(MS_MISCERR, "WKT support is not available, please compile MapServer with GEOS or OGR support.", "msShapeToWKT()");
+  return NULL;
+#endif
+}
+
 void msInitShape(shapeObj *shape)
 {
   /* spatial component */
@@ -158,6 +207,8 @@ void msFreeShape(shapeObj *shape)
 {
   int c;
 
+  if(!shape) return; /* for safety */
+
   for (c= 0; c < shape->numlines; c++)
     free(shape->line[c].point);
   free(shape->line);
@@ -172,6 +223,25 @@ void msFreeShape(shapeObj *shape)
   msInitShape(shape); /* now reset */
 }
 
+void msShapeDeleteLine( shapeObj *shape, int line )
+
+{
+    if( line < 0 || line >= shape->numlines )
+    {
+        assert( 0 );
+        return;
+    }
+
+    free( shape->line[line].point );
+    if( line < shape->numlines - 1 )
+    {
+        memmove( shape->line + line, 
+                 shape->line + line + 1, 
+                 sizeof(lineObj) * (shape->numlines - line - 1) );
+    }
+    shape->numlines--;
+}
+
 void msComputeBounds(shapeObj *shape)
 {
   int i, j;
@@ -193,20 +263,19 @@ void msComputeBounds(shapeObj *shape)
 }
 
 /* checks to see if ring r is an outer ring of shape */
-static int isOuterRing(shapeObj *shape, int r) 
+static int isOuterRing(shapeObj *shape, int r)
 {
-  pointObj point; /* a point in the ring */
-  shapeObj ring;
+  int i, status=MS_TRUE;
 
   if(shape->numlines == 1) return(MS_TRUE);
 
-  msInitShape(&ring); /* convert the ring of interest into its own shape */
-  msAddLine(&ring, &(shape->line[r]));
+  for(i=0; i<shape->numlines; i++) {
+    if(i == r) continue;
+    if(msPointInPolygon(&(shape->line[r].point[0]), &(shape->line[i])) == MS_TRUE)
+      status = !status;
+  }
 
-  msPolygonLabelPoint(&ring, &point, -1); /* generate a point in that ring */
-  msFreeShape(&ring); /* done with it */
-
-  return(msIntersectPointPolygon(&point, shape)); /* test the point against the main shape */
+  return(status);
 }
 
 /*
@@ -251,40 +320,43 @@ int *msGetInnerList(shapeObj *shape, int r, int *outerlist)
   return(list);
 }
 
+/*
+** Add point to a line object.
+**
+** Note that reallocating the point array larger for each point can
+** be pretty inefficient, so use this function sparingly.  Mostly 
+** geometries creators should create their own working lineObj and 
+** then call msAddLine() to add it to a shape.
+*/
+
+int msAddPointToLine(lineObj *line, pointObj *point )
+{
+    line->numpoints += 1;
+    
+    line->point = (pointObj *) 
+        realloc(line->point, sizeof(pointObj) * line->numpoints);
+
+    line->point[line->numpoints-1] = *point;
+
+    return MS_SUCCESS;
+}
+
 int msAddLine(shapeObj *p, lineObj *new_line)
 {
-  int c;
-  lineObj *extended_line;
+    lineObj lineCopy;
 
-  /* Create an extended line array */
-  if((extended_line = (lineObj *)malloc((p->numlines + 1) * sizeof(lineObj))) == NULL) {
-    msSetError(MS_MEMERR, NULL, "msAddLine()");
-    return(MS_FAILURE);
-  }
+    lineCopy.numpoints = new_line->numpoints;
+    lineCopy.point = (pointObj *) malloc(new_line->numpoints*sizeof(pointObj));
+    if( lineCopy.point == NULL )
+    {
+        msSetError(MS_MEMERR, NULL, "msAddLine()");
+        return(MS_FAILURE);
+    }
+    
+    memcpy( lineCopy.point, new_line->point, 
+            sizeof(pointObj) * new_line->numpoints );
 
-  /* Copy the old line into the extended line array */
-  for (c= 0; c < p->numlines; c++)
-    extended_line[c]= p->line[c];
-
-  /* Copy the new line onto the end of the extended line array */
-  c= p->numlines;
-  extended_line[c].numpoints = new_line->numpoints;  
-  if((extended_line[c].point = (pointObj *)malloc(new_line->numpoints * sizeof(pointObj))) == NULL) {
-    msSetError(MS_MEMERR, NULL, "msAddLine()");
-    return(MS_FAILURE);
-  }
-
-  memcpy( extended_line[c].point, new_line->point, 
-          sizeof(pointObj) * new_line->numpoints );
-
-  /* Dispose of the old line */
-  if(p->line) free(p->line);
-
-  /* Update the polygon information */
-  p->numlines++;
-  p->line = extended_line;
-
-  return(MS_SUCCESS);
+    return msAddLineDirectly( p, &lineCopy );
 }
 
 /*
@@ -294,33 +366,23 @@ int msAddLine(shapeObj *p, lineObj *new_line)
 int msAddLineDirectly(shapeObj *p, lineObj *new_line)
 {
   int c;
-  lineObj *extended_line;
-
-  /* Create an extended line array */
-  if((extended_line = (lineObj *)malloc((p->numlines + 1) * sizeof(lineObj))) == NULL) {
-    msSetError(MS_MEMERR, NULL, "msAddLine()");
-    return(MS_FAILURE);
-  }
-
-  /* Copy the old line into the extended line array */
-  for (c= 0; c < p->numlines; c++)
-    extended_line[c]= p->line[c];
 
+  if( p->numlines == 0 )
+      p->line = (lineObj *) malloc(sizeof(lineObj));
+  else
+      p->line = (lineObj *) realloc(p->line, (p->numlines+1)*sizeof(lineObj));
+  
   /* Copy the new line onto the end of the extended line array */
   c= p->numlines;
-  extended_line[c].numpoints = new_line->numpoints;  
-  extended_line[c].point = new_line->point;
+  p->line[c].numpoints = new_line->numpoints;  
+  p->line[c].point = new_line->point;
 
   /* strip points reference off the passed in lineObj */
   new_line->point = NULL;
   new_line->numpoints = 0;
 
-  /* Dispose of the old line */
-  if(p->line) free(p->line);
-
   /* Update the polygon information */
   p->numlines++;
-  p->line = extended_line;
 
   return(MS_SUCCESS);
 }
