@@ -27,6 +27,12 @@
  ******************************************************************************
  *
  * $Log: mapquery.c,v $
+ * Revision 1.89  2006/01/01 02:42:39  sdlime
+ * The bounds that are part of the result cache are projected so there is no need to do any projection when computing the overall result set bounds.
+ *
+ * Revision 1.88  2005/12/21 23:31:19  sdlime
+ * Fixed an error that occurs when doing attribute queries against layers with a FILTER. The code to cache the existing filter blindly called strdup on a value that could be NULL.
+ *
  * Revision 1.87  2005/06/14 16:03:34  dan
  * Updated copyright date to 2005
  *
@@ -49,7 +55,7 @@
 
 #include "map.h"
 
-MS_CVSID("$Id: mapquery.c,v 1.87 2005/06/14 16:03:34 dan Exp $")
+MS_CVSID("$Id: mapquery.c,v 1.89 2006/01/01 02:42:39 sdlime Exp $")
 
 /*
 ** msIsLayerQueryable()  returns MS_TRUE/MS_FALSE
@@ -304,7 +310,8 @@ int msQueryByAttributes(mapObj *map, int qlayer, char *qitem, char *qstring, int
   if(lp->filter.string) {
     old_filtertype = lp->filter.type;
     old_filterstring = strdup(lp->filter.string);
-    old_filteritem = strdup(lp->filteritem);
+    if(lp->filteritem) 
+      old_filteritem = strdup(lp->filteritem);
   }
 
   /* apply the passed query parameters */
@@ -1161,10 +1168,7 @@ int msGetQueryResultBounds(mapObj *map, rectObj *bounds)
     if(lp->resultcache->numresults <= 0) continue;
 
     tmpBounds = lp->resultcache->bounds;
-    #ifdef USE_PROJ  
-    if(lp->project && msProjectionsDiffer(&(lp->projection), &(map->projection)))  
-        msProjectRect(&(lp->projection), &(map->projection), &tmpBounds);
-    #endif
+
     if(found == 0) {
       *bounds = tmpBounds;
     } else {
