@@ -29,6 +29,25 @@
  ******************************************************************************
  *
  * $Log: mapraster.c,v $
+ * Revision 1.134  2005/12/15 16:37:06  frank
+ * Last fix was for bug 1562
+ *
+ * Revision 1.133  2005/12/15 16:35:42  frank
+ * Dont require valid projections before calling mapresample.c code
+ *
+ * Revision 1.132  2005/12/08 19:06:30  hobu
+ * switch off SDE raster support
+ *
+ * Revision 1.131  2005/10/26 17:50:54  frank
+ * Avoid type punning warnings in readGEOTiff().
+ *
+ * Revision 1.130  2005/10/25 16:16:02  assefa
+ * Copy the filteritem and filter into the temporary layer
+ * created when handling tileindex rasters. (Bug 1506).
+ *
+ * Revision 1.129  2005/09/21 01:18:33  frank
+ * use mapresample.c if RESAMPLE is set
+ *
  * Revision 1.128  2005/06/14 16:03:34  dan
  * Updated copyright date to 2005
  *
@@ -92,7 +111,7 @@
 #include "mapresample.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mapraster.c,v 1.128 2005/06/14 16:03:34 dan Exp $")
+MS_CVSID("$Id: mapraster.c,v 1.134 2005/12/15 16:37:06 frank Exp $")
 
 extern int msyyparse(void);
 extern int msyylex(void);
@@ -453,7 +472,8 @@ static int readWorldFile(char *filename, double *ulx, double *uly, double *cx, d
 static int readGEOTiff(TIFF *tif, double *ulx, double *uly, double *cx, double *cy)
 {
   short entries;
-  int i,fpos,swap,tiepos,cellpos;
+  int i,fpos,swap;
+  uint32 tiepos, cellpos;
   double tie[6],cell[6];
   TIFFDirEntry tdir;
   FILE *f;
@@ -471,11 +491,11 @@ static int readGEOTiff(TIFF *tif, double *ulx, double *uly, double *cx, double *
     if (swap) TIFFSwabShort(&tdir.tdir_tag);
     if (tdir.tdir_tag==0x8482) {  /* geoTIFF tie point */
       tiepos=tdir.tdir_offset;
-      if (swap) TIFFSwabLong((long *)&tiepos);
+      if (swap) TIFFSwabLong(&tiepos);
       }
     if (tdir.tdir_tag==0x830e) {  /* geoTIFF cell size */
       cellpos=tdir.tdir_offset;
-      if (swap) TIFFSwabLong((long *)&cellpos);
+      if (swap) TIFFSwabLong(&cellpos);
       }
     }
   if (tiepos==0 || cellpos==0) {
@@ -1258,6 +1278,7 @@ int msDrawRasterLayerLow(mapObj *map, layerObj *layer, imageObj *image)
 
   rectObj searchrect;
   gdImagePtr img;
+  char *pszTmp = NULL;
 
   cwd[0] = '\0';
 
@@ -1338,6 +1359,33 @@ int msDrawRasterLayerLow(mapObj *map, layerObj *layer, imageObj *image)
       tlp->name = strdup("TILE");
       tlp->type = MS_LAYER_TILEINDEX;
       tlp->data = strdup(layer->tileindex);
+      if (layer->filteritem)
+        tlp->filteritem = strdup(layer->filteritem);
+      if (layer->filter.string)
+      {
+          if (layer->filter.type == MS_EXPRESSION)
+          {
+              pszTmp = 
+                (char *)malloc(sizeof(char)*(strlen(layer->filter.string)+3));
+              sprintf(pszTmp,"(%s)",layer->filter.string);
+              msLoadExpressionString(&tlp->filter, pszTmp);
+              free(pszTmp);
+          }
+          else if (layer->filter.type == MS_REGEX || 
+                   layer->filter.type == MS_REGEX == MS_IREGEX)
+          {
+              pszTmp = 
+                (char *)malloc(sizeof(char)*(strlen(layer->filter.string)+3));
+              sprintf(pszTmp,"/%s/",layer->filter.string);
+              msLoadExpressionString(&tlp->filter, pszTmp);
+              free(pszTmp);
+          }
+          else
+            msLoadExpressionString(&tlp->filter, layer->filter.string);
+                   
+          tlp->filter.type = layer->filter.type;
+      }
+
     } else
       tlp = &(layer->map->layers[tilelayerindex]);
       
@@ -1413,7 +1461,7 @@ int msDrawRasterLayerLow(mapObj *map, layerObj *layer, imageObj *image)
 
     if(strlen(filename) == 0) continue;
 
-#ifdef USE_SDE
+/*#ifdef USE_SDE
     if (layer->connectiontype == MS_SDE) {
       status = drawSDE(map, layer, img);
       if(status == -1) {
@@ -1422,7 +1470,8 @@ int msDrawRasterLayerLow(mapObj *map, layerObj *layer, imageObj *image)
       }
       continue;
     }
-#endif
+#endif 
+*/
 
     msBuildPath3(szPath, map->mappath, map->shapepath, filename);
 
@@ -1514,18 +1563,16 @@ int msDrawRasterLayerLow(mapObj *map, layerObj *layer, imageObj *image)
             msGetGDALGeoTransform( hDS, map, layer, adfGeoTransform );
 
             /* 
-            ** We want to resample if the source image is rotated, or if
-            ** the projections differ.  However, due to limitations in 
-            ** msResampleGDALToMap() we can only resample rotated images
-            ** if they also have fully defined projections.
+            ** We want to resample if the source image is rotated, if
+            ** the projections differ or if resampling has been explicitly
+            ** requested.
             */
 #ifdef USE_PROJ
             if( ((adfGeoTransform[2] != 0.0 || adfGeoTransform[4] != 0.0)
-                 && layer->transform
-                 && map->projection.proj != NULL 
-                 && layer->projection.proj != NULL)
+                 && layer->transform )
                 || msProjectionsDiffer( &(map->projection), 
-                                        &(layer->projection) ) )
+                                        &(layer->projection) ) 
+                || CSLFetchNameValue( layer->processing, "RESAMPLE" ) != NULL )
             {
                 status = msResampleGDALToMap( map, layer, image, hDS );
             }
@@ -1535,7 +1582,11 @@ int msDrawRasterLayerLow(mapObj *map, layerObj *layer, imageObj *image)
                 if( adfGeoTransform[2] != 0.0 || adfGeoTransform[4] != 0.0 )
                 {
                     if( layer->debug || map->debug )
-                        msDebug( "Layer %s has rotational coefficients but we are unable to\nuse them, is projection set?", layer->name );
+                        msDebug( 
+                            "Layer %s has rotational coefficients but we\n"
+                            "are unable to use them, projections support\n"
+                            "needs to be built in.", 
+                            layer->name );
                     
                 }
                 status = msDrawRasterLayerGDAL(map, layer, image, hDS );
@@ -1558,7 +1609,7 @@ int msDrawRasterLayerLow(mapObj *map, layerObj *layer, imageObj *image)
             msReleaseLock( TLOCK_GDAL );
         }
     }
-#endif
+#endif /* ifdef USE_GDAL */
 
     /* If GDAL doesn't recognise it, and it wasn't successfully opened 
     ** Generate an error.
