@@ -27,6 +27,12 @@
  ******************************************************************************
  *
  * $Log: mapscale.c,v $
+ * Revision 1.49.2.1  2006/03/27 05:50:52  sdlime
+ * Fixed symbol initialization error with embedded scalebars and legends. (bug 1725)
+ *
+ * Revision 1.49  2005/09/27 15:27:18  sean
+ * Fixed typo to prevent extra scalebar layer creation (bug 1480)
+ *
  * Revision 1.48  2005/06/14 16:03:34  dan
  * Updated copyright date to 2005
  *
@@ -46,7 +52,7 @@
 
 #include "map.h"
 
-MS_CVSID("$Id: mapscale.c,v 1.48 2005/06/14 16:03:34 dan Exp $")
+MS_CVSID("$Id: mapscale.c,v 1.49.2.1 2006/03/27 05:50:52 sdlime Exp $")
 
 #define VMARGIN 3 /* buffer around the scalebar */
 #define HMARGIN 3
@@ -330,6 +336,8 @@ int msEmbedScalebar(mapObj *map, gdImagePtr img)
 
   map->symbolset.symbol[s].type = MS_SYMBOL_PIXMAP; /* intialize a few things */
   map->symbolset.symbol[s].name = strdup("scalebar");  
+  map->symbolset.symbol[s].sizex = map->symbolset.symbol[s].img->sx;
+  map->symbolset.symbol[s].sizey = map->symbolset.symbol[s].img->sy;
 
   /* in 8 bit, mark 0 as being transparent.  In 24bit hopefully already
      have transparency enabled ... */
@@ -364,7 +372,7 @@ int msEmbedScalebar(mapObj *map, gdImagePtr img)
     break;
   }
 
-  l = msGetLayerIndex(map, "__embed_scalebar");
+  l = msGetLayerIndex(map, "__embed__scalebar");
   if(l == -1) {
     l = map->numlayers;
     map->numlayers++;
