@@ -4,9 +4,18 @@
  *
  * This file was originally based on the SWIG interface file mapscript.i
  *
- * $Id: mapscript_i.c,v 1.93 2005/06/06 05:48:20 dan Exp $
+ * $Id: mapscript_i.c,v 1.95 2005/10/13 16:00:36 assefa Exp $
  *
  * $Log: mapscript_i.c,v $
+ * Revision 1.95  2005/10/13 16:00:36  assefa
+ * Add support for whichshape and nextshape (Bug 1491)
+ * Adsd support for config setting/getting at the map level (Bug 1487).
+ *
+ * Revision 1.94  2005/09/08 19:24:50  assefa
+ * Expose GEOS operations through PHP Mapscript (Bug 1327).
+ * Initially only functions buffer and convexhull on a shape object
+ * are available.
+ *
  * Revision 1.93  2005/06/06 05:48:20  dan
  * Added $layerObj->removeClass() (was already in SWIG MapScript)
  * Added $layerObj->removeClass() to PHP MapScript (was already in SWIG
@@ -422,6 +431,30 @@ int layerObj_open(layerObj *self) {
     return msLayerOpen(self);
   }
 
+
+int layerObj_whichShapes(layerObj *self, rectObj *poRect) {
+  msLayerGetItems(self);
+  return msLayerWhichShapes(self, *poRect);
+}
+
+
+shapeObj *layerObj_nextShape(layerObj *self) {
+  int status;
+  shapeObj *shape;
+
+  shape = (shapeObj *)malloc(sizeof(shapeObj));
+  if (!shape) return NULL;
+  msInitShape(shape);
+
+  status = msLayerNextShape(self, shape);
+  if(status != MS_SUCCESS) {
+    msFreeShape(shape);
+    free(shape);
+    return NULL;
+  } else
+    return shape;
+}
+
 void layerObj_close(layerObj *self) {
     msLayerClose(self);
   }
@@ -920,6 +953,17 @@ pointObj *shapeObj_getmeasureusingpoint(shapeObj *self, pointObj *point)
    return NULL;
 }
 
+
+shapeObj *shapeObj_buffer(shapeObj *self, double width)
+{
+    return msGEOSBuffer(self, width);
+}
+
+ 
+shapeObj *shapeObj_convexHull(shapeObj *self)
+{
+    return msGEOSConvexHull(self);
+}
 /**********************************************************************
  * class extensions for rectObj
  **********************************************************************/
