@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: php_mapscript.h,v 1.51 2005/06/14 16:03:36 dan Exp $
+ * $Id: php_mapscript.h,v 1.53 2005/10/13 16:00:36 assefa Exp $
  *
  * Name:     php_mapscript.c
  * Project:  PHP/MapScript extension for MapServer.
@@ -30,6 +30,15 @@
  **********************************************************************
  *
  * $Log: php_mapscript.h,v $
+ * Revision 1.53  2005/10/13 16:00:36  assefa
+ * Add support for whichshape and nextshape (Bug 1491)
+ * Adsd support for config setting/getting at the map level (Bug 1487).
+ *
+ * Revision 1.52  2005/09/08 19:24:50  assefa
+ * Expose GEOS operations through PHP Mapscript (Bug 1327).
+ * Initially only functions buffer and convexhull on a shape object
+ * are available.
+ *
  * Revision 1.51  2005/06/14 16:03:36  dan
  * Updated copyright date to 2005
  *
@@ -191,6 +200,8 @@ char            *mapObj_generateSLD(mapObj *self);
 layerObj       *layerObj_new(mapObj *map);
 void            layerObj_destroy(layerObj* self);
 int             layerObj_open(layerObj *self);
+int             layerObj_whichShapes(layerObj *self, rectObj *poRect);
+shapeObj        *layerObj_nextShape(layerObj *self);
 void            layerObj_close(layerObj *self);
 int             layerObj_getShape(layerObj *self, shapeObj *shape,
                                   int tileindex, int shapeindex);
@@ -291,6 +302,8 @@ int             shapeObj_contains(shapeObj *self, pointObj *point);
 int             shapeObj_intersects(shapeObj *self, shapeObj *shape);
 pointObj        *shapeObj_getpointusingmeasure(shapeObj *self, double m);
 pointObj        *shapeObj_getmeasureusingpoint(shapeObj *self, pointObj *point);
+shapeObj        *shapeObj_buffer(shapeObj *self, double width);
+shapeObj        *shapeObj_convexHull(shapeObj *self);
 
 rectObj        *rectObj_new();
 void            rectObj_destroy(rectObj *self);
