@@ -45,6 +45,9 @@ MS_NO = _mapscript.MS_NO
 MS_SINGLE = _mapscript.MS_SINGLE
 MS_MULTIPLE = _mapscript.MS_MULTIPLE
 MS_GD_ALPHA = _mapscript.MS_GD_ALPHA
+MS_MAXCLASSES = _mapscript.MS_MAXCLASSES
+MS_MAXSTYLES = _mapscript.MS_MAXSTYLES
+MS_MAXLAYERS = _mapscript.MS_MAXLAYERS
 MS_FILE_MAP = _mapscript.MS_FILE_MAP
 MS_FILE_SYMBOL = _mapscript.MS_FILE_SYMBOL
 MS_INCHES = _mapscript.MS_INCHES
@@ -100,6 +103,7 @@ MS_WFS = _mapscript.MS_WFS
 MS_GRATICULE = _mapscript.MS_GRATICULE
 MS_MYGIS = _mapscript.MS_MYGIS
 MS_RASTER = _mapscript.MS_RASTER
+MS_PLUGIN = _mapscript.MS_PLUGIN
 MS_DB_XBASE = _mapscript.MS_DB_XBASE
 MS_DB_CSV = _mapscript.MS_DB_CSV
 MS_DB_MYSQL = _mapscript.MS_DB_MYSQL
@@ -276,6 +280,8 @@ class webObj(object):
     mintemplate = property(_mapscript.webObj_mintemplate_get, _mapscript.webObj_mintemplate_set)
     maxtemplate = property(_mapscript.webObj_maxtemplate_get, _mapscript.webObj_maxtemplate_set)
     queryformat = property(_mapscript.webObj_queryformat_get, _mapscript.webObj_queryformat_set)
+    legendformat = property(_mapscript.webObj_legendformat_get, _mapscript.webObj_legendformat_set)
+    browseformat = property(_mapscript.webObj_browseformat_get, _mapscript.webObj_browseformat_set)
     metadata = property(_mapscript.webObj_metadata_get, _mapscript.webObj_metadata_set)
     def __init__(self, *args):
         newobj = _mapscript.new_webObj(*args)
@@ -367,6 +373,7 @@ class classObj(object):
     def setExpression(*args): return _mapscript.classObj_setExpression(*args)
     def getExpressionString(*args): return _mapscript.classObj_getExpressionString(*args)
     def setText(*args): return _mapscript.classObj_setText(*args)
+    def getTextString(*args): return _mapscript.classObj_getTextString(*args)
     def getMetaData(*args): return _mapscript.classObj_getMetaData(*args)
     def setMetaData(*args): return _mapscript.classObj_setMetaData(*args)
     def getFirstMetaDataKey(*args): return _mapscript.classObj_getFirstMetaDataKey(*args)
@@ -643,48 +650,6 @@ class legendObjPtr(legendObj):
         self.__class__ = legendObj
 _mapscript.legendObj_swigregister(legendObjPtr)
 
-class graticuleObj(object):
-    def __repr__(self):
-        return "<C graticuleObj instance at %s>" % (self.this,)
-    dwhichlatitude = property(_mapscript.graticuleObj_dwhichlatitude_get, _mapscript.graticuleObj_dwhichlatitude_set)
-    dwhichlongitude = property(_mapscript.graticuleObj_dwhichlongitude_get, _mapscript.graticuleObj_dwhichlongitude_set)
-    dstartlatitude = property(_mapscript.graticuleObj_dstartlatitude_get, _mapscript.graticuleObj_dstartlatitude_set)
-    dstartlongitude = property(_mapscript.graticuleObj_dstartlongitude_get, _mapscript.graticuleObj_dstartlongitude_set)
-    dendlatitude = property(_mapscript.graticuleObj_dendlatitude_get, _mapscript.graticuleObj_dendlatitude_set)
-    dendlongitude = property(_mapscript.graticuleObj_dendlongitude_get, _mapscript.graticuleObj_dendlongitude_set)
-    dincrementlatitude = property(_mapscript.graticuleObj_dincrementlatitude_get, _mapscript.graticuleObj_dincrementlatitude_set)
-    dincrementlongitude = property(_mapscript.graticuleObj_dincrementlongitude_get, _mapscript.graticuleObj_dincrementlongitude_set)
-    minarcs = property(_mapscript.graticuleObj_minarcs_get, _mapscript.graticuleObj_minarcs_set)
-    maxarcs = property(_mapscript.graticuleObj_maxarcs_get, _mapscript.graticuleObj_maxarcs_set)
-    minincrement = property(_mapscript.graticuleObj_minincrement_get, _mapscript.graticuleObj_minincrement_set)
-    maxincrement = property(_mapscript.graticuleObj_maxincrement_get, _mapscript.graticuleObj_maxincrement_set)
-    minsubdivides = property(_mapscript.graticuleObj_minsubdivides_get, _mapscript.graticuleObj_minsubdivides_set)
-    maxsubdivides = property(_mapscript.graticuleObj_maxsubdivides_get, _mapscript.graticuleObj_maxsubdivides_set)
-    bvertical = property(_mapscript.graticuleObj_bvertical_get, _mapscript.graticuleObj_bvertical_set)
-    blabelaxes = property(_mapscript.graticuleObj_blabelaxes_get, _mapscript.graticuleObj_blabelaxes_set)
-    ilabelstate = property(_mapscript.graticuleObj_ilabelstate_get, _mapscript.graticuleObj_ilabelstate_set)
-    ilabeltype = property(_mapscript.graticuleObj_ilabeltype_get, _mapscript.graticuleObj_ilabeltype_set)
-    extent = property(_mapscript.graticuleObj_extent_get, _mapscript.graticuleObj_extent_set)
-    pboundinglines = property(_mapscript.graticuleObj_pboundinglines_get, _mapscript.graticuleObj_pboundinglines_set)
-    pboundingpoints = property(_mapscript.graticuleObj_pboundingpoints_get, _mapscript.graticuleObj_pboundingpoints_set)
-    labelformat = property(_mapscript.graticuleObj_labelformat_get, _mapscript.graticuleObj_labelformat_set)
-    def __init__(self, *args):
-        newobj = _mapscript.new_graticuleObj(*args)
-        self.this = newobj.this
-        self.thisown = 1
-        del newobj.thisown
-    def __del__(self, destroy=_mapscript.delete_graticuleObj):
-        try:
-            if self.thisown: destroy(self)
-        except: pass
-
-class graticuleObjPtr(graticuleObj):
-    def __init__(self, this):
-        self.this = this
-        if not hasattr(self,"thisown"): self.thisown = 0
-        self.__class__ = graticuleObj
-_mapscript.graticuleObj_swigregister(graticuleObjPtr)
-
 class layerObj(object):
     def __repr__(self):
         return "<C layerObj instance at %s>" % (self.this,)
@@ -723,6 +688,8 @@ class layerObj(object):
     tileindex = property(_mapscript.layerObj_tileindex_get, _mapscript.layerObj_tileindex_set)
     units = property(_mapscript.layerObj_units_get, _mapscript.layerObj_units_set)
     connection = property(_mapscript.layerObj_connection_get, _mapscript.layerObj_connection_set)
+    plugin_library = property(_mapscript.layerObj_plugin_library_get, _mapscript.layerObj_plugin_library_set)
+    plugin_library_original = property(_mapscript.layerObj_plugin_library_original_get, _mapscript.layerObj_plugin_library_original_set)
     connectiontype = property(_mapscript.layerObj_connectiontype_get, _mapscript.layerObj_connectiontype_set)
     numitems = property(_mapscript.layerObj_numitems_get)
     bandsitem = property(_mapscript.layerObj_bandsitem_get, _mapscript.layerObj_bandsitem_set)
@@ -750,6 +717,8 @@ class layerObj(object):
     def insertClass(*args): return _mapscript.layerObj_insertClass(*args)
     def removeClass(*args): return _mapscript.layerObj_removeClass(*args)
     def open(*args): return _mapscript.layerObj_open(*args)
+    def whichShapes(*args): return _mapscript.layerObj_whichShapes(*args)
+    def nextShape(*args): return _mapscript.layerObj_nextShape(*args)
     def close(*args): return _mapscript.layerObj_close(*args)
     def getFeature(*args): return _mapscript.layerObj_getFeature(*args)
     def getShape(*args): return _mapscript.layerObj_getShape(*args)
@@ -818,14 +787,12 @@ class mapObj(object):
     imagequality = property(_mapscript.mapObj_imagequality_get, _mapscript.mapObj_imagequality_set)
     extent = property(_mapscript.mapObj_extent_get, _mapscript.mapObj_extent_set)
     cellsize = property(_mapscript.mapObj_cellsize_get, _mapscript.mapObj_cellsize_set)
-    gt = property(_mapscript.mapObj_gt_get, _mapscript.mapObj_gt_set)
     saved_extent = property(_mapscript.mapObj_saved_extent_get, _mapscript.mapObj_saved_extent_set)
     units = property(_mapscript.mapObj_units_get, _mapscript.mapObj_units_set)
     scale = property(_mapscript.mapObj_scale_get, _mapscript.mapObj_scale_set)
     resolution = property(_mapscript.mapObj_resolution_get, _mapscript.mapObj_resolution_set)
     shapepath = property(_mapscript.mapObj_shapepath_get, _mapscript.mapObj_shapepath_set)
     mappath = property(_mapscript.mapObj_mappath_get, _mapscript.mapObj_mappath_set)
-    palette = property(_mapscript.mapObj_palette_get, _mapscript.mapObj_palette_set)
     imagecolor = property(_mapscript.mapObj_imagecolor_get, _mapscript.mapObj_imagecolor_set)
     numoutputformats = property(_mapscript.mapObj_numoutputformats_get)
     outputformatlist = property(_mapscript.mapObj_outputformatlist_get)
@@ -938,31 +905,6 @@ class mapObjPtr(mapObj):
         self.__class__ = mapObj
 _mapscript.mapObj_swigregister(mapObjPtr)
 
-class SVGObj(object):
-    def __repr__(self):
-        return "<C SVGObj instance at %s>" % (self.this,)
-    map = property(_mapscript.SVGObj_map_get, _mapscript.SVGObj_map_set)
-    stream = property(_mapscript.SVGObj_stream_get, _mapscript.SVGObj_stream_set)
-    filename = property(_mapscript.SVGObj_filename_get, _mapscript.SVGObj_filename_set)
-    streamclosed = property(_mapscript.SVGObj_streamclosed_get, _mapscript.SVGObj_streamclosed_set)
-    compressed = property(_mapscript.SVGObj_compressed_get, _mapscript.SVGObj_compressed_set)
-    def __init__(self, *args):
-        newobj = _mapscript.new_SVGObj(*args)
-        self.this = newobj.this
-        self.thisown = 1
-        del newobj.thisown
-    def __del__(self, destroy=_mapscript.delete_SVGObj):
-        try:
-            if self.thisown: destroy(self)
-        except: pass
-
-class SVGObjPtr(SVGObj):
-    def __init__(self, this):
-        self.this = this
-        if not hasattr(self,"thisown"): self.thisown = 0
-        self.__class__ = SVGObj
-_mapscript.SVGObj_swigregister(SVGObjPtr)
-
 class imageObj(object):
     def __repr__(self):
         return "<C imageObj instance at %s>" % (self.this,)
@@ -1047,28 +989,6 @@ class rectObjPtr(rectObj):
         self.__class__ = rectObj
 _mapscript.rectObj_swigregister(rectObjPtr)
 
-class vectorObj(object):
-    def __repr__(self):
-        return "<C vectorObj instance at %s>" % (self.this,)
-    x = property(_mapscript.vectorObj_x_get, _mapscript.vectorObj_x_set)
-    y = property(_mapscript.vectorObj_y_get, _mapscript.vectorObj_y_set)
-    def __init__(self, *args):
-        newobj = _mapscript.new_vectorObj(*args)
-        self.this = newobj.this
-        self.thisown = 1
-        del newobj.thisown
-    def __del__(self, destroy=_mapscript.delete_vectorObj):
-        try:
-            if self.thisown: destroy(self)
-        except: pass
-
-class vectorObjPtr(vectorObj):
-    def __init__(self, this):
-        self.this = this
-        if not hasattr(self,"thisown"): self.thisown = 0
-        self.__class__ = vectorObj
-_mapscript.vectorObj_swigregister(vectorObjPtr)
-
 class pointObj(object):
     def __repr__(self):
         return "<C pointObj instance at %s>" % (self.this,)
@@ -1152,6 +1072,7 @@ class shapeObj(object):
         try:
             if self.thisown: destroy(self)
         except: pass
+    fromWKT = staticmethod(_mapscript.shapeObj_fromWKT)
     def project(*args): return _mapscript.shapeObj_project(*args)
     def get(*args): return _mapscript.shapeObj_get(*args)
     def add(*args): return _mapscript.shapeObj_add(*args)
@@ -1159,6 +1080,7 @@ class shapeObj(object):
     def setBounds(*args): return _mapscript.shapeObj_setBounds(*args)
     def clone(*args): return _mapscript.shapeObj_clone(*args)
     def copy(*args): return _mapscript.shapeObj_copy(*args)
+    def toWKT(*args): return _mapscript.shapeObj_toWKT(*args)
     def buffer(*args): return _mapscript.shapeObj_buffer(*args)
     def convexHull(*args): return _mapscript.shapeObj_convexHull(*args)
     def getValue(*args): return _mapscript.shapeObj_getValue(*args)
@@ -1175,6 +1097,8 @@ class shapeObjPtr(shapeObj):
         self.__class__ = shapeObj
 _mapscript.shapeObj_swigregister(shapeObjPtr)
 
+shapeObj_fromWKT = _mapscript.shapeObj_fromWKT
+
 MS_SHAPEFILE_POINT = _mapscript.MS_SHAPEFILE_POINT
 MS_SHAPEFILE_ARC = _mapscript.MS_SHAPEFILE_ARC
 MS_SHAPEFILE_POLYGON = _mapscript.MS_SHAPEFILE_POLYGON
@@ -1267,29 +1191,6 @@ class shapefileObjPtr(shapefileObj):
         self.__class__ = shapefileObj
 _mapscript.shapefileObj_swigregister(shapefileObjPtr)
 
-class msTiledSHPLayerInfo(object):
-    def __repr__(self):
-        return "<C msTiledSHPLayerInfo instance at %s>" % (self.this,)
-    shpfile = property(_mapscript.msTiledSHPLayerInfo_shpfile_get, _mapscript.msTiledSHPLayerInfo_shpfile_set)
-    tileshpfile = property(_mapscript.msTiledSHPLayerInfo_tileshpfile_get, _mapscript.msTiledSHPLayerInfo_tileshpfile_set)
-    tilelayerindex = property(_mapscript.msTiledSHPLayerInfo_tilelayerindex_get, _mapscript.msTiledSHPLayerInfo_tilelayerindex_set)
-    def __init__(self, *args):
-        newobj = _mapscript.new_msTiledSHPLayerInfo(*args)
-        self.this = newobj.this
-        self.thisown = 1
-        del newobj.thisown
-    def __del__(self, destroy=_mapscript.delete_msTiledSHPLayerInfo):
-        try:
-            if self.thisown: destroy(self)
-        except: pass
-
-class msTiledSHPLayerInfoPtr(msTiledSHPLayerInfo):
-    def __init__(self, this):
-        self.this = this
-        if not hasattr(self,"thisown"): self.thisown = 0
-        self.__class__ = msTiledSHPLayerInfo
-_mapscript.msTiledSHPLayerInfo_swigregister(msTiledSHPLayerInfoPtr)
-
 class projectionObj(object):
     def __repr__(self):
         return "<C projectionObj instance at %s>" % (self.this,)
@@ -1360,7 +1261,7 @@ class symbolObj(object):
     filled = property(_mapscript.symbolObj_filled_get, _mapscript.symbolObj_filled_set)
     stylelength = property(_mapscript.symbolObj_stylelength_get, _mapscript.symbolObj_stylelength_set)
     style = property(_mapscript.symbolObj_style_get, _mapscript.symbolObj_style_set)
-    imagepath = property(_mapscript.symbolObj_imagepath_get, _mapscript.symbolObj_imagepath_set)
+    imagepath = property(_mapscript.symbolObj_imagepath_get)
     transparent = property(_mapscript.symbolObj_transparent_get, _mapscript.symbolObj_transparent_set)
     transparentcolor = property(_mapscript.symbolObj_transparentcolor_get, _mapscript.symbolObj_transparentcolor_set)
     character = property(_mapscript.symbolObj_character_get, _mapscript.symbolObj_character_set)
@@ -1380,6 +1281,7 @@ class symbolObj(object):
         try:
             if self.thisown: destroy(self)
         except: pass
+    def setImagepath(*args): return _mapscript.symbolObj_setImagepath(*args)
     def setPoints(*args): return _mapscript.symbolObj_setPoints(*args)
     def getPoints(*args): return _mapscript.symbolObj_getPoints(*args)
     def setStyle(*args): return _mapscript.symbolObj_setStyle(*args)
@@ -1492,24 +1394,6 @@ class hashTableObjPtr(hashTableObj):
         self.__class__ = hashTableObj
 _mapscript.hashTableObj_swigregister(hashTableObjPtr)
 
-
-msCreateHashTable = _mapscript.msCreateHashTable
-
-initHashTable = _mapscript.initHashTable
-
-msFreeHashTable = _mapscript.msFreeHashTable
-
-msFreeHashItems = _mapscript.msFreeHashItems
-
-msInsertHashTable = _mapscript.msInsertHashTable
-
-msLookupHashTable = _mapscript.msLookupHashTable
-
-msRemoveHashTable = _mapscript.msRemoveHashTable
-
-msFirstKeyFromHashTable = _mapscript.msFirstKeyFromHashTable
-
-msNextKeyFromHashTable = _mapscript.msNextKeyFromHashTable
 MAX_PARAMS = _mapscript.MAX_PARAMS
 MS_GET_REQUEST = _mapscript.MS_GET_REQUEST
 MS_POST_REQUEST = _mapscript.MS_POST_REQUEST
@@ -1542,33 +1426,3 @@ class OWSRequestPtr(OWSRequest):
 _mapscript.OWSRequest_swigregister(OWSRequestPtr)
 
 
-loadParams = _mapscript.loadParams
-
-getword = _mapscript.getword
-
-makeword_skip = _mapscript.makeword_skip
-
-makeword = _mapscript.makeword
-
-fmakeword = _mapscript.fmakeword
-
-x2c = _mapscript.x2c
-
-unescape_url = _mapscript.unescape_url
-
-plustospace = _mapscript.plustospace
-
-rind = _mapscript.rind
-
-_getline = _mapscript._getline
-
-send_fd = _mapscript.send_fd
-
-ind = _mapscript.ind
-
-escape_shell_cmd = _mapscript.escape_shell_cmd
-
-msAllocCgiObj = _mapscript.msAllocCgiObj
-
-msFreeCgiObj = _mapscript.msFreeCgiObj
-
