@@ -11,7 +11,7 @@ mapscriptvars.close
 
 # $CFLAGS works only with 1.8 ??? -> the -Wall argument is not needed !!!
 $CFLAGS = ""
-$CPPFLAGS = " -idirafter $(rubylibdir)/$(arch) " + make_define
+$CPPFLAGS = make_inc + " -idirafter $(rubylibdir)/$(arch) " + make_define
 $LDFLAGS += " -fPIC"
 $LOCAL_LIBS += " -L../.. " + make_libs + " " + make_static_libs
 
@@ -31,7 +31,7 @@ end
 #if cmake
   create_makefile("mapscript")
 	make_file = File.open("Makefile", "a")
-	make_file << "mapscript_wrap.c: mapscript.i\n\tswig -ruby mapscript.i"
+	make_file << "mapscript_wrap.c: ../mapscript.i\n\tswig -ruby -o mapscript_wrap.c ../mapscript.i"
 	make_file.close
 	exit 0
 #end
