@@ -27,6 +27,49 @@
  ******************************************************************************
  *
  * $Log: mapsde.c,v $
+ * Revision 1.101.2.4  2006/03/30 16:16:37  hobu
+ * make sure to return a value in msSDELayerClose
+ *
+ * Revision 1.101.2.3  2006/03/10 18:59:36  hobu
+ * Fix 1699, where point queries in msSDEWhichShape were
+ * causing the SDE C API to complain about creating an invalid
+ * rectangle
+ *
+ * Revision 1.101.2.2  2006/02/24 23:09:39  hobu
+ * backport of fixes for SDE (bug 1682)
+ *
+ * Revision 1.104  2006/02/24 22:41:50  sdlime
+ * Added missing malloc in mapsde in msSDELayerGetItems().
+ *
+ * Revision 1.103  2006/02/24 20:24:23  hobu
+ * make sure we check if layer->iteminfo exists before hammering it and reallocating
+ * just use LayerClose to close layer information rather than the
+ * LayerClose/LayerConnectionClose dichotomy that wasn't working
+ *
+ * Revision 1.102  2006/01/10 19:27:20  hobu
+ * fix the memory allocation bug described in bug 1606
+ *
+ * Revision 1.101  2006/01/10 00:19:49  hobu
+ * Fix up allocation of the ROW_ID columns and how the functions that
+ * call it were using it. (bug 1605)
+ *
+ * Revision 1.100  2005/12/15 15:36:59  frank
+ * avoid comment-in-comment warnings
+ *
+ * Revision 1.99  2005/12/08 19:44:46  hobu
+ * oops.  Move msSDELayerGetRowIDColumn to the right spot for
+ * non-SDE builds
+ *
+ * Revision 1.98  2005/12/08 19:07:36  hobu
+ * switch off SDE raster support and fix up msSDELayerGetRowIDColumn
+ *
+ * Revision 1.97  2005/10/28 01:09:42  jani
+ * MS RFC 3: Layer vtable architecture (bug 1477)
+ *
+ * Revision 1.96  2005/07/07 15:03:33  hobu
+ * put thread locking around static data members
+ * used in the lcache (msSDELCacheAdd)
+ *
  * Revision 1.95  2005/06/14 16:03:34  dan
  * Updated copyright date to 2005
  *
@@ -116,18 +159,24 @@
 */
 
 #include <time.h>
+#include <assert.h>
 
 #include "map.h"
 #include "maperror.h"
 #include "maptime.h"
+#include "mapthread.h"
+
+
 
 #ifdef USE_SDE
 #include <sdetype.h> /* ESRI SDE Client Includes */
 #include <sdeerno.h>
 
+/*
 #ifdef USE_SDERASTER
 #include <sderaster.h>
 #endif
+*/
 
 #define MS_SDE_MAXBLOBSIZE 1024*50 /* 50 kbytes */
 #define MS_SDE_NULLSTRING "<null>"
@@ -161,6 +210,7 @@ typedef struct {
 /*
  * Layer ID caching section.
  */
+ 
 static int lcacheCount = 0;
 static int lcacheMax = 0;
 static layerId *lcache = NULL;
@@ -172,6 +222,8 @@ static layerId *lcache = NULL;
 /*                                                                      */
 /************************************************************************/
 
+
+
 /* -------------------------------------------------------------------- */
 /* msSDECloseConnection                                                 */
 /* -------------------------------------------------------------------- */
@@ -221,6 +273,93 @@ static void sde_error(long error_code, char *routine, char *sde_routine)
   return;
 }
 
+/* -------------------------------------------------------------------- */
+/* msSDELayerGetRowIDColumn                                             */
+/* -------------------------------------------------------------------- */
+/*     A helper function to return unique row ID column for             */ 
+/*     an opened SDE layer                                              */
+/* -------------------------------------------------------------------- */
+char *msSDELayerGetRowIDColumn(layerObj *layer)
+{
+#ifdef USE_SDE
+  long status, column_type; 
+  char* column_name;
+  SE_REGINFO registration;
+
+  msSDELayerInfo *sde=NULL;
+  sde = layer->layerinfo;
+
+
+  column_name = (char*) malloc(SE_MAX_COLUMN_LEN);
+  if(!sde) {
+    msSetError( MS_SDEERR, 
+                "SDE layer has not been opened.", 
+                "msSDELayerGetSpatialColumn()");
+    return(NULL);
+  }   
+  
+  if (sde->state_id == SE_DEFAULT_STATE_ID) {
+    if(layer->debug) 
+      msDebug("msSDELayerGetRowIDColumn(): State ID was SE_DEFAULT_STATE_ID, "
+              "reverting to %s.\n", 
+              MS_SDE_ROW_ID_COLUMN);
+      return(strdup(MS_SDE_ROW_ID_COLUMN));
+  }
+  else 
+  {
+    status = SE_reginfo_create (&registration);
+    if(status != SE_SUCCESS) {
+      sde_error(status, "msSDELayerGetRowIDColumn()", "SE_reginfo_create()");
+      return(NULL);
+    }
+    
+    status = SE_registration_get_info ( sde->connection, 
+                                        sde->table, 
+                                        registration);
+    if(status != SE_SUCCESS) {
+      sde_error(status, 
+                "msSDELayerGetRowIDColumn()", 
+                "SE_registration_get_info()");
+      return(NULL);
+    }
+    
+    status= SE_reginfo_get_rowid_column ( registration, 
+                                          column_name, 
+                                          &column_type);
+    SE_reginfo_free(registration);
+    if(status != SE_SUCCESS) {
+      sde_error(status, 
+                "msSDELayerGetRowIDColumn()", 
+                "SE_reginfo_get_rowid_column()");
+      return(NULL);
+    }
+    if (column_type == SE_REGISTRATION_ROW_ID_COLUMN_TYPE_NONE){
+      if(layer->debug) {
+        msDebug("msSDELayerGetRowIDColumn(): Table was not registered, "
+        "returning %s.\n", 
+        MS_SDE_ROW_ID_COLUMN);
+      }
+      return (MS_SDE_ROW_ID_COLUMN);
+    }
+    
+    if (column_name){
+
+      return (column_name); 
+    }
+    else {
+      free(column_name);
+      return(strdup(MS_SDE_ROW_ID_COLUMN));
+    }
+}
+#else
+  msSetError( MS_MISCERR, 
+              "SDE support is not available.", 
+              "msSDELayerGetRowIDColumn()");
+  return(NULL);
+#endif
+}
+
+
 /* -------------------------------------------------------------------- */
 /* msSDELCacheAdd                                                       */
 /* -------------------------------------------------------------------- */
@@ -232,9 +371,12 @@ long msSDELCacheAdd( layerObj *layer,
                      char *columnName,
                      char *connectionString) 
 {
+  
   layerId *lid = NULL;
   int status = 0;
   
+  msAcquireLock( TLOCK_SDE );
+  
   if (layer->debug){
     msDebug( "%s: Caching id for %s, %s, %s\n", "msSDELCacheAdd()", 
             tableName, columnName, connectionString);
@@ -248,6 +390,7 @@ long msSDELCacheAdd( layerObj *layer,
     lcache = (layerId *)realloc(lcache, sizeof(layerId) * lcacheMax);
     if(lcache == NULL)
     {
+      msReleaseLock( TLOCK_SDE );
       msSetError(MS_MEMERR, NULL, "msSDELCacheAdd()");
       return (MS_FAILURE);
     }
@@ -262,12 +405,15 @@ long msSDELCacheAdd( layerObj *layer,
   status = SE_layerinfo_get_id(layerinfo, &lid->layerId);
   if(status != SE_SUCCESS)
   {
+        msReleaseLock( TLOCK_SDE );
         sde_error(status, "msSDELCacheAdd()", "SE_layerinfo_get_id()");
         return(MS_FAILURE);
   }
   lid->table = strdup(tableName);
   lid->column = strdup(columnName);
   lid->connection = strdup(connectionString);
+  
+  msReleaseLock( TLOCK_SDE );
   return (MS_SUCCESS);
 }
 
@@ -366,13 +512,16 @@ long msSDEGetLayerInfo(layerObj *layer,
 /*     Copies a SDE shape into a MapServer shapeObj                     */
 /* -------------------------------------------------------------------- */
 static int sdeShapeCopy(SE_SHAPE inshp, shapeObj *outshp) {
-	
-  long numparts, numsubparts, numpoints;
-  long *subparts=NULL;
+
   SE_POINT *points=NULL;
   SE_ENVELOPE envelope;
   long type, status;
-
+  long *part_offsets = NULL;
+  long *subpart_offsets = NULL;
+  long num_parts = -1;
+  long num_subparts = -1;
+  long num_points = -1;
+  
   lineObj line={0,NULL};
 
   int i,j,k;
@@ -411,20 +560,16 @@ static int sdeShapeCopy(SE_SHAPE inshp, shapeObj *outshp) {
     return(MS_FAILURE);
   }
 
-  SE_shape_get_num_points(inshp, 0, 0, &numpoints);
-  SE_shape_get_num_parts(inshp, &numparts, &numsubparts);
 
-  if(numsubparts > 0) {
-    subparts = (long *)malloc(numsubparts*sizeof(long));
-    if(!subparts) {
-      msSetError( MS_MEMERR, 
-                  "Unable to allocate parts array.", 
-                  "sdeCopyShape()");
-      return(MS_FAILURE);
-    }
-  }
+  SE_shape_get_num_parts (inshp, &num_parts, &num_subparts);
+  SE_shape_get_num_points (inshp, 0, 0, &num_points); 
+	 
+  part_offsets = (long *) malloc( (num_parts + 1) * sizeof(long));
+  subpart_offsets = (long *) malloc( (num_subparts + 1)	* sizeof(long));
+  part_offsets[num_parts] = num_subparts;
+  subpart_offsets[num_subparts]	= num_points;
 
-  points = (SE_POINT *)malloc(numpoints*sizeof(SE_POINT));
+  points = (SE_POINT *)malloc(num_points*sizeof(SE_POINT));
   if(!points) {
     msSetError( MS_MEMERR, 
                 "Unable to allocate points array.", 
@@ -434,8 +579,8 @@ static int sdeShapeCopy(SE_SHAPE inshp, shapeObj *outshp) {
 
   status = SE_shape_get_all_points( inshp, 
                                     SE_DEFAULT_ROTATION, 
-                                    NULL, 
-                                    subparts, 
+                                    part_offsets, 
+                                    subpart_offsets, 
                                     points, 
                                     NULL, 
                                     NULL);
@@ -445,12 +590,12 @@ static int sdeShapeCopy(SE_SHAPE inshp, shapeObj *outshp) {
   }
 
   k = 0; /* overall point counter */
-  for(i=0; i<numsubparts; i++) {
+  for(i=0; i<num_subparts; i++) {
     
-    if( i == numsubparts-1)
-      line.numpoints = numpoints - subparts[i];
+    if( i == num_subparts-1)
+      line.numpoints = num_points - subpart_offsets[i];
     else
-      line.numpoints = subparts[i+1] - subparts[i];
+      line.numpoints = subpart_offsets[i+1] - subpart_offsets[i];
 
     line.point = (pointObj *)malloc(sizeof(pointObj)*line.numpoints);
     if(!line.point) {
@@ -470,7 +615,8 @@ static int sdeShapeCopy(SE_SHAPE inshp, shapeObj *outshp) {
     free(line.point);
   }
 
-  free(subparts);
+  free(part_offsets);
+  free(subpart_offsets);
   free(points);
 
   /* finally copy the bounding box for the entire shape */
@@ -684,6 +830,7 @@ int msSDELayerOpen(layerObj *layer) {
 #ifdef USE_SDE
   long status=-1;
   char **params=NULL;
+  char **data_params=NULL;
   int numparams=0;
   SE_ERROR error;
   SE_STATEINFO state;
@@ -706,14 +853,13 @@ int msSDELayerOpen(layerObj *layer) {
   }
  
   sde->state_id = SE_BASE_STATE_ID;
+  
   /* initialize the table and spatial column names */
-  sde->table = sde->column = NULL;
-  /* initialize the row_id column */
-  sde->row_id_column = (char*) malloc(SE_MAX_COLUMN_LEN);
+  sde->table = NULL;
+  sde->column = NULL;
 
   /* request a connection and stream from the pool */
   poolinfo = (msSDEConnPoolInfo *)msConnPoolRequest( layer ); 
-
   
   /* If we weren't returned a connection and stream, initialize new ones */
   if (!poolinfo) {
@@ -782,12 +928,12 @@ int msSDELayerOpen(layerObj *layer) {
     msConnPoolRegister(layer, poolinfo, msSDECloseConnection);
     msFreeCharArray(params, numparams); /* done with parameter list */
   }
-  
+
   /* Split the DATA member into its parameters using the comma */
   /* Periods (.) are used to denote table names and schemas in SDE,  */
   /* as are underscores (_). */
-  params = split(layer->data, ',', &numparams);
-  if(!params) {
+  data_params = split(layer->data, ',', &numparams);
+  if(!data_params) {
     msSetError(MS_MEMERR, 
         "Error spliting SDE layer information.", "msSDELayerOpen()");
     return(MS_FAILURE);
@@ -799,29 +945,31 @@ int msSDELayerOpen(layerObj *layer) {
     return(MS_FAILURE);
   }
 
-  sde->table = params[0]; 
-  sde->column = params[1];
+  sde->table = strdup(data_params[0]); 
+  sde->column = strdup(data_params[1]);
+
 
   if (numparams < 3){ 
     /* User didn't specify a version, we won't use one */
     if (layer->debug) {
       msDebug("msSDELayerOpen(): Layer %s did not have a " 
               "specified version.\n", layer->name);
-    }
+    } 
     sde->state_id = SE_DEFAULT_STATE_ID;
   } 
   else {
     if (layer->debug) {
       msDebug("msSDELayerOpen(): Layer %s specified version %s.\n", 
               layer->name, 
-              params[2]);
+              data_params[2]);
     }
     status = SE_versioninfo_create (&(version));
     if(status != SE_SUCCESS) {
       sde_error(status, "msSDELayerOpen()", "SE_versioninfo_create()");
       return(MS_FAILURE);
     }
-    status = SE_version_get_info(poolinfo->connection, params[2], version);
+    status = SE_version_get_info(poolinfo->connection, data_params[2], version);
+    
     if(status != SE_SUCCESS) {
        
       if (status == SE_INVALID_RELEASE) {
@@ -834,7 +982,8 @@ int msSDELayerOpen(layerObj *layer) {
         sde_error(status, "msSDELayerOpen()", "SE_version_get_info()");
         return(MS_FAILURE);
       }
-    } 
+    }
+  
   }
 
   /* Get the STATEID from the given version and set the stream to  */
@@ -864,7 +1013,8 @@ int msSDELayerOpen(layerObj *layer) {
       return(MS_FAILURE);
     }
     SE_stateinfo_free (state); 
-    
+
+        msFreeCharArray(data_params, numparams);  
   } /* if (!(sde->state_id == SE_DEFAULT_STATE_ID)) */
   
   
@@ -877,8 +1027,8 @@ int msSDELayerOpen(layerObj *layer) {
 
   status = msSDEGetLayerInfo( layer,
                               poolinfo->connection,
-                              params[0],
-                              params[1],
+                              sde->table,
+                              sde->column,
                               layer->connection,
                               sde->layerinfo);
 
@@ -951,14 +1101,14 @@ int msSDELayerIsOpen(layerObj *layer) {
 /*     Closes the MapServer layer.  This doesn't necessarily close the  */
 /*     connection to the layer.                                         */
 /* -------------------------------------------------------------------- */
-void msSDELayerClose(layerObj *layer) {
+int  msSDELayerClose(layerObj *layer) {
 #ifdef USE_SDE
 
 
   msSDELayerInfo *sde=NULL;
 
   sde = layer->layerinfo;
-  if (sde == NULL) return;  /* Silently return if layer not opened. */
+  if (sde == NULL) return MS_SUCCESS;  /* Silently return if layer not opened. */
 
   if(layer->debug) 
     msDebug("msSDELayerClose(): Closing layer %s.\n", layer->name);
@@ -974,15 +1124,52 @@ void msSDELayerClose(layerObj *layer) {
   sde->connPoolInfo = NULL;
   if (layer->layerinfo) free(layer->layerinfo);
   layer->layerinfo = NULL;
+  return MS_SUCCESS;
 
 #else
   msSetError( MS_MISCERR, 
               "SDE support is not available.", 
               "msSDELayerClose()");
-  return;
+  return MS_FALSE;
 #endif
 }
 
+
+/* -------------------------------------------------------------------- */
+/* msSDELayerCloseConnection                                            */
+/* -------------------------------------------------------------------- */
+/* Virtual table function                                               */
+/* -------------------------------------------------------------------- */
+/*int msSDELayerCloseConnection(layerObj *layer) 
+{
+	
+
+#ifdef USE_SDE
+
+
+  msSDELayerInfo *sde=NULL;
+
+  sde = layer->layerinfo;
+  if (sde == NULL) return;   Silently return if layer not opened./
+
+  if(layer->debug)
+    msDebug("msSDELayerCloseConnection(): Closing connection for layer %s.\n", layer->name);
+
+  msConnPoolRelease( layer, sde->connPoolInfo );
+  sde->connection = NULL;
+  sde->connPoolInfo = NULL;
+
+#else
+  msSetError( MS_MISCERR,
+              "SDE support is not available.",
+              "msSDELayerClose()");
+  return;
+#endif
+
+    return MS_SUCCESS;
+}
+*/
+
 /* -------------------------------------------------------------------- */
 /* msSDELayerWhichShapes                                                */
 /* -------------------------------------------------------------------- */
@@ -1035,6 +1222,16 @@ int msSDELayerWhichShapes(layerObj *layer, rectObj rect) {
   envelope.maxx = MS_MIN(rect.maxx, envelope.maxx);
   envelope.maxy = MS_MIN(rect.maxy, envelope.maxy);
 
+  if( envelope.minx == envelope.maxx && envelope.miny == envelope.maxy){
+        /* fudge a rectangle so we have a valid one for generate_rectangle */
+        /* FIXME: use the real shape for the query and set the filter_type 
+           to be an appropriate type */
+        envelope.minx = envelope.minx - 0.001;
+        envelope.maxx = envelope.maxx + 0.001;
+        envelope.miny = envelope.miny - 0.001;
+        envelope.maxy = envelope.maxy + 0.001;
+    }
+    
   status = SE_shape_generate_rectangle(&envelope, shape);
   if(status != SE_SUCCESS) {
     sde_error(status, 
@@ -1076,7 +1273,7 @@ int msSDELayerWhichShapes(layerObj *layer, rectObj rect) {
   if(!(layer->filter.string))
     /* set to empty string */
     status = SE_queryinfo_set_where_clause (query_info, 
-                                            (const CHAR * ) strdup(""));
+                                            (const CHAR * ) "");
   else
     /* set to the layer's filter.string */
     status = SE_queryinfo_set_where_clause (query_info, 
@@ -1239,10 +1436,12 @@ int msSDELayerGetItems(layerObj *layer) {
                 "msSDELayerGetItems()");
     return(MS_FAILURE);
   }
-
-  sde->row_id_column = (char*) malloc(SE_MAX_COLUMN_LEN);
-  sde->row_id_column = msSDELayerGetRowIDColumn(layer);
   
+  if (!sde->row_id_column) {
+    sde->row_id_column = (char*) malloc(SE_MAX_COLUMN_LEN);
+  }
+  sde->row_id_column = msSDELayerGetRowIDColumn(layer);
+
   status = SE_table_describe(sde->connection, sde->table, &n, &itemdefs);
   if(status != SE_SUCCESS) {
     sde_error(status, "msSDELayerGetItems()", "SE_table_describe()");
@@ -1253,26 +1452,21 @@ int msSDELayerGetItems(layerObj *layer) {
 
   layer->items = (char **)malloc(layer->numitems*sizeof(char *));
   if(!layer->items) {
-    msSetError( MS_MEMERR, 
-                "Error allocating layer items array.", 
-                "msSDELayerGetItems()");
+    msSetError( MS_MEMERR, "Error allocating layer items array.", "msSDELayerGetItems()");
     return(MS_FAILURE);
   }
 
   for(i=0; i<n; i++) layer->items[i] = strdup(itemdefs[i].column_name);
   layer->items[n] = strdup(sde->row_id_column); /* row id */
 
-  layer->iteminfo = (SE_COLUMN_DEF *) calloc( layer->numitems, 
-                                              sizeof(SE_COLUMN_DEF));
-  if(!layer->iteminfo) {
-    msSetError( MS_MEMERR, 
-                "Error allocating SDE item information.", 
-                "msSDELayerGetItems()");
-    return(MS_FAILURE);
+  if (!layer->iteminfo){
+    layer->iteminfo = (SE_COLUMN_DEF *) calloc( layer->numitems, sizeof(SE_COLUMN_DEF));
+    if(!layer->iteminfo) {
+      msSetError( MS_MEMERR, "Error allocating SDE item information.", "msSDELayerGetItems()");
+      return(MS_FAILURE);
+    }
   }
 
-
-  
   for(i=0; i<layer->numitems; i++) { /* requested columns */
     if(strcmp(layer->items[i],sde->row_id_column) == 0)      
       continue;
@@ -1400,6 +1594,15 @@ int msSDELayerGetShape(layerObj *layer, shapeObj *shape, long record) {
 #endif
 }
 
+/* -------------------------------------------------------------------- */
+/* msSDELayerGetShapeVT                                                 */
+/* -------------------------------------------------------------------- */
+/* Overloaded version for virtual table                                 */
+/* -------------------------------------------------------------------- */
+int msSDELayerGetShapeVT(layerObj *layer, shapeObj *shape, int tile, long record) {
+	return msSDELayerGetShape(layer, shape, record);
+}
+
 /* -------------------------------------------------------------------- */
 /* msSDELayerInitItemInfo                                               */
 /* -------------------------------------------------------------------- */
@@ -1417,10 +1620,9 @@ int msSDELayerInitItemInfo(layerObj *layer)
   msSDELayerInfo *sde=NULL;
 
   sde = layer->layerinfo;
-
-  sde->row_id_column = (char*) malloc(SE_MAX_COLUMN_LEN);
   sde->row_id_column = msSDELayerGetRowIDColumn(layer);
 
+  
   if(!sde) {
     msSetError( MS_SDEERR, 
                 "SDE layer has not been opened.", 
@@ -1435,6 +1637,7 @@ int msSDELayerInitItemInfo(layerObj *layer)
     return(MS_FAILURE);
   }
 
+if (!layer->iteminfo){
   layer->iteminfo = (SE_COLUMN_DEF *) calloc( layer->numitems, 
                                               sizeof(SE_COLUMN_DEF));
   if(!layer->iteminfo) {
@@ -1443,6 +1646,7 @@ int msSDELayerInitItemInfo(layerObj *layer)
                 "msSDELayerInitItemInfo()");
     return(MS_FAILURE);
   }
+}
 
   for(i=0; i<layer->numitems; i++) { /* requested columns */
     if(strcmp(layer->items[i],sde->row_id_column) == 0)      
@@ -1520,95 +1724,16 @@ char *msSDELayerGetSpatialColumn(layerObj *layer)
 #endif
 }
 
-/* -------------------------------------------------------------------- */
-/* msSDELayerGetRowIDColumn                                             */
-/* -------------------------------------------------------------------- */
-/*     A helper function to return unique row ID column for             */ 
-/*     an opened SDE layer                                              */
-/* -------------------------------------------------------------------- */
-char *msSDELayerGetRowIDColumn(layerObj *layer)
-{
-#ifdef USE_SDE
-  long status, column_type; 
-  char* row_id_column;
-  SE_REGINFO registration;
 
-  msSDELayerInfo *sde=NULL;
-  sde = layer->layerinfo;
-  row_id_column = (char*) malloc(SE_MAX_COLUMN_LEN);
-  if(!sde) {
-    msSetError( MS_SDEERR, 
-                "SDE layer has not been opened.", 
-                "msSDELayerGetSpatialColumn()");
-    return(NULL);
-  }   
-  
-  if (sde->state_id == SE_DEFAULT_STATE_ID) {
-    if(layer->debug) 
-      msDebug("msSDELayerGetRowIDColumn(): State ID was SE_DEFAULT_STATE_ID, "
-              "reverting to %s.\n", 
-              MS_SDE_ROW_ID_COLUMN);
-      return(strdup(MS_SDE_ROW_ID_COLUMN));
-  }
-  else 
-  {
-    status = SE_reginfo_create (&registration);
-    if(status != SE_SUCCESS) {
-      sde_error(status, "msSDELayerGetRowIDColumn()", "SE_reginfo_create()");
-      return(NULL);
-    }
-    
-    status = SE_registration_get_info ( sde->connection, 
-                                        strdup(sde->table), 
-                                        registration);
-    if(status != SE_SUCCESS) {
-      sde_error(status, 
-                "msSDELayerGetRowIDColumn()", 
-                "SE_registration_get_info()");
-      return(NULL);
-    }
-    
-    status= SE_reginfo_get_rowid_column ( registration, 
-                                          row_id_column, 
-                                          &column_type);
-    if(status != SE_SUCCESS) {
-      sde_error(status, 
-                "msSDELayerGetRowIDColumn()", 
-                "SE_reginfo_get_rowid_column()");
-      return(NULL);
-    }
-    if (column_type == SE_REGISTRATION_ROW_ID_COLUMN_TYPE_NONE){
-      if(layer->debug) {
-        msDebug("msSDELayerGetRowIDColumn(): Table was not registered, "
-        "returning %s.\n", 
-        MS_SDE_ROW_ID_COLUMN);
-      }
-      SE_reginfo_free(registration);
-      return (MS_SDE_ROW_ID_COLUMN);
-    }
-    
-    SE_reginfo_free(registration);
-    if (row_id_column){
-      return (strdup(row_id_column)); 
-    }
-    else {
-      return(strdup(MS_SDE_ROW_ID_COLUMN));
-    }
-}
-#else
-  msSetError( MS_MISCERR, 
-              "SDE support is not available.", 
-              "msSDELayerGetRowIDColumn()");
-  return(NULL);
-#endif
-}
 
-/* -------- Raster support -------- */
+/* -------- Raster support -------- 
+Turned off for now.
 
 #define MAXCOLORS 256
 
 #ifdef USE_SDERASTER
-void msSDEGetColorMap(mapObj *map, layerObj *layer, gdImagePtr img, SE_RASBANDINFO rasterband, int cmap[]) { /* FIXME: size check for cmap[] ! */
+void msSDEGetColorMap(mapObj *map, layerObj *layer, gdImagePtr img, SE_RASBANDINFO rasterband, int cmap[]) { 
+  // FIXME: size check for cmap[] ! 
   LONG rc, cm_bottom, cm, num_cm_entries;
   SE_COLORMAP_TYPE colormap_type;
   SE_COLORMAP_DATA_TYPE data_type;
@@ -1619,12 +1744,12 @@ void msSDEGetColorMap(mapObj *map, layerObj *layer, gdImagePtr img, SE_RASBANDIN
   
   if (!SE_rasbandinfo_has_colormap(rasterband)) {
     if(layer->debug) msDebug("msSDEGetColorMap(): Generating color map\n");
-    /* TODO: other than 8 bit (from drawTIFF mapraster.c) */
+    // TODO: other than 8 bit (from drawTIFF mapraster.c) 
     for (cm=0; cm<256; cm++) {
-      pixel.red = pixel.green = pixel.blue = pixel.pen = cm; /* offsite would be specified in range 0 to 255 */
+      pixel.red = pixel.green = pixel.blue = pixel.pen = cm;  offsite would be specified in range 0 to 255 
 
-      /* if(!MS_COMPARE_COLORS(pixel, layer->offsite))	     */
-      /* cmap[cm] = msAddColorGD(map,img, 0, (cm>>4)*17, (cm>>4)*17, (cm>>4)*17); // use raster color */
+      // if(!MS_COMPARE_COLORS(pixel, layer->offsite))	     
+      // cmap[cm] = msAddColorGD(map,img, 0, (cm>>4)*17, (cm>>4)*17, (cm>>4)*17); // use raster color 
       cmap[cm] = gdImageColorAllocate(img, (cm>>4)*17, (cm>>4)*17, (cm>>4)*17);
     }
     return;
@@ -1632,7 +1757,7 @@ void msSDEGetColorMap(mapObj *map, layerObj *layer, gdImagePtr img, SE_RASBANDIN
 
   if(layer->debug) msDebug("msSDEGetColorMap(): Fetching color map\n");
   
-  /* Fetch the colormap data from the rasterband_info structure. */
+  // Fetch the colormap data from the rasterband_info structure. 
   rc = SE_rasbandinfo_get_colormap (rasterband,
                                     &colormap_type,
                                     &data_type,
@@ -1684,10 +1809,12 @@ void msSDEGetColorMap(mapObj *map, layerObj *layer, gdImagePtr img, SE_RASBANDIN
   }
 }
 #endif
+*/
+
 /************************************************************************/
 /*                              drawSDE()                              */
 /************************************************************************/
-
+/*
 int drawSDE(mapObj *map, layerObj *layer, gdImagePtr img)
 {
 #ifdef USE_SDERASTER
@@ -1696,7 +1823,7 @@ int drawSDE(mapObj *map, layerObj *layer, gdImagePtr img)
   char **params=NULL;
   int numparams=0;
   
-  int i,j; /* loop counters */
+  int i,j; // loop counters 
   int cmap[MAXCOLORS];  
   gdImagePtr rgbimg;
   int col;
@@ -1738,14 +1865,14 @@ int drawSDE(mapObj *map, layerObj *layer, gdImagePtr img)
 
   tpix = (int*) malloc(sizeof(int));
 
-  /* Allocate an SE_SQL_CONSTRUCT */
+  // Allocate an SE_SQL_CONSTRUCT 
   status = SE_sql_construct_alloc (1, &sqlc);
   if(status != SE_SUCCESS) {
     sde_error(status, "drawSDE()", "SE_sql_construct_alloc");
     return(MS_FAILURE);
   }
 
-  /* Fill in the details of the SQL query */
+  // Fill in the details of the SQL query 
   sqlc->where = (char *)malloc(20);
   sqlc->num_tables = 1;
   strcpy (sqlc->tables[0], sde->table);
@@ -1777,7 +1904,7 @@ int drawSDE(mapObj *map, layerObj *layer, gdImagePtr img)
      msDebug("our bands were specified: red=%d, green=%d, blue=%d, alpha=%d\n", 
               red_band,green_band,blue_band,alpha_band);
   }
-  /* Define the number and names of the table columns to be returned */
+  // Define the number and names of the table columns to be returned 
   
   proc_value = NULL;
   proc_value = msLayerGetProcessingKey(layer,"RASTERCOLUMN");
@@ -1788,33 +1915,33 @@ int drawSDE(mapObj *map, layerObj *layer, gdImagePtr img)
   }
   num_cols = 1;
   attrs = (CHAR **) malloc (num_cols * sizeof(CHAR *));
-  attrs[0] = proc_value;/* sde->column; */
+  attrs[0] = proc_value;// sde->column; 
 
   proc_value = NULL;
   free(proc_value);
   
-  /* Define the stream query */
+  // Define the stream query 
   status = SE_stream_query (sde->stream, num_cols, (const CHAR **) attrs, sqlc);
   if(status != SE_SUCCESS) {
     sde_error(status, "drawSDE()", "SE_stream_query");
     return(MS_FAILURE);
   }
 
-  /* Execute the query. */
+  // Execute the query. 
   status = SE_stream_execute (sde->stream);
   if(status != SE_SUCCESS) {
     sde_error(status, "drawSDE()", "SE_stream_execute");
     return(MS_FAILURE);
   }
 
-  /* Fetch the first row. */
+  // Fetch the first row. 
   status = SE_stream_fetch (sde->stream);
   if(status != SE_SUCCESS) {
     sde_error(status, "drawSDE()", "SE_stream_fetch");
     return(MS_FAILURE);
   }
 
-  /* Obtain the image metadata in the current row. */
+  // Obtain the image metadata in the current row. 
   status = SE_rasterattr_create (&hAttr, FALSE);
   if(status != SE_SUCCESS) {
     sde_error(status, "drawSDE()", "SE_rasterattr_create");
@@ -1834,7 +1961,7 @@ int drawSDE(mapObj *map, layerObj *layer, gdImagePtr img)
   if(layer->debug) msDebug("map->extent minx: %lf miny: %lf maxx: %lf maxy: %lf\n",
     map->extent.minx, map->extent.miny, map->extent.maxx, map->extent.maxy);      
   
-  /* Get raster band metadata */
+  // Get raster band metadata 
 
   status = SE_rasterattr_get_rasterband_info (hAttr, &hRasBnd, 1);
   if(status != SE_SUCCESS) {
@@ -1872,7 +1999,7 @@ int drawSDE(mapObj *map, layerObj *layer, gdImagePtr img)
   for (; level >= 0; level--) {
     if (layer->debug) msDebug("Try pyramid level: %ld \n",level);  
 
-    /* Get the pixel dimensions of the current level. */    
+    // Get the pixel dimensions of the current level.  
     status = SE_rasterattr_get_image_size_by_level(hAttr,
                                                &band_width,
                                                &band_height,
@@ -1895,14 +2022,14 @@ int drawSDE(mapObj *map, layerObj *layer, gdImagePtr img)
     if(layer->debug) msDebug("Raster extent minx: %lf miny: %lf maxx: %lf maxy: %lf offset_x: %lf offset_y: %lf\n",
       env.minx, env.miny, env.maxx, env.maxy, coord_offset_x, coord_offset_y);
   
-    /* calculate the pixel cell size in world coordinates */
+    // calculate the pixel cell size in world coordinates 
     cell_size = (env.maxx - env.minx) / (band_width - 1);
         
-    /* compute the real world coordinate width and height */
+    // compute the real world coordinate width and height 
     coord_tile_width = cell_size * tile_width;
     coord_tile_height = cell_size * tile_height;
   
-    /* compute the tile coordinates */
+    // compute the tile coordinates 
     effminx = MS_MAX(env.minx - coord_offset_x, minx);
     effmaxx = MS_MIN(env.maxx - coord_offset_x, maxx);
     effminy = MS_MAX(env.miny - coord_offset_y, miny);
@@ -1918,16 +2045,16 @@ int drawSDE(mapObj *map, layerObj *layer, gdImagePtr img)
       effminx, effminy, effmaxx, effmaxy);
 
     if (((maxx-minx)/cell_size) >= img->sx && ((maxy-miny)/cell_size) >= img->sy)
-      break; /* use this level         */
+      break; // use this level         
   }
 
-  /* make sure the user enter extent overlaps the image extent */
+  // make sure the user enter extent overlaps the image extent 
   if (minx > env.maxx || maxx < env.minx || miny > env.maxy || maxy < env.miny) {
     sde_error(status, "drawSDE()", "Invalid entry: The coordinate extent enter does not overlap image extent");
     return(MS_FAILURE);
   }
       
-  /* Add constraint & get the tiles. */
+  // Add constraint & get the tiles. 
 
   status = SE_rasconstraint_create (&hConstraint);
   if(status != SE_SUCCESS) {
@@ -1961,13 +2088,13 @@ int drawSDE(mapObj *map, layerObj *layer, gdImagePtr img)
     return(MS_FAILURE);
   }
 
-  /* Setup RGB image */
+  // Setup RGB image 
   rgbimg = gdImageCreateTrueColor((constrmaxx-constrminx+1)*tile_width, (constrmaxy-constrminy+1)*tile_height);
-  /* int backcol = gdImageColorAllocate(rgbimg, map->imagecolor.red, map->imagecolor.green, map->imagecolor.blue); */
+  // int backcol = gdImageColorAllocate(rgbimg, map->imagecolor.red, map->imagecolor.green, map->imagecolor.blue); 
   backcol = gdImageColorAllocate(rgbimg, 0, 0, 0);
   gdImageFilledRectangle(rgbimg, 0, 0, rgbimg->sx, rgbimg->sy, backcol);
   
-  /* set up the color map (for 1-band images) */
+  // set up the color map (for 1-band images) 
   msSDEGetColorMap(map, layer, rgbimg, hRasBnd, cmap);
 
   while (SE_SUCCESS == status)
@@ -1981,12 +2108,12 @@ int drawSDE(mapObj *map, layerObj *layer, gdImagePtr img)
         return(MS_FAILURE);
       }
 
-      /* status = SE_rastileinfo_get_level (hTile, &level);
+      // status = SE_rastileinfo_get_level (hTile, &level);
       if(status != SE_SUCCESS) {
         sde_error(status, "drawSDE()", "SE_rastileinfo_get_level");
         return(MS_FAILURE);
       }
-      if(layer->debug) msDebug("Got band from level %ld\n", level); */
+      if(layer->debug) msDebug("Got band from level %ld\n", level); 
 
       status = SE_rastileinfo_get_rowcol (hTile, &row, &column);
       if(status != SE_SUCCESS) {
@@ -2000,7 +2127,7 @@ int drawSDE(mapObj *map, layerObj *layer, gdImagePtr img)
         return(MS_FAILURE);
       }
 
-      if (length==0) continue; /* holes (partial tiles: length != tile_height*tile_width) */
+      if (length==0) continue; // holes (partial tiles: length != tile_height*tile_width) 
       
       for(i=0; i<tile_height; i++)
         for(j=0; j<tile_width; j++) {
@@ -2034,7 +2161,7 @@ int drawSDE(mapObj *map, layerObj *layer, gdImagePtr img)
     }
   }
 
-  /* Release memory */
+  // Release memory 
   SE_rastileinfo_free(hTile);
   SE_rasconstraint_free(hConstraint);
   SE_rasterattr_free(hAttr);
@@ -2042,25 +2169,85 @@ int drawSDE(mapObj *map, layerObj *layer, gdImagePtr img)
   free (sqlc->where);
   free (&tpix);
   SE_sql_construct_free (sqlc);
-  /* ret = SE_stream_free (sde->stream); */
+  // ret = SE_stream_free (sde->stream); 
 
-  /* backcol = gdImageColorResolve(img, map->imagecolor.red, map->imagecolor.green, map->imagecolor.blue); */
-  /* gdImageFilledRectangle(img, 0, 0, img->sx, img->sy, backcol); */
+  // backcol = gdImageColorResolve(img, map->imagecolor.red, map->imagecolor.green, map->imagecolor.blue); 
+  // gdImageFilledRectangle(img, 0, 0, img->sx, img->sy, backcol); 
   
   gdImageCopyResized(img, rgbimg, 0, 0, 
     (LONG) ((minx - (env.minx-coord_offset_x)-constrminx*coord_tile_width) / cell_size),
     (LONG) (((env.maxy - coord_offset_y) - maxy -constrminy*coord_tile_height) / cell_size),
     img->sx, img->sy,
     (LONG) ((maxx-minx)/cell_size), (LONG) ((maxy-miny)/cell_size));
-    /* int dstX, int dstY, int srcX, int srcY, int destW, int destH, int srcW, int srcH) */
+    // int dstX, int dstY, int srcX, int srcY, int destW, int destH, int srcW, int srcH) 
   gdImageDestroy(rgbimg);
   
-  /* using pooled connections for SDE, closed when map file is closed */
-  /* msSDELayerClose(layer);  */
+  // using pooled connections for SDE, closed when map file is closed 
+  // msSDELayerClose(layer);  
 
   return(0);
+
+} */
+
+/* -------------------------------------------------------------------- */
+/* msSDELayerCreateItems                                                */
+/* -------------------------------------------------------------------- */
+/* Special item allocator for SDE                                       */
+/* -------------------------------------------------------------------- */
+int
+msSDELayerCreateItems(layerObj *layer,
+                      int nt) 
+{
+#ifdef USE_SDE
+    /* should be more than enough space, 
+     * SDE always needs a couple of additional items  
+     */
+
+    layer->items = (char **)calloc(nt+2, sizeof(char *)); 
+    if( ! layer->items) {
+        msSetError(MS_MEMERR, NULL, "msSDELayerCreateItems()");
+        return(MS_FAILURE);
+      }
+    layer->items[0] = msSDELayerGetRowIDColumn(layer); /* row id */
+    layer->items[1] = msSDELayerGetSpatialColumn(layer);
+    layer->numitems = 2;
+    return MS_SUCCESS;
+
 #else
-  msSetError(MS_MISCERR, "SDE support is not available.", "drawSDE()");
-  return(-1);
+  msSetError( MS_MISCERR, 
+              "SDE support is not available.", 
+              "msSDELayerGetRowIDColumn()");
+  return(MS_FAILURE);
 #endif
 }
+
+int
+msSDELayerInitializeVirtualTable(layerObj *layer)
+{
+    assert(layer != NULL);
+    assert(layer->vtable != NULL);
+
+    layer->vtable->LayerInitItemInfo = msSDELayerInitItemInfo;
+    layer->vtable->LayerFreeItemInfo = msSDELayerFreeItemInfo;
+    layer->vtable->LayerOpen = msSDELayerOpen;
+    layer->vtable->LayerIsOpen = msSDELayerIsOpen;
+    layer->vtable->LayerWhichShapes = msSDELayerWhichShapes;
+    layer->vtable->LayerNextShape = msSDELayerNextShape;
+    layer->vtable->LayerGetShape = msSDELayerGetShapeVT;
+    layer->vtable->LayerClose = msSDELayerClose;
+    layer->vtable->LayerGetItems = msSDELayerGetItems;
+    layer->vtable->LayerGetExtent = msSDELayerGetExtent;
+
+    /* layer->vtable->LayerGetAutoStyle, use default */
+    /* layer->vtable->LayerApplyFilterToLayer, use default */
+
+    /* SDE uses pooled connections, close from msCloseConnections */
+    /*layer->vtable->LayerCloseConnection = msSDELayerCloseConnection;*/
+
+    layer->vtable->LayerSetTimeFilter = msLayerMakePlainTimeFilter;
+    layer->vtable->LayerCreateItems = msSDELayerCreateItems;
+    /* layer->vtable->LayerGetNumFeatures, use default */
+
+    return MS_SUCCESS;
+}
+
