@@ -1,12 +1,12 @@
 /******************************************************************************
- * $Id: mapserv.h,v 1.26 2004/10/21 04:30:54 frank Exp $
+ * $Id: mapserv.h,v 1.28 2005/06/14 16:03:34 dan Exp $
  *
  * Project:  MapServer
  * Purpose:  Declarations supporting mapserv.c.
  * Author:   Steve Lime and the MapServer team.
  *
  ******************************************************************************
- * Copyright (c) 1996-2004 Regents of the University of Minnesota.
+ * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -28,6 +28,12 @@
  ******************************************************************************
  *
  * $Log: mapserv.h,v $
+ * Revision 1.28  2005/06/14 16:03:34  dan
+ * Updated copyright date to 2005
+ *
+ * Revision 1.27  2005/02/18 03:06:47  dan
+ * Turned all C++ (//) comments into C comments (bug 1238)
+ *
  * Revision 1.26  2004/10/21 04:30:54  frank
  * Added standardized headers.  Added MS_CVSID().
  *
@@ -73,7 +79,7 @@ static char *modeStrings[26] = {"BROWSE","ZOOMIN","ZOOMOUT","MAP","LEGEND","REFE
 /*
 ** Global variables
 */
-int SearchMap=MS_FALSE; // apply pan/zoom BEFORE doing the query (e.g. query the output image rather than the input image)
+int SearchMap=MS_FALSE; /* apply pan/zoom BEFORE doing the query (e.g. query the output image rather than the input image) */
 
 char *QueryFile=NULL;
 char *QueryLayer=NULL, *SelectLayer=NULL;
