@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapsvg.c,v 1.17.2.2 2005/07/05 14:35:37 assefa Exp $
+ * $Id: mapsvg.c,v 1.21.2.1 2006/01/16 20:41:23 sdlime Exp $
  *
  * Name:     mapsvg.c
  * Project:  MapServer
@@ -31,10 +31,19 @@
  ******************************************************************************
  *
  * $Log: mapsvg.c,v $
- * Revision 1.17.2.2  2005/07/05 14:35:37  assefa
+ * Revision 1.21.2.1  2006/01/16 20:41:23  sdlime
+ * Fixed error with image legends (shifted text) introduced by the 1449 bug fix. (bug 1607)
+ *
+ * Revision 1.21  2005/10/26 17:53:57  frank
+ * Avoid warning about unused function.
+ *
+ * Revision 1.20  2005/09/26 19:47:14  assefa
+ * Correct typo bugs when outputing symbols (Bug 1407)
+ *
+ * Revision 1.19  2005/07/05 14:36:33  assefa
  * Correct a bug : using msFree on a wrong pointer : Bug 1406.
  *
- * Revision 1.17.2.1  2005/06/22 15:04:45  assefa
+ * Revision 1.18  2005/06/22 15:16:25  assefa
  * Correct problems with multipolygons (Bug 1390).
  * Remove unecessary spaces when generating the SVG.
  * Contribution to these 2 issues by Makoto Niwa niwa@orkney.co.jp.
@@ -102,7 +111,7 @@
 #endif
 #include "map.h"
 
-MS_CVSID("$Id: mapsvg.c,v 1.17.2.2 2005/07/05 14:35:37 assefa Exp $")
+MS_CVSID("$Id: mapsvg.c,v 1.21.2.1 2006/01/16 20:41:23 sdlime Exp $")
 
 
 static int msIO_fprintfgz(FILE *fp, int bCompressed, const char *format, ... )
@@ -466,13 +475,14 @@ void msTransformShapeSVG(shapeObj *shape, rectObj extent, double cellsize,
 
     bUseGeoCoord= 0;
     
-    /* if (strcasecmp(msGetOutputFormatOption(image->format, "USE_GEOGRAPHICAL_COORDINATES",""),  */
-    /* "TRUE") == 0) */
-    /* bUseGeoCoord = 1; */
+    /* if (strcasecmp(msGetOutputFormatOption(image->format, 
+                                           "USE_GEOGRAPHICAL_COORDINATES",""),
+                                           "TRUE") == 0) */
+    /* bUseGeoCoord = 1;*/
 
     /* if it is full resolution and we want geographical output */
     /* just return, no transformations needed. */
-    if (bFullRes && bUseGeoCoord)
+     if (bFullRes && bUseGeoCoord)
       return;
 
     /* low resolution and pixel output */
@@ -813,6 +823,8 @@ void msDrawShadeSymbolSVG(symbolSetObj *symbolset, imageObj *image,
 #define Tcl_UniChar int
 #define TCL_UTF_MAX 3
 /*comes from gd source code (ggdft.c).*/
+
+#ifdef notdef /* disabled since not currently used */
 static int gdTcl_UtfToUniChar (char *str, Tcl_UniChar * chPtr)
 {
   int byte;
@@ -908,6 +920,7 @@ static int gdTcl_UtfToUniChar (char *str, Tcl_UniChar * chPtr)
   *chPtr = (Tcl_UniChar) byte;
   return 1;
 }
+#endif /* def notdef */
 
 static void drawSVGText(FILE *fp, int bCompressed, int x, int y, 
                         char *string, double size, 
@@ -1212,7 +1225,7 @@ int msDrawLabelCacheSVG(imageObj *image, mapObj *map)
     if(!cachePtr->text || strlen(cachePtr->text) == 0)
       continue; /* not an error, just don't want to do anything */
 
-    if(msGetLabelSize(cachePtr->text, labelPtr, &r, &(map->fontset), layerPtr->scalefactor) == -1)
+    if(msGetLabelSize(cachePtr->text, labelPtr, &r, &(map->fontset), layerPtr->scalefactor, MS_TRUE) == -1)
       return(-1);
 
     if(labelPtr->autominfeaturesize && ((r.maxx-r.minx) > cachePtr->featuresize))
@@ -1616,14 +1629,14 @@ void msDrawMarkerSymbolSVG(symbolSetObj *symbolset, imageObj *image,
                 k = 0; /* point counter */
                 for(j=0;j < symbol->numpoints;j++) 
                 {
-                    if((symbol->points[j].x < 0) && (symbol->points[j].x < 0)) 
+                    if((symbol->points[j].x < 0) && (symbol->points[j].y < 0)) 
                     { /* new polygon (PENUP) */
                         if(k>2) 
                         {
                             
                             msIO_fprintfgz(image->img.svg->stream, 
                                            image->img.svg->compressed,  
-                                         "<polygon %s %s stroke-width=\"%d\" %s points=\"", 
+                                         "<polygon %s %s stroke-width=\"%d\" points=\"", 
                                          pszFill, pszStroke, width);
                             
                             for (i=0; i<k;i++)
@@ -1650,7 +1663,7 @@ void msDrawMarkerSymbolSVG(symbolSetObj *symbolset, imageObj *image,
 
                 msIO_fprintfgz(image->img.svg->stream, 
                                image->img.svg->compressed,  
-                             "<polygon %s %s stroke-width=\"%d\" %s points=\"", 
+                             "<polygon %s %s stroke-width=\"%d\" points=\"", 
                              pszFill, pszStroke, width);
                             
                 for (i=0; i<k;i++)
@@ -1672,7 +1685,7 @@ void msDrawMarkerSymbolSVG(symbolSetObj *symbolset, imageObj *image,
                 oldpnt.y = MS_NINT(d*symbol->points[0].y + offset_y);
       
                 for(j=1;j < symbol->numpoints;j++) { /* step through the marker s */
-                    if((symbol->points[j].x < 0) && (symbol->points[j].x < 0)) {
+                    if((symbol->points[j].x < 0) && (symbol->points[j].y < 0)) {
                         oldpnt.x = MS_NINT(d*symbol->points[j].x + offset_x);
                         oldpnt.y = MS_NINT(d*symbol->points[j].y + offset_y);
                     } else {
