@@ -27,6 +27,50 @@
  ******************************************************************************
  *
  * $Log: mapsymbol.c,v $
+ * Revision 1.90.2.5  2006/03/30 03:46:30  sdlime
+ * Fixed a few more instances where PIXMAP symbols are created but sizex and sizey are not explicitly set. (bug 1725)
+ *
+ * Revision 1.90.2.4  2006/03/21 06:29:29  sdlime
+ * Reverted to old means of scaling symbols based solely on height. Fixed possiblity of memory leak with symbol rotation. Made rotation and scaling behavior more consistent across all GD rendering functions (point, line, polygon and circle). (bugs 1684 and 1705)
+ *
+ * Revision 1.90.2.3  2006/03/02 01:38:43  jani
+ * Revert 1.90.2.2; (bug fix #1684).
+ * I accidentally committed to 4.8 branch. Sorry about that.
+ *
+ * Revision 1.90.2.2  2006/03/02 01:34:57  jani
+ * When we add a pixmap symbol, be sure to initialize sizex,sizey fields of
+ * symbol. These fields are used later by mapgd.c:msDrawMarkerSymbolGD
+ * to calculate scaling factor, if they are uninitialized (0.0),
+ * the scaling factor will be 1.
+ *
+ * This fixes bug #1684
+ *
+ * * msAddImageSymbol: Initialize sizex, sizey
+ *
+ * Revision 1.90.2.1  2006/01/16 20:01:56  sean
+ * close handle when saving symbolset, pointed out by Albert Rovira
+ *
+ * Revision 1.90  2006/01/09 18:04:19  frank
+ * fix for gd calls when different heaps in use - win32 (bug 1513)
+ *
+ * Revision 1.89  2006/01/03 03:19:05  sdlime
+ * Rotation for pixmap symbols is *very* close but transparency is not handled correctly for 8-bit images. I think this is a bug in GD but I can't be sure yet. Rotated images look crappy anyway so this is just a start.
+ *
+ * Revision 1.88  2005/12/15 05:50:30  sdlime
+ * Symbol writer ignores type SIMPLE. It doesn't anymore...
+ *
+ * Revision 1.87  2005/11/17 06:13:23  sdlime
+ * Fixed symbol set copying so that the image cache is not copied and the destination symbol set cache is initialized properly. (bug 1521)
+ *
+ * Revision 1.86  2005/11/05 05:34:41  sdlime
+ * Removed misplaced closing of msyyin in loadSymbol(). (bug 178)
+ *
+ * Revision 1.85  2005/10/14 05:04:12  sdlime
+ * Added msRotateSymbol(), changed freeSymbol to a public function called msFreeSymbol() in mapsymbol.c. Added to map.h as well.
+ *
+ * Revision 1.84  2005/09/21 04:23:27  sdlime
+ * Updated msLoadImageSymbol() to clean-up any previously allocated values for imagepath and img. (bug 1472)
+ *
  * Revision 1.83  2005/06/14 16:03:35  dan
  * Updated copyright date to 2005
  *
@@ -74,7 +118,7 @@
 #include "mapcopy.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mapsymbol.c,v 1.83 2005/06/14 16:03:35 dan Exp $")
+MS_CVSID("$Id: mapsymbol.c,v 1.90.2.5 2006/03/30 03:46:30 sdlime Exp $")
 
 extern int msyylex(void); /* lexer globals */
 extern void msyyrestart(FILE *);
@@ -185,7 +229,7 @@ void initSymbol(symbolObj *s)
   s->linejoinmaxsize = 3;
 }
 
-static void freeSymbol(symbolObj *s) {
+void msFreeSymbol(symbolObj *s) {
   if(!s) return;
   if(s->name) free(s->name);
   if(s->img) gdImageDestroy(s->img);
@@ -249,7 +293,6 @@ int loadSymbol(symbolObj *s, char *symbolpath)
     case(IMAGE):
       if(msyylex() != MS_STRING) { /* get image location from next token */
 	msSetError(MS_TYPEERR, "Parsing error near (%s):(line %d)", "loadSymbol()", msyylineno);
-	fclose(msyyin);
 	return(-1);
       }
       
@@ -257,48 +300,47 @@ int loadSymbol(symbolObj *s, char *symbolpath)
       {
 	msSetError(MS_IOERR, "Parsing error near (%s):(line %d)", "loadSymbol()", 
                    msyytext, msyylineno);
-	fclose(msyyin);
 	return(-1);
       }
       
       /* Set imagepath */
       s->imagepath = strdup(msyytext);
 
-          
       fread(bytes,8,1,stream); /* read some bytes to try and identify the file */
       rewind(stream); /* reset the image for the readers */
       if (memcmp(bytes,"GIF8",4)==0) 
       {
 #ifdef USE_GD_GIF
         ctx = msNewGDFileCtx(stream);
-	    s->img = gdImageCreateFromGifCtx(ctx);
+	s->img = gdImageCreateFromGifCtx(ctx);
         ctx->gd_free(ctx);
 #else
-	    msSetError(MS_MISCERR, "Unable to load GIF symbol.", "loadSymbol()");
-	    fclose(stream);
-	    return(-1);
+	msSetError(MS_MISCERR, "Unable to load GIF symbol.", "loadSymbol()");
+	fclose(stream);
+	return(-1);
 #endif
       } 
       else if (memcmp(bytes,PNGsig,8)==0) 
       {
 #ifdef USE_GD_PNG
         ctx = msNewGDFileCtx(stream);
-	    s->img = gdImageCreateFromPngCtx(ctx);
+	s->img = gdImageCreateFromPngCtx(ctx);
         ctx->gd_free(ctx);
 #else
-	    msSetError(MS_MISCERR, "Unable to load PNG symbol.", "loadSymbol()");
-	    fclose(stream);
-	    return(-1);
+	msSetError(MS_MISCERR, "Unable to load PNG symbol.", "loadSymbol()");
+	fclose(stream);
+	return(-1);
 #endif
       }
 
       fclose(stream);
       
       if(s->img == NULL) {
-	msSetError(MS_GDERR, NULL, "loadSymbol()");
-	fclose(msyyin);
+	msSetError(MS_GDERR, NULL, "loadSymbol()");	
 	return(-1);
       }
+      s->sizex = s->img->sx;
+      s->sizey = s->img->sy;            
       break;
     case(LINECAP):
       if((s->linecap = getSymbol(4,MS_CJC_BUTT, MS_CJC_ROUND, MS_CJC_SQUARE, MS_CJC_TRIANGLE)) == -1)
@@ -329,9 +371,7 @@ int loadSymbol(symbolObj *s, char *symbolpath)
 	  s->numpoints++;
 	  break;
 	default:
-	  msSetError(MS_TYPEERR, "Parsing error near (%s):(line %d)", "loadSymbol()",  
-                     msyytext, msyylineno); 	  
-	  fclose(msyyin);
+	  msSetError(MS_TYPEERR, "Parsing error near (%s):(line %d)", "loadSymbol()", msyytext, msyylineno);
 	  return(-1);
 	}
 
@@ -359,9 +399,7 @@ int loadSymbol(symbolObj *s, char *symbolpath)
 	  s->stylelength++;
 	  break;
 	default:
-	  msSetError(MS_TYPEERR, "Parsing error near (%s):(line %d)", "loadSymbol()", 
-                     msyytext, msyylineno); 	  
-	  fclose(msyyin);
+	  msSetError(MS_TYPEERR, "Parsing error near (%s):(line %d)", "loadSymbol()", msyytext, msyylineno);
 	  return(-1);
 	}
 	if(done == MS_TRUE)
@@ -384,9 +422,7 @@ int loadSymbol(symbolObj *s, char *symbolpath)
 	s->type = MS_SYMBOL_TRUETYPE;
       break;
     default:
-      msSetError(MS_IDENTERR, "Parsing error near (%s):(line %d)", "loadSymbol()",
-                 msyytext, msyylineno);
-      fclose(msyyin);
+      msSetError(MS_IDENTERR, "Parsing error near (%s):(line %d)", "loadSymbol()", msyytext, msyylineno);
       return(-1);
     } /* end switch */
   } /* end for */
@@ -424,19 +460,17 @@ void writeSymbol(symbolObj *s, FILE *stream)
     fprintf(stream, "    LINEJOIN %s\n", msCapsJoinsCorners[s->linejoin]);
     fprintf(stream, "    LINEJOINMAXSIZE %g\n", s->linejoinmaxsize);
     break;
-  case(MS_SYMBOL_SIMPLE):
-    break;
-  case(MS_SYMBOL_ELLIPSE):
-    /* default = MS_SYMBOL_VECTOR */
   default:
     if(s->type == MS_SYMBOL_ELLIPSE)
       fprintf(stream, "    TYPE ELLIPSE\n");
-    else
+    else if(s->type == MS_SYMBOL_VECTOR)
       fprintf(stream, "    TYPE VECTOR\n");
+    else
+      fprintf(stream, "    TYPE SIMPLE\n");
     
     if(s->filled == MS_TRUE) fprintf(stream, "    FILLED TRUE\n");
     
-    /* POINT */
+    /* POINTS */
     if(s->numpoints != 0) {
       fprintf(stream, "    POINTS\n");
       for(i=0; i<s->numpoints; i++) {
@@ -444,6 +478,7 @@ void writeSymbol(symbolObj *s, FILE *stream)
       }
       fprintf(stream, "    END\n");
     }
+
     /* STYLE */
     if(s->stylelength != 0) {
       fprintf(stream, "    STYLE\n     ");
@@ -506,19 +541,25 @@ int msAddImageSymbol(symbolSetObj *symbolset, char *filename)
   rewind(stream); /* reset the image for the readers */
   if (memcmp(bytes,"GIF8",4)==0) {
 #ifdef USE_GD_GIF
-    symbolset->symbol[i].img = gdImageCreateFromGif(stream);
+      gdIOCtx *ctx;
+      ctx = msNewGDFileCtx(stream);
+      symbolset->symbol[i].img = gdImageCreateFromGifCtx(ctx);
+      ctx->gd_free(ctx);
 #else
-    msSetError(MS_MISCERR, "Unable to load GIF symbol.", "msAddImageSymbol()");
-    fclose(stream);
-    return(-1);
+      msSetError(MS_MISCERR, "Unable to load GIF symbol.", "msAddImageSymbol()");
+      fclose(stream);
+      return(-1);
 #endif
   } else if (memcmp(bytes,PNGsig,8)==0) {
 #ifdef USE_GD_PNG
-    symbolset->symbol[i].img = gdImageCreateFromPng(stream);
+      gdIOCtx *ctx;
+      ctx = msNewGDFileCtx(stream);
+      symbolset->symbol[i].img = gdImageCreateFromPngCtx(ctx);
+      ctx->gd_free(ctx);
 #else
-    msSetError(MS_MISCERR, "Unable to load PNG symbol.", "msAddImageSymbol()");
-    fclose(stream);
-    return(-1);
+      msSetError(MS_MISCERR, "Unable to load PNG symbol.", "msAddImageSymbol()");
+      fclose(stream);
+      return(-1);
 #endif
   }
 
@@ -531,6 +572,8 @@ int msAddImageSymbol(symbolSetObj *symbolset, char *filename)
 
   symbolset->symbol[i].name = strdup(filename);
   symbolset->symbol[i].type = MS_SYMBOL_PIXMAP;
+  symbolset->symbol[i].sizex = symbolset->symbol[i].img->sx;
+  symbolset->symbol[i].sizey = symbolset->symbol[i].img->sy;
   symbolset->numsymbols++;
 
   return(i);
@@ -542,7 +585,7 @@ void msFreeSymbolSet(symbolSetObj *symbolset)
 
   freeImageCache(symbolset->imagecache);
   for(i=1; i<symbolset->numsymbols; i++)
-    freeSymbol(&(symbolset->symbol[i]));
+    msFreeSymbol(&(symbolset->symbol[i]));
 
   /* no need to deal with fontset, it's a pointer */
 }
@@ -631,9 +674,7 @@ int loadSymbolSet(symbolSetObj *symbolset, mapObj *map)
     case(SYMBOLSET):
       break;
     default:
-      msSetError(MS_IDENTERR, "Parsing error near (%s):(line %d)",
-                              "loadSymbolSet()",
-                 msyytext, msyylineno);      
+      msSetError(MS_IDENTERR, "Parsing error near (%s):(line %d)", "loadSymbolSet()", msyytext, msyylineno);
       status = -1;
     } /* end switch */
 
@@ -807,7 +848,17 @@ int msSaveSymbolSet(symbolSetObj *symbolset, const char *filename) {
         return MS_FAILURE;
     }
     stream = fopen(filename, "w");
-    retval = msSaveSymbolSetStream(symbolset, stream);
+    if (stream)
+    {
+        retval = msSaveSymbolSetStream(symbolset, stream);
+        fclose(stream);
+    }
+    else 
+    {
+        msSetError(MS_SYMERR, "Could not write to %s", "msSaveSymbolSet()",
+                   filename);
+        retval = MS_FAILURE;
+    }
     return retval;
 }
 
@@ -826,6 +877,11 @@ int msLoadImageSymbol(symbolObj *symbol, const char *filename) {
           return MS_FAILURE;
     }
 
+    if(symbol->imagepath) free(symbol->imagepath);
+    symbol->imagepath = strdup(filename);
+
+    if(symbol->img) gdImageDestroy(symbol->img);
+
     fread(bytes,8,1,stream); /* read some bytes to try and identify the file */
     rewind(stream); /* reset the image for the readers */
     if (memcmp(bytes,"GIF8",4)==0) 
@@ -863,6 +919,8 @@ int msLoadImageSymbol(symbolObj *symbol, const char *filename) {
     }
 
     symbol->type = MS_SYMBOL_PIXMAP;
+    symbol->sizex = symbol->img->sx;
+    symbol->sizey = symbol->img->sy;
 
     return MS_SUCCESS;
 }
@@ -915,25 +973,20 @@ int msCopySymbol(symbolObj *dst, symbolObj *src, mapObj *map) {
 
   /* Copy the actual symbol imagery */
   if (src->img) {
-     if (dst->img) {
-       gdFree(dst->img);
-     }
-     
-     if (gdImageTrueColor(src->img)) {
-        dst->img = gdImageCreateTrueColor(gdImageSX(src->img),
-                                          gdImageSY(src->img));
-        gdImageColorTransparent(dst->img, gdImageGetTransparent(src->img));
-        gdImageAlphaBlending(dst->img, 0);
-        gdImageCopy(dst->img, src->img, 0, 0, 0, 0,
-                    gdImageSX(src->img), gdImageSY(src->img));
-     }
-     else {
-        dst->img = gdImageCreate(gdImageSX(src->img), gdImageSY(src->img));
-        gdImageAlphaBlending(dst->img, 0);
-        gdImageColorTransparent(dst->img, gdImageGetTransparent(src->img));
-        gdImageCopy(dst->img, src->img, 0, 0, 0, 0,
-                    gdImageSX(src->img), gdImageSY(src->img));
-     }
+    if (dst->img)
+      gdFree(dst->img);
+    
+    if (gdImageTrueColor(src->img)) {
+      dst->img = gdImageCreateTrueColor(gdImageSX(src->img), gdImageSY(src->img));
+      gdImageColorTransparent(dst->img, gdImageGetTransparent(src->img));
+      gdImageAlphaBlending(dst->img, 0);
+      gdImageCopy(dst->img, src->img, 0, 0, 0, 0, gdImageSX(src->img), gdImageSY(src->img));
+    } else {
+      dst->img = gdImageCreate(gdImageSX(src->img), gdImageSY(src->img));
+      gdImageAlphaBlending(dst->img, 0);
+      gdImageColorTransparent(dst->img, gdImageGetTransparent(src->img));
+      gdImageCopy(dst->img, src->img, 0, 0, 0, 0, gdImageSX(src->img), gdImageSY(src->img));
+    }
   }
 
   return(MS_SUCCESS);
@@ -965,13 +1018,16 @@ int msCopySymbolSet(symbolSetObj *dst, symbolSetObj *src, mapObj *map)
     }
   }
 
-  MS_COPYSTELEM(imagecachesize);
+  /* MS_COPYSTELEM(imagecachesize); */
   
   /* I have a feeling that the code below is not quite right - Sean */
   /*copyProperty(&(dst->imagecache), &(src->imagecache),
                sizeof(struct imageCacheObj));
    */
 
+  dst->imagecachesize = 0; /* since we are not copying the cache set the cache  to NUNLL and the size to 0 (bug 1521) */
+  dst->imagecache = NULL;
+
   return(MS_SUCCESS);
 }
 
@@ -1090,7 +1146,148 @@ int msSymbolSetImageGD(symbolObj *symbol, imageObj *image)
     gdImageCopy(symbol->img, image->img.gd, 0, 0, 0, 0,
                 image->width, image->height);
 
+    symbol->sizex = symbol->img->sx;
+    symbol->sizey = symbol->img->sy;
+
     return MS_SUCCESS;
 }
 
+static void get_bbox(pointObj *poiList, int numpoints, double *minx, double *miny, double *maxx, double *maxy) {
+  int j;
 
+  *minx = *maxx = poiList[0].x;
+  *miny = *maxy = poiList[0].y;
+  for(j=1; j<numpoints; j++) {
+    if ((poiList[j].x==-99.0) || (poiList[j].y==-99.0)) continue;
+    *minx = MS_MIN(*minx, poiList[j].x);
+    *maxx = MS_MAX(*maxx, poiList[j].x);
+    *miny = MS_MIN(*miny, poiList[j].y);
+    *maxy = MS_MAX(*maxy, poiList[j].y);
+  }
+
+  return;
+}
+
+/*
+** msRotateSymbol - Clockwise rotation of a symbol definition. Contributed
+** by MapMedia, with clean up by SDL. Currently only type VECTOR and PIXMAP 
+** symbols are handled.
+*/
+symbolObj *msRotateSymbol(symbolObj *symbol, double angle)
+{
+  double angle_rad=0.0;
+  double cos_a, sin_a;
+  double minx=0.0, miny=0.0, maxx=0.0, maxy=0.0;
+  symbolObj *newSymbol = NULL;
+
+  if(!(symbol->type == MS_SYMBOL_VECTOR || symbol->type == MS_SYMBOL_PIXMAP)) {
+    msSetError(MS_SYMERR, "Only symbols with type VECTOR or PIXMAP may be rotated.", "msRotateSymbol()");
+    return NULL;
+  }
+
+  newSymbol = (symbolObj *) malloc(sizeof(symbolObj));
+  msCopySymbol(newSymbol, symbol, NULL); /* TODO: do we really want to do this for all symbol types? */
+
+  angle_rad = (MS_DEG_TO_RAD*angle);
+
+  switch(symbol->type) {
+  case(MS_SYMBOL_VECTOR):
+    {
+      double dp_x, dp_y, xcor, ycor;
+      double TOL=0.00000000001;
+      int i;
+
+      sin_a = sin(angle_rad);
+      cos_a = cos(angle_rad);
+
+      dp_x = symbol->sizex * .5; /* get the shift vector at 0,0 */
+      dp_y = symbol->sizey * .5;
+
+      /* center at 0,0 and rotate; then move back */
+      for( i=0;i < symbol->numpoints;i++) {
+	/* don't rotate PENUP commands (TODO: should use a constant here) */
+	if ((symbol->points[i].x == -99.0) || (symbol->points[i].x == -99.0) ) {
+	  newSymbol->points[i].x = -99.0;
+	  newSymbol->points[i].y = -99.0;
+	  continue;
+	}
+            
+	newSymbol->points[i].x = dp_x + ((symbol->points[i].x-dp_x)*cos_a - (symbol->points[i].y-dp_y)*sin_a);
+	newSymbol->points[i].y = dp_y + ((symbol->points[i].x-dp_x)*sin_a + (symbol->points[i].y-dp_y)*cos_a);
+      }
+	
+      /* get the new bbox of the symbol, because we need it to get the new dimensions of the new symbol */
+      get_bbox(newSymbol->points, newSymbol->numpoints, &minx, &miny, &maxx, &maxy);
+      if ( (fabs(minx)>TOL) || (fabs(miny)>TOL) ) {
+	xcor = minx*-1.0; /* symbols always start at 0,0 so get the shift vector */
+	ycor = miny*-1.0;
+	for( i=0;i < newSymbol->numpoints;i++) {
+	  if ((newSymbol->points[i].x == -99.0) || (newSymbol->points[i].x == -99.0))
+	    continue;
+	  newSymbol->points[i].x = newSymbol->points[i].x + xcor;
+	  newSymbol->points[i].y = newSymbol->points[i].y + ycor;
+	}
+	
+	/* update the bbox to get the final dimension values for the symbol */
+	get_bbox(newSymbol->points, newSymbol->numpoints, &minx, &miny, &maxx, &maxy);
+      }
+
+      newSymbol->sizex = maxx;
+      newSymbol->sizey = maxy;
+      break;
+    }
+  case(MS_SYMBOL_PIXMAP):
+    {
+      double cos_a, sin_a;
+
+      double x1 = 0.0, y1 = 0.0; /* destination rectangle */
+      double x2 = 0.0, y2 = 0.0;
+      double x3 = 0.0, y3 = 0.0;
+      double x4 = 0.0, y4 = 0.0;
+
+      long minx, miny, maxx, maxy;
+
+      int width=0, height=0;
+      /* int color; */
+
+      sin_a = sin(angle_rad);
+      cos_a = cos(angle_rad);
+
+      /* compute distination rectangle (x1,y1 is known) */
+      x1 = 0 ; y1 = 0 ;
+      x2 = symbol->img->sy * sin_a;
+      y2 = -symbol->img->sy * cos_a;
+      x3 = (symbol->img->sx * cos_a) + (symbol->img->sy * sin_a);
+      y3 = (symbol->img->sx * sin_a) - (symbol->img->sy * cos_a);
+      x4 = (symbol->img->sx * cos_a);
+      y4 = (symbol->img->sx * sin_a);
+		
+      minx = (long) MS_MIN(x1,MS_MIN(x2,MS_MIN(x3,x4)));
+      miny = (long) MS_MIN(y1,MS_MIN(y2,MS_MIN(y3,y4)));
+      maxx = (long) MS_MAX(x1,MS_MAX(x2,MS_MAX(x3,x4)));
+      maxy = (long) MS_MAX(y1,MS_MAX(y2,MS_MAX(y3,y4)));
+
+      width = (int)ceil(maxx-minx);
+      height = (int)ceil(maxy-miny);
+
+      /* create the new image based on the computed width/height */
+      gdFree(newSymbol->img);
+      if (gdImageTrueColor(symbol->img)) {
+	newSymbol->img = gdImageCreateTrueColor(width, height);
+	gdImageAlphaBlending(newSymbol->img, 0);
+      } else {
+	newSymbol->img = gdImageCreate(width, height);	
+      }
+
+      newSymbol->sizex = width;
+      newSymbol->sizey = height;
+
+      gdImageCopyRotated (newSymbol->img, symbol->img, width*0.5, height*0.5, 0, 0, gdImageSX(symbol->img), gdImageSY(symbol->img), angle);
+      break;
+    }
+  default:
+    break; /* should never get here */   
+  } /* end symbol type switch */
+
+  return newSymbol;
+}
