@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapthread.c,v 1.12 2005/06/14 16:03:35 dan Exp $
+ * $Id: mapthread.c,v 1.15 2005/12/30 23:05:10 jani Exp $
  *
  * Project:  UMN MapServer
  * Purpose:  Support code for abstracting thread issues.
@@ -28,6 +28,16 @@
  ******************************************************************************
  *
  * $Log: mapthread.c,v $
+ * Revision 1.15  2005/12/30 23:05:10  jani
+ * mapthread.c: Added missing lock_name "LAYER_VTABLE", this should go to
+ * 4.8.0 release.
+ *
+ * Revision 1.14  2005/07/15 13:37:07  frank
+ * Added OWS lock name.
+ *
+ * Revision 1.13  2005/07/07 14:38:00  frank
+ * added TLOCK_ORACLE and TLOCK_SDE definitions
+ *
  * Revision 1.12  2005/06/14 16:03:35  dan
  * Updated copyright date to 2005
  *
@@ -176,13 +186,14 @@ files instead.
 #include "map.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mapthread.c,v 1.12 2005/06/14 16:03:35 dan Exp $")
+MS_CVSID("$Id: mapthread.c,v 1.15 2005/12/30 23:05:10 jani Exp $")
 
 #if defined(USE_THREAD)
 static int thread_debug = 0;
 
 static char *lock_names[] = 
-{ NULL, "PARSER", "GDAL", "ERROROBJ", "PROJ", "TTF", "POOL", NULL };
+{ NULL, "PARSER", "GDAL", "ERROROBJ", "PROJ", "TTF", "POOL", "SDE", 
+  "ORACLE", "OWS", "LAYER_VTABLE", NULL };
 #endif
 
 /************************************************************************/
