@@ -27,6 +27,18 @@
  ******************************************************************************
  *
  * $Log: mapthread.h,v $
+ * Revision 1.11  2005/10/29 02:03:43  jani
+ * MS RFC 8: Pluggable External Feature Layer Providers (bug 1477).
+ *
+ * Revision 1.10  2005/07/15 13:36:46  frank
+ * Added comment on lock_names[] in mapthread.c.
+ *
+ * Revision 1.9  2005/07/14 21:20:44  hobu
+ * added TLOCK_OWS definition
+ *
+ * Revision 1.8  2005/07/07 14:38:00  frank
+ * added TLOCK_ORACLE and TLOCK_SDE definitions
+ *
  * Revision 1.7  2005/06/14 16:03:35  dan
  * Updated copyright date to 2005
  *
@@ -60,12 +72,21 @@ void msReleaseLock(int);
 #define msReleaseLock(x)
 #endif
 
+/*
+** lock ids - note there is a corresponding lock_names[] array in 
+** mapthread.c that needs to be extended when new ids are added.
+*/
+
 #define TLOCK_PARSER	1
 #define TLOCK_GDAL	2
 #define TLOCK_ERROROBJ  3
 #define TLOCK_PROJ      4
 #define TLOCK_TTF       5
 #define TLOCK_POOL      6
+#define TLOCK_SDE       7
+#define TLOCK_ORACLE    8
+#define TLOCK_OWS       9
+#define TLOCK_LAYER_VTABLE 10
 
 #define TLOCK_STATIC_MAX 20
 #define TLOCK_MAX       100
