@@ -27,6 +27,9 @@
  **********************************************************************
  *
  * $Log: mapwfslayer.c,v $
+ * Revision 1.36  2005/10/28 01:09:42  jani
+ * MS RFC 3: Layer vtable architecture (bug 1477)
+ *
  * Revision 1.35  2005/05/12 18:41:54  assefa
  * Use %f instead of %lf for bbox output : Bug 1239.
  *
@@ -145,12 +148,13 @@
 #include "mapows.h"
 
 #include <time.h>
+#include <assert.h>
 
 #if defined(_WIN32) && !defined(__CYGWIN__)
 #include <process.h>
 #endif
 
-MS_CVSID("$Id: mapwfslayer.c,v 1.35 2005/05/12 18:41:54 assefa Exp $")
+MS_CVSID("$Id: mapwfslayer.c,v 1.36 2005/10/28 01:09:42 jani Exp $")
 
 #define WFS_V_0_0_14  14
 #define WFS_V_1_0_0  100
@@ -918,6 +922,17 @@ int msWFSLayerOpen(layerObj *lp,
 #endif /* USE_WFS_LYR */
 }
 
+/**********************************************************************
+ *                          msWFSLayerOpenVT()
+ *
+ * Overloaded version of msWFSLayerOpen for virtual table architecture
+ **********************************************************************/
+int 
+msWFSLayerOpenVT(layerObj *lp)
+{
+	return msWFSLayerOpen(lp, NULL, NULL);
+}
+
 /**********************************************************************
  *                          msWFSLayerIsOpen()
  *
@@ -1220,3 +1235,33 @@ char *msWFSExecuteGetFeature(layerObj *lp)
 
 }
 
+int
+msWFSLayerInitializeVirtualTable(layerObj *layer)
+{
+    assert(layer != NULL);
+    assert(layer->vtable != NULL);
+
+    layer->vtable->LayerInitItemInfo = msWFSLayerInitItemInfo;
+    layer->vtable->LayerFreeItemInfo = msOGRLayerFreeItemInfo; /* yes, OGR */
+    layer->vtable->LayerOpen = msWFSLayerOpenVT;
+    layer->vtable->LayerIsOpen = msWFSLayerIsOpen;
+    layer->vtable->LayerWhichShapes = msWFSLayerWhichShapes;
+    layer->vtable->LayerNextShape = msOGRLayerNextShape; /* yes, OGR */
+    layer->vtable->LayerGetShape = msOGRLayerGetShape; /* yes, OGR */
+
+    layer->vtable->LayerClose = msWFSLayerClose;
+    layer->vtable->LayerGetItems = msWFSLayerGetItems;
+    layer->vtable->LayerGetExtent = msOGRLayerGetExtent; /* yes, OGR */
+    /* layer->vtable->LayerGetAutoStyle, use default */
+
+    /* layer->vtable->LayerApplyFilterToLayer, use default */
+
+    /* layer->vtable->LayerCloseConnection, use default */
+
+    layer->vtable->LayerSetTimeFilter = msLayerMakePlainTimeFilter;
+    /* layer->vtable->LayerCreateItems, use default */
+    /* layer->vtable->LayerGetNumFeatures, use default */
+
+    return MS_SUCCESS;
+}
+
