@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mapwmslayer.c,v 1.76 2005/05/13 17:23:34 dan Exp $
+ * $Id: mapwmslayer.c,v 1.78 2005/10/26 17:51:28 frank Exp $
  *
  * Project:  MapServer
  * Purpose:  Implementation of WMS CONNECTIONTYPE - client to WMS servers
@@ -27,6 +27,12 @@
  * DEALINGS IN THE SOFTWARE.
  **********************************************************************
  * $Log: mapwmslayer.c,v $
+ * Revision 1.78  2005/10/26 17:51:28  frank
+ * avoid warnings about unused functions
+ *
+ * Revision 1.77  2005/09/26 18:55:20  assefa
+ * use transparency set at the layer level on wms client layers  (Bug 1458).
+ *
  * Revision 1.76  2005/05/13 17:23:34  dan
  * First pass at properly handling XML exceptions from CONNECTIONTYPE WMS
  * layers. Still needs some work. (bug 1246)
@@ -155,6 +161,8 @@ void msFreeWmsParamsObj(wmsParamsObj *wmsparams)
  *                          msSetWMSParamString()
  *
  **********************************************************************/
+
+#ifdef USE_WMS_LYR
 static int msSetWMSParamString(wmsParamsObj *psWMSParams, 
                                const char *name, const char * value,
                                int urlencode) 
@@ -174,11 +182,14 @@ static int msSetWMSParamString(wmsParamsObj *psWMSParams,
 
     return MS_SUCCESS;
 }
+#endif /* def USE_WMS_LYR */
 
 /**********************************************************************
  *                          msSetWMSParamInt()
  *
  **********************************************************************/
+
+#ifdef USE_WMS_LYR
 static int msSetWMSParamInt(wmsParamsObj *wmsparams, 
                             const char *name, int value) 
 {
@@ -190,6 +201,7 @@ static int msSetWMSParamInt(wmsParamsObj *wmsparams,
 
     return MS_SUCCESS;
 }
+#endif /* def USE_WMS_LYR */
 
 /**********************************************************************
  *                          msBuildWMSParamsUrl()
@@ -1213,7 +1225,7 @@ int msDrawWMSLayerLow(int nLayerId, httpRequestObj *pasReqInfo,
             fclose(fp);
 
             /* GDAL should be called to reproject automatically. */
-            if (msDrawRasterLayerLow(map, lp, img) != 0)
+            if (msDrawLayer(map, lp, img) != 0)
                 status = MS_FAILURE;
 
             if (!lp->debug)
